<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Jan 19, 2010
 */

/**
 * Base test for API test cases.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_API_Framework_TestCase extends IWorx_Framework_TestCase
{
    protected $_xml = null;

    /**
     * Setup function.
     */
    public function setup()
    {
        exec('rm -rf /tmp/wsdl*');

        exec('/bin/hostname', $result);
        $this->_initApiObj($result[ 0 ]);
    }

    /**
     * Init api object.
     *
     * @param string $hostname
     */
    protected function _initApiObj($hostname)
    {
        $url        = "http://{$hostname}:2080/nodeworx/xmlrpc";
        $this->_xml = new Zend_XmlRpc_Client($url);
        $this->_xml->getHttpClient()->setConfig(array( 'timeout' => 100 ));
    }

    /**
     * Gets the installed api key.
     *
     * @return string
     */
    protected function _getGoodApiKey()
    {
        $sql = 'SELECT apikey FROM nodeworx WHERE nodeworx_id = ' . q(1);
        $key = IWorxDbUtil::db_getOne(IW::DB(), $sql);
        if ($key === '') {
            $NW    = self::getMasterNodeWorx();
            $Reply = IW::FC()->routeFromPHP(
                'Ctrl_Nodeworx_Apikey',
                'generate',
                array(),
                $NW
            );
            $this->assertReplySuccessful($Reply);
            $sql = 'SELECT apikey FROM nodeworx WHERE nodeworx_id = ' . q(1);
            $key = IWorxDbUtil::db_getOne(IW::DB(), $sql);
        }
        $this->assertTrue(trim($key) !== '');
        $apikey = API_Key::db2pretty(1, $key);
        return $apikey;
    }

    /**
     * Api call func.
     *
     * @return array result of the api call
     */
    protected function _apiCall()
    {
        $args   = func_get_args();
        $method = array_shift($args);
        $result = $this->_xml->call("iworx.{$method}", $args[0]);

        $this->_assertResponseLooksValid($result);
        return $result;
    }

    /**
     * Assert response looks valid.
     *
     * @param mixed $resp
     */
    protected function _assertResponseLooksValid($resp)
    {
        $this->assertTrue(is_array($resp), 'response not an array');
        $this->assertArrayHasKey('status', $resp);
        $this->assertArrayHasKey('payload', $resp);
        $this->assertArrayHasKey('reply_code', $resp);
    }

    /**
     * Assert response is "win" bundle.
     *
     * @param mixed $resp
     */
    protected function _assertResponseIsWinBundle($resp)
    {
        $this->assertEquals(API::SUCCESS, $resp['status']);
        $this->assertEquals('You Win', $resp['payload']);
        $this->assertEquals(Reply::OK_WITH_DATA, $resp['reply_code']);
    }

    /**
     * Assert response is "fail" bundle.
     *
     * @param mixed $resp
     */
    protected function _assertResponseIsFailBundle($resp)
    {
        $this->assertEquals(API::GENERAL_FAILURE, $resp['status']);
        $this->assertEquals('You Fail', $resp['payload']);
        $this->assertEquals(Reply::ERROR, $resp['reply_code']);
    }
}
