<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jon Daniel <jdaniel@interworx.com>
 * @date    Jan 28, 2010
 */

/**
 * Cluster Framework.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Cluster_Framework_TestCase extends IWorx_Framework_TestCase
{

    /**
     * Default setup.
     *
     * Will skip the test if cluster-tests is turned off in the config file.
     */
    public function setUp()
    {
        parent::setUp();

        if (TestConfig::get('cluster-tests') != '1') {
            $this->markTestSkipped('cluster tests skipped');
        }
    }

    /**
     * Add all nodes in conf to the cluster.
     */
    protected function _addAllNodesToCluster()
    {
        $NodeWorx = self::getMasterNodeWorx();
        $Nmanager = new NodeManager();

        foreach (TestConfig::get('cluster-nodes') as $data) {
            $Node = new Node();
            $Node->setHostname($data['hostname']);
            $Node->setAPIKey($data['apikey']);

            $where  = 'hostname = ' . q($data['hostname']);
            if (!$Node->exists($where)) {
                $Nmanager->addNode($Node, $block = true);
            }//end if
        }//end foreach
    }

    /**
     * Remove all nodes in conf from the cluster.
     */
    protected function _removeAllNodesFromCluster()
    {
        $NodeWorx = self::getMasterNodeWorx();
        foreach (TestConfig::get('cluster-nodes') as $data) {
            $Node   = new Node();
            $where  = 'hostname = ' . q($data['hostname']);

            if ($Node->exists($where)) {
                $Node->get($where);

                $nmanager = new NodeManager();
                $nmanager->deleteNode($Node, $force = true);
            }//end if
        }//end foreach
    }
}
