<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 23, 2009
 */

use \PHPUnit\Framework;

/**
 * Helper functions as a base class for unit tests.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Framework_TestCase extends Framework\TestCase
{
    /**
     * @var string
     */
    const NW_USER = NW_USER;
    /**
     * @var string
     */
    const NW_PASS = NW_PASS;

    /**
     * @var string
     */
    const SECONDARY_NICK = SECONDARY_NICK;
    /**
     * @var string
     */
    const SECONDARY_EMAIL = SECONDARY_EMAIL;
    /**
     * @var string
     */
    const SECONDARY_PASS = SECONDARY_PASS;

    /**
     * @var string
     */
    const RSL_NICKNAME = RSL_NICKNAME;
    /**
     * @var string
     */
    const RSL_EMAIL = RSL_EMAIL;
    /**
     * @var string
     */
    const RSL_PASSWORD = RSL_PASSWORD;
    /**
     * @var string
     */
    const RSL_STATUS = User::ACTIVE;

    /**
     * @var string
     */
    const SW_DOMAIN = SW_DOMAIN;
    /**
     * @var string
     */
    const SW_UNIQNAME = SW_UNIQNAME;
    /**
     * @var string
     */
    const SW_EMAIL = SW_EMAIL;
    /**
     * @var string
     */
    const SW_PASS = SW_PASS;
    /**
     * @var string
     */
    const SW_NICK = SW_NICK;

    /**
     * @var string
     */
    const RSL_SW_DOMAIN = RSL_SW_DOMAIN;
    /**
     * @var string
     */
    const RSL_SW_UNIQNAME = RSL_SW_UNIQNAME;

    /**
     * @var string
     */
    const SW_SECONDARY_DOMAIN = SW_SECONDARY_DOMAIN;

    /**
     * @var string
     */
    const SW_SECONDARY_NICK = SW_SECONDARY_NICK;
    /**
     * @var string
     */
    const SW_SECONDARY_EMAIL = SW_SECONDARY_EMAIL;
    /**
     * @var string
     */
    const SW_SECONDARY_PASS = SW_SECONDARY_PASS;

    /**
     * @var string
     */
    const SW_POINTER_DOMAIN = SW_POINTER_DOMAIN;

    const SW_PACKAGE = 'Package By Test';

    const FORM_ERROR_HEADER =
    '##LG_INPUT_FORM_DID_NOT_VALIDATE##. ##LG_INPUT_FORM_SEE_DETAILS_BELOW##.';

    const QUICK_TEST_TIME_LIMIT = 15;

    public const DUMMY_IP = '192.168.0.5';

    /**
     * NodeWorx object for the master user.
     *
     * @var NodeWorx
     */
    private static $_NodeWorx;

    /**
     * SiteWorx object for the self::SW_DOMAIN domain.
     *
     * @var SiteWorx
     */
    private $_SiteWorx;

    /**
     * @var integer.
     */
    protected $_maxRunningTime = 0;

    private $_log_size = 0;
    protected $_expected_stacktraces = 0;

    /**
     * Run Test.
     *
     * @return mixed
     */
    public function runTest()
    {
        PHP_Timer::start();
        $testResult = parent::runTest();
        $time = PHP_Timer::stop();

        if ($this->_maxRunningTime != 0 &&
        $time > $this->_maxRunningTime) {
            $this->fail(sprintf(
                'expected running time: <= %s but was: %s',
                $this->_maxRunningTime,
                $time
            ));
        }

        return $testResult;
    }

    /**
     * Sets up the world database.
     *
     * @return void
     * @throws \IWorx_Exception
     * @throws \Process\Exception\CmdValidationException
     * @throws \Process\Exception\LogicException
     * @throws \Process\Exception\RuntimeException
     */
    public static function setUpWorldDatabase()
    {
        $dsn      = Ini::get(Ini::IWORX, 'dsn');
        $dsn_data = DB::parseDSN($dsn);
        $db_user  = $dsn_data['username'];
        $db_sock  = $dsn_data['socket'];
        $db_pass  = $dsn_data['password'];
        $file     = '/usr/local/interworx/TESTS/unit/IWorx/Search/world.sql';

        $defaults = IWorxDbUtil::getTempDefaultsFile(
            $db_pass,
            IWorxDbUtil::USE_CLIENT
        );

        // do the import
        $cmd = Ini::get(Ini::BIN, 'mysql') .
            " --defaults-file={$defaults}" .
            " --user=$db_user" .
            " --socket=$db_sock" .
            " iworx < $file";

        $flags  = IWorxExec::ALLOW_REDIR_IN | IWorxExec::STDERR_2_STDOUT;
        $retval = -1;
        $result = [];
        IWorxExec::exec($cmd, $result, $retval, $flags);

        // Cleanup
        IWorxFileSys::rm($defaults);
    }

    /**
     * Runs before each test.
     *
     * This is duplicated in IWorx_SeleniumTestCase
     */
    public function assertPreConditions()
    {
        $this->_log_size = filesize('/usr/local/interworx/var/log/iworx.log');
        $this->_expected_stacktraces = 0;
    }

    /**
     * Runs after each test.
     *
     * This is duplicated in IWorx_SeleniumTestCase
     */
    public function assertPostConditions()
    {
        $file = '/usr/local/interworx/var/log/iworx.log';

        $stack_count = 0;

        $fp = fopen($file, 'r');
        fseek($fp, $this->_log_size);

        $content = '';
        while (!feof($fp)) {
            $content .= fread($fp, 8096);
        }

        fclose($fp);

        $lines = explode("\n", $content);

        foreach ($lines as $line) {
            if (strpos($line, '[EMERG]') !== false &&
            strpos($line, 'ERROR:') !== false) {
                $stack_count++;
            }
        }

        $this->assertEquals(
            $this->_expected_stacktraces,
            $stack_count,
            'Found stacktraces in log output'
        );
    }

    /**
     * Fail if running as part of Jenkins, otherwise just skip.
     *
     * @param string $msg
     */
    public function failOnJenkins($msg)
    {
        $hostname = Ini::get(Ini::IWORX, 'hostname');
        if (strpos($hostname, 'unit.interworx.com') !== false) {
            // Fail if it's running on the actual Jenkins test boxes.
            $this->fail($msg);
        } else {
            // Skip if it's running on a personal test box, or whatever.
            $this->markTestSkipped($msg);
        }
    }

    /**
     * Set Max Running Time.
     *
     * @param integer $maxRunningTime
     */
    public function setMaxRunningTime($maxRunningTime)
    {
        if (is_integer($maxRunningTime) &&
        $maxRunningTime >= 0) {
            $this->_maxRunningTime = $maxRunningTime;
        } else {
            throw PHPUnit_Util_InvalidArgumentHelper::factory(1, 'positive integer');
        }
    }

    /**
     * Check if we're running the new install test suite or not.
     *
     * This is not ideal. But it works, and it provides an easy place for improvement
     * later, if we think of a better way.
     *
     * @return boolean
     */
    public function isNewInstallTest()
    {
        $hostname = gethostname();

        $regex = '/^in[0-9]+\.unit\.interworx\.com$/';
        if (preg_match($regex, $hostname) === 1) {
            return true;
        }

        return false;
    }

    /**
     * Get max running time.
     *
     * @return integer
     */
    public function getMaxRunningTime()
    {
        return $this->_maxRunningTime;
    }

    /**
     * Turns octal chmod modes into string representation.
     *
     * @param integer $perms
     * @return string
     */
    public function getFilePermsAsString($perms)
    {
        $p = $perms;
        if (($p & 0xC000) == 0xC000) {
            // Socket
            $info = 's';
        } elseif (($p & 0xA000) == 0xA000) {
            // Symbolic Link
            $info = 'l';
        } elseif (($p & 0x8000) == 0x8000) {
            // Regular
            $info = '-';
        } elseif (($p & 0x6000) == 0x6000) {
            // Block special
            $info = 'b';
        } elseif (($p & 0x4000) == 0x4000) {
            // Directory
            $info = 'd';
        } elseif (($p & 0x2000) == 0x2000) {
            // Character special
            $info = 'c';
        } elseif (($p & 0x1000) == 0x1000) {
            // FIFO pipe
            $info = 'p';
        } else {
            // Unknown
            $info = 'u';
        }//end if

        // Owner
        $info .= (($p & 0x0100) ? 'r' : '-');
        $info .= (($p & 0x0080) ? 'w' : '-');
        $info .=
        (($p & 0x0040) ? (($p & 0x0800) ? 's' : 'x' ) : (($p & 0x0800) ? 'S' : '-'));

        // Group
        $info .= (($p & 0x0020) ? 'r' : '-');
        $info .= (($p & 0x0010) ? 'w' : '-');
        $info .=
        (($p & 0x0008) ? (($p & 0x0400) ? 's' : 'x' ) : (($p & 0x0400) ? 'S' : '-'));

        // World
        $info .= (($p & 0x0004) ? 'r' : '-');
        $info .= (($p & 0x0002) ? 'w' : '-');
        $info .=
        (($p & 0x0001) ? (($p & 0x0200) ? 't' : 'x' ) : (($p & 0x0200) ? 'T' : '-'));

        return $info;
    }

    /**
     * Gets the master NodeWorx object.
     *
     * @return NodeWorx
     */
    public static function getMasterNodeWorx()
    {
        if (is_null(self::$_NodeWorx) === true) {
            self::$_NodeWorx = new NodeWorx(self::NW_USER);
        }

        return self::$_NodeWorx;
    }

    /**
     * Gets the Reseller NodeWorx object.
     *
     * DO NOT CACHE
     *
     * @param string $email
     * @return NodeWorx
     */
    public static function getResellerNodeWorx($email = self::RSL_EMAIL)
    {
        return new NodeWorx($email);
    }

    /**
     * Gets the account id of a SW domain.
     *
     * @param string $domain
     * @return integer
     */
    public static function getSWAccountId($domain = self::SW_DOMAIN)
    {
        $SiteWorx = new SiteWorx($domain);
        return intval($SiteWorx->getId());
    }

    /**
     * Gets the SW User Id for the info provided.
     *
     * @param string $domain
     * @param string $email
     * @return integer|false
     */
    public static function getSWUserId(
        $domain = self::SW_DOMAIN,
        $email = self::SW_EMAIL
    ) {
        if (!$email) {
            return false;
        }

        $SiteWorx = new SiteWorx($domain);
        $user     = $SiteWorx->getUser($email);

        if (empty($user)) {
            return false;
        } else {
            $user_ID = $user->getId();
            return intval($user_ID);
        }
    }

    /**
     * Gets the NW User Id for the info provided.
     *
     * @param string $email
     * @return integer|false
     */
    public static function getNWUserId($email = self::SW_SECONDARY_EMAIL)
    {
        if (!$email) {
            return false;
        }

        $NodeWorx = self::getMasterNodeWorx();
        $user     = $NodeWorx->getUserInfoByEmail($email);

        if (empty($user)) {
            return false;
        } else {
            $user_ID = $user->getId();
            return intval($user_ID);
        }
    }

    /**
     * Gets the ID of the reseller account.
     *
     * @return integer
     */
    public static function getResellerId()
    {
        if (self::getResellerAccountExists() === true) {
            return self::getResellerNodeWorx()->getId();
        } else {
            return -1;
        }
    }

    /**
     * Checks if a reseller exists.
     *
     * @param string $email
     * @return boolean
     */
    public static function getResellerAccountExists($email = self::RSL_EMAIL)
    {
        return self::getResellerNodeWorx($email)->getId() !== -1;
    }

    /**
     * Adds the reseller account if it doesn't exist.
     *
     * @return boolean
     */
    public function addResellerIfNecessary()
    {
        if (self::getResellerAccountExists() === true) {
            return true;
        }

        $input = self::_getResellerInput();

        $this->_addResellerFromInput($input);

        $NW    = self::getResellerNodeWorx();
        $input = array( 'ns1' => 'ns1.interworx.com',
                    'ns2' => 'ns2.interworx.com' );
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Dns',
            'defaultServers',
            $input,
            $NW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Adds a SiteWorx account for the reseller.
     */
    public function addResellerSwAccountIfNecessary()
    {
        if (self::getSWAccountExists(self::RSL_SW_DOMAIN) === true) {
            return;
        }
        $this->addResellerIfNecessary();
        $NW    = self::getResellerNodeWorx();
        $input = $this->_getResellerSWAccountInput();

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'add',
            $input,
            $NW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Deletes the SiteWorx account for the reseller.
     *
     * @param string $email
     */
    public function deleteResellerSwAccountIfNecessary($email = self::RSL_EMAIL)
    {
        $NW = self::getResellerNodeWorx($email);
        if (false !== $NW->getSiteWorx(self::RSL_SW_DOMAIN)) {
            $input = array( 'domain' => self::RSL_SW_DOMAIN );
            $Reply = IW::FC()->routeFromPHP(
                'Ctrl_Nodeworx_Siteworx',
                'delete',
                $input,
                $NW
            );
            $this->assertReplySuccessful($Reply);
        }
    }

    /**
     * Checks if a domain exists.
     *
     * @param string $domain
     * @return boolean
     */
    public static function getSWAccountExists($domain = self::SW_DOMAIN)
    {
        return self::getSWAccountId($domain) !== -1;
    }

    /**
     * Adds the default SW account if it doesn't exist.
     *
     * @return boolean
     */
    public static function addSWAccountIfNecessary()
    {
        if (self::getSWAccountExists() === false) {
            $result = self::addSWAccount();

            if ($result !== true) {
                echo "\nUnable to add SiteWorx Account for test\n";
                return false;
            } else {
                return true;
            }
        }

        return true;
    }

    /**
     * Add secondary domain if it doesn't exist.
     */
    public static function addSecondaryDomainIfNecessary()
    {
        self::addSWAccountIfNecessary();
        if (Domain::exists(self::SW_SECONDARY_DOMAIN) === false) {
            $SW    = new SiteWorx(self::getSWAccountId());
            $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );
            $Reply = IW::FC()->routeFromPHP('Ctrl_Siteworx_DomainsSlave', 'add', $input, $SW);
        }
    }

    /**
     * Delete secondary domain if it does exist.
     */
    public static function deleteSecondaryDomainIfNecessary()
    {
        if (Domain::exists(self::SW_SECONDARY_DOMAIN)) {
            $SW    = new SiteWorx(self::getSWAccountId());
            $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );
            $Reply = IW::FC()->routeFromPHP('Ctrl_Siteworx_DomainsSlave', 'delete', $input, $SW);
        }
    }

    /**
     * Gets the array of input needed for adding the account.
     *
     * @param boolean $dedicated_ip
     * @return array
     */
    protected static function _getSWAccountInput($dedicated_ip = false)
    {
        $input                  = [];
        $input['master_domain'] = self::SW_DOMAIN;

        if ($dedicated_ip === true) {
            $input['master_domain_ipv4'] = self::_getFirstAvailableDedicatedIP();
        } else {
            $input['master_domain_ipv4'] = self::_getFirstSharedIP();
        }

        self::assertTrue(!empty($input['master_domain_ipv4']), 'IP input is empty');

        $input['uniqname']         = self::SW_UNIQNAME;
        $input['nickname']         = self::SW_NICK;
        $input['email']            = self::SW_EMAIL;
        $input['password']         = self::SW_PASS;
        $input['confirm_password'] = $input['password'];

        $input['additional_ipv4'] = self::_getAdditionalIPv4();
        $input['additional_ipv6'] = self::_getAdditionalIPv6();

        $options = Option::getOptionList();

        foreach ($options as $opt) {
            if ($opt['displayGroup'] !== '1') {
                continue;
            }
            $input_name = substr($opt['name'], 3);
            $input[ $input_name ] = Ini::get(Ini::CONSTANTS, 'unlimited');
        }

        foreach ($options as $opt) {
            $input_name = substr($opt['name'], 3);
            if ($opt['displayGroup'] !== '2') {
                continue;
            }
            if ($input_name != 'OPT_SUPHP' && $input_name != 'OPT_SSI') {
                $input[ $input_name ] = 1;
            }
        }

        $input = array_merge($input, self::_getPackageOptionsInput());

        return $input;
    }

    /**
     * Gets the array of input needed for creating a sw account under a reseller.
     *
     * @return array
     */
    protected function _getResellerSWAccountInput()
    {
        $input = self::_getSWAccountInput();
        $input['master_domain']      = self::RSL_SW_DOMAIN;
        $input['uniqname']           = self::RSL_SW_UNIQNAME;
        $input['master_domain_ipv4'] = $this->_getFirstSharedResellerIP();
        $input['additional_ipv4']    = [];
        $input['additional_ipv6']    = [];
        return $input;
    }

    /**
     * Gets an array of siteworx package options w/ default values.
     *
     * @return array
     */
    protected static function _getPackageOptionsInput()
    {
        $options = Option::getOptionList();
        $input   = [];

        foreach ($options as $opt) {
            if ($opt['displayGroup'] !== '1') {
                continue;
            }
            $input_name = substr($opt['name'], 3);
            $input[ $input_name ] = Ini::get(Ini::CONSTANTS, 'unlimited');
        }

        foreach ($options as $opt) {
            $input_name = substr($opt['name'], 3);
            if ($opt['displayGroup'] === '1') {
                continue;
            }
            if ($input_name != 'OPT_SUPHP' && $input_name != 'OPT_SSI') {
                $input[ $input_name ] = 1;
            } else {
                $input[ $input_name ] = 0;
            }
        }

        return $input;
    }

    /**
     * Gets valid reseller input data.
     *
     * @return array
     */
    protected function _getResellerInput()
    {
        $DatabaseServer = MySQL::getLocalServer();

        $input = array( 'nickname'         => self::RSL_NICKNAME,
                    'email'            => self::RSL_EMAIL,
                    'password'         => self::RSL_PASSWORD,
                    'confirm_password' => self::RSL_PASSWORD,
                    'status'           => self::RSL_STATUS,
                    'database_servers' => array( $DatabaseServer->getNickname() ),
                    'ips'              =>
                    array( $this->_getFirstSharedResellerIP() ) );
        $input = array_merge($input, $this->_getResellerPackageOptionsInput());
        return $input;
    }

    /**
     * Gets reseller package option input.
     *
     * @return array
     */
    protected static function _getResellerPackageOptionsInput()
    {
        $options = Reseller_Option::getOptionList();

        foreach ($options as $opt) {
            if ($opt['displayGroup'] !== '1') {
                continue;
            }
            $input_name = substr($opt['name'], 3);
            $input[ $input_name ] = Ini::get(Ini::CONSTANTS, 'unlimited');
        }

        foreach ($options as $opt) {
            $input_name = substr($opt['name'], 3);
            if ($opt['displayGroup'] === '1') {
                continue;
            }
            $input[ $input_name ] = 1;
        }

        return $input;
    }

    /**
     * Gets additional IPv4 input.
     *
     * @return array
     */
    protected static function _getAdditionalIPv4()
    {
        $search_input = array( 'version'      => IWorx_IP::VERSION_4,
                           'is_static'    => '0',
                           'is_available' => '1' );
        $ips = IWorx_IP::factoryFromSearchEngine($search_input);

        $i         = 0;
        $output    = [];
        $count_ips = count($ips);

        while ($i < 3 && $i < $count_ips) {
            $output[] = $ips[ $i ]->getIP();
            ++$i;
        }

        return $output;
    }

    /**
     * Gets additional IPv6 input.
     *
     * @return array
     */
    protected static function _getAdditionalIPv6()
    {
        $search_input = array( 'version'      => IWorx_IP::VERSION_6,
                           'is_static'    => '0',
                           'is_available' => '1' );
        $ips = IWorx_IP::factoryFromSearchEngine($search_input);

        $i         = 0;
        $output    = [];
        $count_ips = count($ips);

        while ($i < 3 && $i < $count_ips) {
            $output[] = $ips[ $i ]->getIP();
            ++$i;
        }

        return $output;
    }

    /**
     * Gets the first shared ip.
     *
     * @return string
     */
    protected static function _getFirstSharedIP()
    {
        $ips = IWorx_IP::getAvailableIPv4(IWorx_IP::TYPE_SHARED);
        $ip  = array_shift($ips);
        self::assertTrue(is_object($ip), 'Could not find a shared IP address :(');
        return $ip->getIP();
    }

    /**
     * Gets the first available dedicated ip.
     *
     * @return string
     */
    protected static function _getFirstAvailableDedicatedIP()
    {
        $ips = IWorx_IP::getAvailableIPv4(IWorx_IP::TYPE_STATIC);

        self::assertGreaterThan(
            0,
            count($ips),
            'Booooo.  No free dedicated IPs, I need one.'
        );

        $ip = array_shift($ips);
        return $ip->getIP();
    }

    /**
     * @param string $ip
     */
    protected function _addDummyIpIfNecessary($ip = self::DUMMY_IP)
    {
        if (!in_array($ip, $this->_getAdditionalIPv4())) {
            $this->_addIp($ip);
        }
    }

    /**
     * Add IP address
     *
     * @param string $ip
     * @param string $type
     */
    protected static function _addIp($ip, $type = 'shared')
    {
        $NW = self::getMasterNodeWorx();

        $input = array('ip_start' => $ip,
                       'ip_end' => $ip,
                       'netmask' => '255.255.255.0',
                       'device' => 'eth0',
                       'type' => $type,
                       'reseller' => '0',
                       'vip' => '0'
        );

        $Reply = IW::FC()->routeFromPHP(
            Ctrl_Nodeworx_Ip::class,
            'add',
            $input,
            $NW
        );

        if (!$Reply->wasSuccessful()) {
            self::fail('COULD NOT ADD IP: ' . $Reply->getErrorsAsString());
        }
    }

    /**
     * Gets the first shared reseller ip.
     *
     * @return array
     */
    protected function _getFirstSharedResellerIP()
    {
        $ips = IWorx_IP::getAvailableResellerIPv4(IWorx_IP::TYPE_SHARED);

        $this->assertGreaterThan(
            0,
            count($ips),
            'When adding reseller, no shared reseller ips'
        );
        $ip = array_shift($ips);
        return $ip->getIP();
    }

    /**
     * Adds a SiteWorx account for testing.
     *
     * @param boolean $use_dedicated_ip
     * @param array   $input_overrides
     * @return boolean
     */
    public static function addSWAccount($use_dedicated_ip = false, array $input_overrides = null)
    {
        $input = self::_getSWAccountInput($use_dedicated_ip);
        if (!is_null($input_overrides)) {
            foreach ($input_overrides as $key => $value) {
                $input[$key] = $value;
            }
        }
        return self::addSWAccountFromInput($input);
    }

    /**
     * Adds a siteworx account from an array of data.
     *
     * @param array $input
     * @return boolean
     */
    public static function addSWAccountFromInput(array $input)
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'add',
            $input,
            $NW
        );

        if ($Reply->wasSuccessful() === false) {
            echo implode("\n", $Reply->getErrors());
        }

        return $Reply->wasSuccessful();
    }

    /**
     * Deletes the testing SW account.
     */
    public function deleteSWAccount()
    {
        $SW = $this->getSWAccount();

        $NW = self::getMasterNodeWorx();

        $input = ['domain' => $SW->getMasterDomainName()];
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'deleteCommit',
            $input,
            $NW
        );
        $this->_SiteWorx = null;
    }

    /**
     * Deletes the testing SW account.
     *
     * @param string $domain
     */
    public static function deleteSWAccountStatic($domain = self::SW_DOMAIN)
    {
        $SW = new SiteWorx($domain);
        $NW = self::getMasterNodeWorx();
        $input = ['domain' => $SW->getMasterDomainName()];
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'deleteCommit',
            $input,
            $NW
        );
    }

    /**
     * Deletes the test SW Account if it exists.
     */
    public function deleteSWAccountIfNecessary()
    {
        if (self::getSWAccountExists() === true) {
            $this->setMaxRunningTime(0);
            $this->deleteSWAccount();
        }
    }

    /**
     * Deletes the test SW Account if it exists.
     *
     * @param string $d
     */
    public static function deleteSWAccountIfNecessaryStatic($d = self::SW_DOMAIN)
    {
        if (self::getSWAccountExists($d) === true) {
            self::deleteSWAccountStatic($d);
        }
    }

    /**
     * Deletes the test Reseller account if it exists.
     *
     * @param string $email
     */
    public function deleteResellerAccountIfNecessary($email = self::RSL_EMAIL)
    {
        if (self::getResellerAccountExists($email) === true) {
            $this->deleteResellerSwAccountIfNecessary($email);
            $reseller_id = self::getResellerNodeWorx($email)->getId();

            IW::FC()->routeFromPHP(
                'Ctrl_Nodeworx_Reseller',
                'deleteCommit',
                ['reseller_id' => $reseller_id],
                NodeWorx::getDefaultNodeWorx()
            );
        }
    }

    /**
     * Do we have a SiteWorx account.
     *
     * @return boolean
     */
    public function hasSWAccount()
    {
        return !is_null($this->_SiteWorx);
    }

    /**
     * Gets a SiteWorx object for the self::SW_DOMAIN domain.
     *
     * @return SiteWorx
     */
    public function getSWAccount()
    {
        if (is_null($this->_SiteWorx)) {
            $this->setMaxRunningTime(0);
            self::addSWAccountIfNecessary();
            $this->_SiteWorx = new SiteWorx(self::SW_DOMAIN);
            ApacheVhostReader::getInstance($reset = true);
        }
        return $this->_SiteWorx;
    }

    /**
     * Set the cached siteworx account to null.
     */
    public function resetSWAccount()
    {
        $this->_SiteWorx = null;
    }

    /**
     * Gets the index data from a controller.
     *
     * @param string $controller
     * @return array
     */
    public function getSWIndexData($controller)
    {
        $SW = $this->getSWAccount();

        return IW::FC()->routeFromPHP($controller, 'index', array(), $SW)->getData();
    }

    /**
     * Assert that a message has been inserted into iworx.log.
     *
     * @param string  $message   Message to be logged
     * @param string  $log_id    Prefix of the log id string in the iworx.log
     * @param integer $timeframe Unix timestamp to look AFTER
     * @param integer $level
     * @param string  $log_file
     */
    public function assertLogEntryExists(
        $message,
        $log_id = null,
        $timeframe = null,
        $level = null,
        $log_file = null
    ) {
        if ($timeframe === null) {
            // The default timeframe is "within the last 10 seconds"
            $timeframe = IW::now() - 10;
        }

        $grep = Ini::get(Ini::BIN, 'grep');
        $tail = Ini::get(Ini::BIN, 'tail');
        $log  = ($log_file ?: Ini::get(Ini::LOG, 'file'));

        if ($log_id === null) {
            $log_id = IW::Env()->getLogSessionId() . '-' . IW::Env()->getLogRequestId();
        }

        $cmd = "{$grep} '[${log_id}' {$log} | {$grep} \"{$message}\" | {$tail} -n 1";
        $Cmd = new Command($cmd);
        $Cmd->allowPipe()->allowQuoteDouble();

        if ($Cmd->exec() === false) {
            $this->fail(__FUNCTION__ . ' failed, command failed');
        } else {
            if (count($Cmd->getOutput()) === 0) {
                $this->fail('Expected log message was not found in iworx.log');
            }

            $log_line  = $Cmd->getOutput()[0];
            $log_parts = explode(' ', $log_line);
            $log_time  = strtotime($log_parts[0] . ' ' . $log_parts[1]);

            if ($log_time < $timeframe) {
                $this->fail('Message has not been logged within the specified timeframe');
            }

            if ($level !== null) {
                $strLevel = IW::getLog()->getZendConstantStringFromMonologConstant($level);
                $this->assertEquals("[{$strLevel}]", $log_parts[3]);
            } else {
                // Increments the assertion count for the test
                $this->assertTrue(true);
            }
        }//end if
    }

    /**
     * Asserts that a command object was executed successfully.
     *
     * @param Command     $Cmd
     * @param string|null $error_message
     */
    public function assertCommandSuccessful(Command $Cmd, $error_message = null)
    {
        if ($error_message === null) {
            $error_message = implode("\n", $Cmd->getOutput());
        }

        if ($Cmd->wasSuccessful() === false) {
            $Cmd->logResult();
        }

        $this->assertTrue($Cmd->wasSuccessful(), $error_message);
    }

    /**
     * Asserts that the reply was successful.
     *
     * @param Reply  $Reply
     * @param string $error_message
     */
    public function assertReplySuccessful(Reply $Reply, $error_message = null)
    {
        if (is_null($error_message)) {
            $error_message = $Reply->getErrorsAsString();
        }
        $this->assertTrue($Reply->wasSuccessful(), $error_message);
    }

    /**
     * Asserts that the reply was not successful.
     *
     * @param Reply  $Reply
     * @param string $error_message
     */
    public function assertReplyNotSuccessful(Reply $Reply, $error_message)
    {
        $this->assertFalse(
            $Reply->wasSuccessful(),
            $error_message . ' - Reply Code: "' .
            Reply::getCodeText($Reply->getCode()) . '"'
        );
    }

    /**
     * Asserts that the passed in var is an array, and is a "Form Error" array.
     *
     * @param mixed $array not typehinted to an array on purpose
     */
    public function assertIsFormErrorArray($array)
    {
        $function  = 'var_export';
        $formatted = $function($array, 1);

        $this->assertTrue(
            is_array($array),
            'Looking for a Form Error Array, but got a ' .
                       gettype($array) .
            " ({$formatted})"
        );

        $this->assertEquals(
            self::FORM_ERROR_HEADER,
            $array[ 0 ],
            'Looking for a Form Error array, but got a ' .
                         gettype($array[ 0 ]) .
            " ({$array[ 0 ]})"
        );
    }

    /**
     * Asserts a Form Error Array contains "needle".
     *
     * @param string $needle
     * @param mixed  $haystack not typehinted to an array on purpose
     */
    public function assertFormErrorArrayContains($needle, $haystack)
    {
        $this->assertIsFormErrorArray($haystack);
        $this->assertContains(
            $needle,
            $haystack,
            'Form Error Array did not contain ' . $needle
        );
    }

    /**
     * Asserts a Form Error Array DOES NOT contain "needle".
     *
     * @param string $needle
     * @param mixed  $haystack not typehinted to an array on purpose
     */
    public function assertFormErrorArrayNotContains($needle, $haystack)
    {
        $this->assertIsFormErrorArray($haystack);
        $this->assertNotContains(
            $needle,
            $haystack,
            'Form Error Array contains ' . $needle .
            'but I did not want it to'
        );
    }

    /**
     * Verify that a string is in the content of a file.
     *
     * @param string      $file
     * @param string      $expected
     * @param string|null $message
     */
    public function assertFileContains($file, $expected, $message = null)
    {
        if ($message === null) {
            $message = "File {$file} does not contain {$expected}";
        }

        $this->assertFileExists($file, "File {$file} does not exist");
        $this->assertTrue(is_readable($file), "File {$file} is not readable");

        $content = file_get_contents($file);

        $this->assertContains($expected, $content, $message);
    }

    /**
     * Assert an array contains a value
     *
     * @param array  $array
     * @param mixed  $expected
     * @param string $message
     */
    public function assertArrayContains(array $array, $expected, $message = null)
    {
        if ($message === null) {
            $message = "Array does not contain {$expected}";
        }

        $this->assertTrue(in_array($expected, $array), $message);
    }

    /**
     * Assert an array does not contain a value.
     *
     * @param array  $array
     * @param mixed  $expected
     * @param string $message
     */
    public function assertArrayNotContains(array $array, $expected, $message = null)
    {
        if ($message === null) {
            $message = "Array does contain {$expected}";
        }

        $this->assertFalse(in_array($expected, $array), $message);
    }

    /**
     * Asserts that an number is between two other numbers.
     *
     * @param mixed  $min
     * @param mixed  $max
     * @param mixed  $actual
     * @param string $message
     */
    public function assertBetween($min, $max, $actual, $message = null)
    {
        if (is_null($message)) {
            $message = "Failed asserting that $actual is between $min and $max.";
        }

        $this->assertGreaterThanOrEqual($min, $actual, $message);
        $this->assertLessThanOrEqual($max, $actual, $message);
    }

    /**
     * Looks for a string in the output of an InterWorx CLI command.
     *
     * Regular assertContains doesn't work easily in this case - it looks for full
     * matches only, and the colorization of our output means that there are frequently
     * terminal control characters as part of the string. To work with the existing
     * function, you'd need to know which color things may be, and tack on the
     * characters manually, which is just a pain in the ass.
     *
     * @param string $needle
     * @param array  $output
     */
    public function assertCliOutputContains($needle, array $output)
    {
        if ($this->_isInCliOutput($needle, $output) === false) {
            $this->fail('CLI output did not contain ' . $needle);
        }
    }

    /**
     * Assert that a string is not present in an array.
     *
     * Be careful, this is sorta vague.  If you have a particularly short $needle, for
     * example.
     *
     * @param string $needle
     * @param array  $output
     */
    public function assertCliOutputNotContains($needle, array $output)
    {
        if ($this->_isInCliOutput($needle, $output)) {
            $this->fail('CLI output contained ' . $needle);
        }
    }

    /**
     * Checks if a needle is a partial match to a haystack.
     *
     * @param string $needle
     * @param array  $output
     * @return boolean
     */
    private function _isInCliOutput($needle, array $output)
    {
        foreach ($output as $line) {
            if (strpos($line, $needle) !== false) {
                return true;
            }
        }

        return false;
    }

    /**
     * Resets the forms before every test.
     */
    public function setUp()
    {
        Form::resetInstances();
        IW::Env()->resetSession();

        // Clear the input values
        $input = IW::Env()->getAllInput();

        foreach ($input as $name => $value) {
            IW::Env()->addInput($name, null);
        }

        IWorxDbUtil::killCachedDbInstance();
        $this->_setMaxTimeLimit();
    }

    /**
     * Sets the max time limit depending on the group selection.
     */
    private function _setMaxTimeLimit()
    {
        $opts = array( 'exclude-group=s' => '' );
        $Opt  = new IWorx_Console_Getopt($opts);
        if ($Opt->getOption('exclude-group') === 'slow-tests') {
            $this->setMaxRunningTime(self::QUICK_TEST_TIME_LIMIT);
        } else {
            $this->setMaxRunningTime(0);
        }
    }

    /**
     * I can't believe this all is now one function. :P.
     *
     * Does a backup and import on an account.
     *
     * @param string $object_callback
     * @param string $domain
     * @return boolean
     */
    public function doBackupAndImport(
        $object_callback = null,
        $domain = self::SW_DOMAIN
    ) {
        self::addSWAccountIfNecessary();

        $tgz_path = $this->backupAccount($domain);

        $this->deleteSWAccount();

        if (!is_null($object_callback)) {
            // This is useful to make system changes before the import happens.
            $this->{$object_callback}();
        }

        return $this->importBackup($tgz_path);
    }

    /**
     * Import a backup file.
     *
     * @param string $tgz_path
     * @return boolean
     */
    public function importBackup($tgz_path)
    {
        $NW = self::getMasterNodeWorx();

        // Import the backup
        $ip        = self::_getFirstSharedIP();
        $db_server = $NW->getMySQLDriver()->getLocalServer();

        $input = array( 'archive'     => $tgz_path,
                    'cp'          => 'siteworx',
                    'dbserver'    => $db_server->getNickname(),
                    'reseller_id' => $NW->getId(),
                    'ipaddr'      => $ip );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Import',
            'import',
            $input,
            $NW
        );
        //Cleanup
        IWorxFileSys::rm($tgz_path);

        $this->assertReplySuccessful($Reply);

        return $Reply->wasSuccessful();
    }

    /**
     * Backs up the account and returns the path of the backup.
     *
     * @param string $domain
     * @return string $filepath
     */
    public function backupAccount($domain)
    {
        $tmp      = Ini::get(Ini::DIR, 'tmp');
        $path     = "{$tmp}/";
        $filename = 'backupforunittest';
        $tgz_path = "{$path}{$filename}.tgz";

        $backup = Ini::get(Ini::IWORX_BIN, 'backup');

        $cmd = "$backup
      --domain {$domain}
      --output-dir={$path}
      --filename-format={$filename}";

        IWorxExec::exec($cmd, $result, $retval);
        $this->assertTrue($retval == 0);

        return $tgz_path;
    }

    /**
     * Change the database on the dsn to the one given.
     *
     * @param string $database
     * @param string $dsn
     * @return string
     */
    public function changeDatabaseOnDsn($database, $dsn)
    {
        $new_dsn = preg_replace('/\/mysql$/', "/{$database}", $dsn);
        return $new_dsn;
    }

    /**
     * Check to see if the given routine exists.
     *
     * @param string $database
     * @param string $routine_name
     * @return boolean
     */
    public function checkIfRoutineExists($database, $routine_name)
    {
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);
        $Driver   = $SiteWorx->getMySQLDriver();
        $db       = IWorxDbUtil::db_connect($Driver->getDsn(), $die = false);

        $q_routine_name = q($routine_name);
        $q_database     = q($Driver->getFQDN($database));

        $sql = "SELECT COUNT(*)
            FROM   `proc`
            WHERE  `name` = {$q_routine_name} AND
                   `db`   = {$q_database};";

        $routine_count = IWorxDbUtil::db_getOne($db, $sql);
        return $routine_count > 0;
    }

    /**
     * Deletes the routine as the "SUPER" iworx user.
     *
     * @param string $database
     * @param string $routine_name
     * @return boolean
     */
    public function deleteRoutineAsSuper($database, $routine_name)
    {
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);
        $Driver   = $SiteWorx->getMySQLDriver();

        $dsn = $this->changeDatabaseOnDsn($database, $Driver->getDsn());
        $db  = IWorxDbUtil::db_connect($dsn, $die = false);

        $sql    = "DROP PROCEDURE IF EXISTS `{$routine_name}`";
        $result = IWorxDbUtil::db_query($db, $sql);

        return Db::isError($result) === false;
    }

    /**
     * Adds a reseller given an array of input.
     *
     * @param array $input
     * @return Reply
     */
    protected function _addResellerFromInput(array $input)
    {
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Reseller',
            'addCommit',
            $input,
            self::getMasterNodeWorx()
        );

        $this->assertReplySuccessful($Reply);
        return $Reply;
    }

    /**
     * Logs in a given SiteWorx account, falsifying the active session.
     *
     * @param SiteWorx $SW
     */
    protected function _loginThisSiteworx(SiteWorx $SW)
    {
        $SW->startSession();
    }

    /**
     * Wait until the command queues are caught up.
     */
    protected function _waitForCommandQueueToCatchUp()
    {
        if (IW::Env()->isManager() === false) {
            return;
        }

        IW::getLog()->info('Checking Command Queue status');

        do {
            $has_unplayed = false;

            foreach (NodeManager::getNodeInstanceList() as $Node) {
                if (IW::CommandQueue()->getUnplayedCommands($Node->getIPv4())) {
                    IW::getLog()->info("Waiting on Command Queue on node {$Node->getIPv4()}");
                    $has_unplayed = true;
                }
            }

            if (IW::CommandQueue()->hasStoppedNodes()) {
                $this->fail('Command Queue has offline nodes, unable to recover');
            }

            if ($has_unplayed) {
                sleep(2);
            }
        } while ($has_unplayed === true);
    }

    /**
     * Call protected/private method of a class.
     *
     * @param object $object     Instantiated object that we will run method on.
     * @param string $methodName Method name to call
     * @param array  $parameters Array of parameters to pass into method.
     *
     * @return mixed Method return.
     */
    public function invokeMethod(&$object, $methodName, array $parameters = array())
    {
        $reflection = new \ReflectionClass(get_class($object));
        $method = $reflection->getMethod($methodName);
        $method->setAccessible(true);

        return $method->invokeArgs($object, $parameters);
    }

    /**
     * Disconnect DB.
     */
    public static function tearDownAfterClass()
    {
        IW::DB()->disconnect();
    }

    /**
     * @param string        $domain_name
     * @param string        $type
     * @param string        $point_to
     * @param SiteWorx|null $SW
     */
    protected function _addPointerDomain(
        $domain_name = self::SW_POINTER_DOMAIN,
        $type = Domain::PTR_ALIAS,
        $point_to = self::SW_DOMAIN,
        $SW = null
    ) {
        $SW = is_null($SW) ? $this->getSWAccount() : $SW;
        if ($SW->domainExists($domain_name)) {
            return;
        }

        // need to set to false for redir to validate (when points_to not on server)
        $create_mail_alias = $type === Domain::PTR_ALIAS;

        $input = array(
            'domain' => $domain_name,
            'points_to' => $point_to,
            'redir_type' => $type,
            'create_mail_alias' => $create_mail_alias
        );

        $Reply = IW::FC()->routeFromPHP(
            Ctrl_Siteworx_DomainsPointer::class,
            'add',
            $input,
            $SW
        );

        $this->assertReplySuccessful($Reply);
    }
}
