<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 2, 2008
 */

/**
 * Util class used for managing debug status.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class TestDebug {
  private static $_initial_debug_state;

  /**
   * Sets the initial debug state.
   */
  private static function _setInitialDebugState() {
    if( is_null( self::$_initial_debug_state ) ) {
      if( Ini::get( Ini::IWORX, 'debug' ) === '1' ) {
        self::$_initial_debug_state = true;
      } else {
        self::$_initial_debug_state = false;
      }
    }
  }

  /**
   * Turns debug on in iworx.ini.
   */
  public static function turnDebugOn() {
    self::_setInitialDebugState();
    Ini::set( Ini::IWORX, 'debug', '1' );
    Ini::write();
  }

  /**
   * Turns debug off in iworx.ini.
   */
  public static function turnDebugOff() {
    self::_setInitialDebugState();
    Ini::set( Ini::IWORX, 'debug', '0' );
    Ini::write();
  }

  /**
   * Restores debug to initial state in iworx.ini.
   */
  public static function restoreDebug() {
    if( self::$_initial_debug_state === true ) {
      self::turnDebugOn();
    } else {
      self::turnDebugOff();
    }
  }
}
