<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    May 21, 2009
 */

/**
 * Test to make sure the proper elements are being displayed on the import.
 * single account pages
 *
 * @group NodeWorx
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageNWImportTest extends IWorxSeleniumTestCase
{

    /**
     * TestImportInitialFormRender.
     */
    public function testImportInitialFormRender()
    {
        $this->loginToNWAndGoTo(
            '/nodeworx/import?action=importSingle',
            self::NW_USER,
            self::NW_PASS
        );

        $this->assertPageRenderedWithoutErrors();
        $this->click('submitbutton');
        $this->waitForPageToLoad();
        $this->assertPageRenderedWithoutErrors();

        $this->assertTranslatedTextPresent('##LG_SWITCH_TO_FILE_LOCAL##');
    }

    /**
     * TestImportUploadFormRender.
     */
    public function testImportUploadFormRender()
    {
        $this->loginToNWAndGoTo(
            '/nodeworx/import?action=importSingle',
            self::NW_USER,
            self::NW_PASS
        );

        $this->assertPageRenderedWithoutErrors();

        $this->open('/nodeworx/import?action=importSingle');
        $this->click('submitbutton');
        $this->waitForPageToLoad();

        $this->assertTranslatedTextPresent('##LG_LOCALBACKUPFILE##');

        $this->assertElementPresent('archive');
        $this->assertElementPresent('cp');
        $this->assertElementPresent('reseller_id_copy');
        $this->assertElementPresent('assign_ips_manually');
        $this->assertElementPresent('dbserver');
    }

    /**
     * TestImportLocalFileFormRender.
     */
    public function testImportLocalFileFormRender()
    {
        $this->loginToNWAndGoTo(
            '/nodeworx/import?action=importSingle',
            self::NW_USER,
            self::NW_PASS
        );

        $this->assertPageRenderedWithoutErrors();
        $this->click('submitbutton');
        $this->waitForPageToLoad();

        $this->assertPageRenderedWithoutErrors();
        $this->click('archive_switcher');
        $this->assertElementPresent('archive');
        $this->assertElementPresent('cp');
        $this->assertElementPresent('reseller_id_copy');
        $this->assertElementPresent('assign_ips_manually');
        $this->assertElementPresent('dbserver');
    }

    /**
     * TestActualImport.
     */
    public function testActualImport()
    {
        $this->addSWAccountIfNecessary();

        $this->loginToNWAndGoTo(
            '/nodeworx/import?action=importSingle',
            self::NW_USER,
            self::NW_PASS
        );

        $this->open('/nodeworx/import?action=importSingle');
        $this->click('submitbutton');
        $this->waitForPageToLoad();

        $backup_path = $this->_backupAccount();

        $this->assertPageRenderedWithoutErrors();
        $this->click('archive_switcher');
        $this->typeInto('archive', $backup_path);
        $this->clickAndWaitForPopupForm('submitbutton');

        $this->click('submitbutton');

        $this->waitForMessageBanner();
        $this->assertTranslatedTextPresent('##LG_RESTORE_SUCCESS##');

        $homedir   = '/home/' . self::SW_UNIQNAME;
        $homeperms = substr(sprintf('%o', fileperms($homedir)), -4);
        $this->assertNotEquals('0700', $homeperms, "{$homedir} perms are bad");
    }

    /**
     * Backs up the test siteworx account.
     *
     * @return string
     */
    private function _backupAccount()
    {
        $domain = self::SW_DOMAIN;

        $tmp      = Ini::get(Ini::DIR, 'tmp');
        $path     = "{$tmp}/";
        $filename = 'backupforunittest';
        $tgz_path = "{$path}{$filename}.tgz";

        $backup = Ini::get(Ini::IWORX_BIN, 'backup');

        $cmd = "$backup
      --domain {$domain}
      --output-dir={$path}
      --filename-format={$filename}";

        IWorxExec::exec($cmd, $result, $retval);
        $this->assertTrue($retval == 0);

        return $tgz_path;
    }
}
