<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @group      NodeWorx
 * @date       Sep 11, 2008
 * $Id$
 */

/**
 * Tests for the index page of NodeWorx.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageNWIndexTest extends IWorxSeleniumTestCase
{

    /**
     * Url to the phpunit coverage start script.
     *
     * @var string
     */
    //  protected $coverageScriptUrl = 'http://dev.interworx.com:2080/nodeworx/phpunit_coverage.php';

    /**
     * Tests that the login works.
     */
    public function testLoginWithGoodCredentials()
    {
        $this->open('/nodeworx/');
        $this->assertPageRenderedWithoutErrors();

        $this->loginToNWAsUser();

        $this->assertPageRenderedWithoutErrors();

        // Make sure we were redirected where we expected to be
        $this->assertContains('/nodeworx/overview', $this->getLocation());
        // Check the title of the page
        $this->assertEquals('InterWorx-CP :: NodeWorx', $this->getTitle());
    }

    /**
     * Tests that logging in with bad information fails.
     */
    public function testLoginWithBadPassword()
    {
        $this->open('/nodeworx/');

        $this->assertPageRenderedWithoutErrors();
        // Check with bad password
        $this->loginToNWAsUser(self::NW_USER, 'badpassword');
        $this->assertTextPresent(IWorxTranslate::tr('##LG_AUTHENTICATIONERROR##'));
    }

    /**
     * Tests that loggin in without a password fails.
     */
    public function testLoginWithNoPassword()
    {
        $this->open('/nodeworx/');

        $this->assertPageRenderedWithoutErrors();
        // Check with password blank
        $this->loginToNWAsUser(self::NW_USER, '');
        $msg = '##LG_INPUT_FORM_DID_NOT_VALIDATE##';
        $this->assertTextPresent(IWorxTranslate::tr($msg));
    }

    /**
     * Tests that logging in with a bad username fails.
     */
    public function testLoginWithBadUsername()
    {
        $this->open('/nodeworx/');

        $this->assertPageRenderedWithoutErrors();
        // Check with bad username
        $this->loginToNWAsUser('baduser@derp.com');
        $msg = '##LG_INPUT_FORM_DID_NOT_VALIDATE##';
        $this->assertTextPresent(IWorxTranslate::tr($msg));
    }

    /**
     * Tests that logging in with a blank username fails.
     */
    public function testLoginWithNoUsername()
    {
        $this->open('/nodeworx/');

        $this->assertPageRenderedWithoutErrors();
        // Check with no username
        $this->loginToNWAsUser('');
        $msg = '##LG_INPUT_FORM_DID_NOT_VALIDATE##';
        $this->assertTextPresent(IWorxTranslate::tr($msg));
    }

    /**
     * Tests that anonymous logins fail.
     */
    public function testLoginWithNoPasswordOrUsername()
    {
        $this->open('/nodeworx/');

        $this->assertPageRenderedWithoutErrors();
        // Check with no username and no password
        $this->loginToNWAsUser('', '');
        $msg = '##LG_INPUT_FORM_DID_NOT_VALIDATE##';
        $this->assertTextPresent(IWorxTranslate::tr($msg));
    }

    /**
     * Tests that the logout button actually logs you out.
     */
    public function testLogoutByQuickMenu()
    {
        $this->loginToNWAndGoTo('/nodeworx/overview');

        $link = IWorxTranslate::tr('##LG_LOGOUT##');
        $this->clickAndWait('link=' . $link);
        // Make sure we're back at the login screen
        $this->assertElementPresent('iw-login');

        // Now try to open something again
        $this->open('/nodeworx/overview');

        // Make sure we're back at the login screen
        $this->assertElementPresent('iw-login');
    }

    /**
     * Tests that the logout button actually logs you out.
     */
    public function testLogoutByMainMenu()
    {
        $this->loginToNWAndGoTo('/nodeworx/overview');

        $this->clickAndWait('iw-menu-logout');

        // Make sure we're back at the login screen
        $this->assertElementPresent('iw-login');

        // Now try to open something again
        $this->open('/nodeworx/overview');

        // Make sure we're back at the login screen
        $this->assertElementPresent('iw-login');
    }
}
