<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 */

/**
 * Tests for the Updates page.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 * @retryLimit 10
 */
class PageNWUpdatesTest extends IWorxSeleniumTestCase
{
    /**
     * Force installs an old package.
     */
    private function _forceInstallOldPackage()
    {
        $cmd = 'yum -y downgrade tzdata';

        IWorxExec::exec($cmd);
    }

    /**
     * Checks if updates are available.
     *
     * @return boolean
     */
    private function _isUpdatesAvailable()
    {
        return !( $this->isTranslatedTextPresent('##LG_NOUPDATESPENDING##') );
    }

    /**
     * Checks if YUM is still running.
     *
     * @return boolean
     */
    private function _yumIsStillRunning()
    {
        $js     = 'window.yumLogHandler.isStillRunning';
        $result =  $this->getEval($js);

        return $result === true;
    }

    /**
     * Sets up the old package, if necessary.
     */
    public function testUpdatesAvailable()
    {
        $this->loginToNWAndGoTo('nodeworx/updates');

        if ($this->_isUpdatesAvailable() === false) {
            $this->_forceInstallOldPackage();
            $NW = IWorx_Framework_TestCase::getMasterNodeWorx();

            $Reply = IW::FC()->routeFromPHP(
                'Ctrl_Nodeworx_Updates',
                'refresh',
                array(),
                $NW
            );

            $this->assertReplySuccessful($Reply);
        }
    }

    /**
     * Doublechecks that the force install worked.
     */
    public function testUpdatesAvailableFollowup()
    {
        $this->loginToNWAndGoTo('nodeworx/updates');

        $this->assertTranslatedTextNotPresent(
            '##LG_NOUPDATESPENDING##',
            'Force install of old RPM failed'
        );
    }

    /**
     * Installs the updates.
     */
    public function testInstallingUpdates()
    {
        $this->loginToNWAndGoTo('nodeworx/updates');
        $xpath = "xpath=(//div[@id='available_updates-shell']" .
        "//input[@type='checkbox'])[1]";

        if ($this->_isUpdatesAvailable() === true) {
            $this->click($xpath);
            $this->select('iw_action_name4', 'label=' .
                     IWorxTranslate::tr('##LG_INSTALL_UPDATES##'));
            $this->click('available_updates-submit');

            $this->waitForElement('css=.iw-log-output', 5);

            do {
                // Sleep a bit to wait for the install to complete
                sleep(5);
            } while ($this->_yumIsStillRunning());
        }
    }

    /**
     * Checks that updates applied.
     */
    public function testInstallWorked()
    {
        $this->loginToNWAndGoTo('nodeworx/updates');

        $this->assertFalse($this->_isUpdatesAvailable());
    }
}
