<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Aug 28, 2009
 */

/**
 * Tests for htaccess.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageSWHtaccessTest extends IWorxSeleniumTestCase
{

    static private $_first_setup = false;

    /**
     * Make sure the SiteWorx account has been deleted.
     */
    public function setUp()
    {
        parent::setUp();

        if (self::$_first_setup === false) {
            $this->_fwDeleteSWAccountIfNecessary();
            $this->_fwAddSWAccountIfNecessary();
            self::$_first_setup = true;
        }//end if
    }

    /**
     * Enables global htaccess.
     */
    public function testEnableGlobalHtaccess()
    {
        $this->addSWAccountIfNecessary();
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);

        $this->loginToSWAndGoTo('/siteworx/htaccess/');

        $this->clickAndWaitForPopupForm('htaccess-add-link');
        $this->click('htaccess_add-submit');
        $this->waitForMessageBanner();
        $this->assertTranslatedTextPresent('##LG_HTACCESSMGR_ADD_SUCCEEDED##');

        $this->assertPageRenderedWithoutErrors();
    }


    /**
     * Tests disabling a global htaccess file.
     */
    public function testHtaccessDisableGlobal()
    {
        $this->addSWAccountIfNecessary();
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);

        $this->loginToSWAndGoTo('/siteworx/htaccess/');

        $this->click('dir_0');
        $this->select('iw_action_name2', 'label=Disable');
        $this->clickAndWaitForPopupForm('submitbutton');

        $this->click('htaccess_dir-submit');
        $this->waitForMessageBanner();
        $this->assertPageRenderedWithoutErrors();
        $this->assertTranslatedTextPresent('##LG_HTACCESSMGR_DISABLE_SUCCEEDED##');
    }

    /**
     * Deletes the global htaccess.
     */
    public function testDeleteGlobalHtaccess()
    {
        $this->addSWAccountIfNecessary();
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);

        $this->loginToSWAndGoTo('/siteworx/htaccess/');

        $path = '//a[@href="/siteworx/htaccess?action=delete&htaccess_dirs=/home/addedbyt/addedbytest.com/html"]';
        $this->clickAndWaitForPopupForm($path);

        $this->assertPageRenderedWithoutErrors();

        $this->click('submitbutton');
        $this->waitForMessageBanner();

        $this->assertTranslatedTextPresent('##LG_HTACCESS_DELETE_SUCCEEDED##');
    }
}
