<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 26, 2010
 */

/**
 * Tests for the siteworx/mysql/db page.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageSWMysqlDbTest extends IWorxSeleniumTestCase
{

    const DB_NAME   = 'testdb';
    const USER_NAME = 'db_test';
    const DB_PASS   = 'testpass888818818';

    static private $_first_setup = false;

    /**
     * Make sure the SiteWorx account has been deleted.
     */
    public function setUp()
    {
        parent::setUp();

        if (self::$_first_setup === false) {
            $this->_fwDeleteSWAccountIfNecessary();
            $this->_fwAddSWAccountIfNecessary();
            self::$_first_setup = true;
        }//end if

        $this->tearDown();
    }

    /**
     * Tear down function.
     */
    public function tearDown()
    {
        $SiteWorx  = new SiteWorx(self::SW_DOMAIN);
        $Driver    = $SiteWorx->getMySQLDriver();
        $db_name   = self::DB_NAME;
        $user_name = self::USER_NAME;

        if ($Driver->databaseExists($db_name) === true) {
            $Driver->deleteDatabase($db_name, $purge_perms = true);
        }
        if ($Driver->userExists($user_name)) {
            $Driver->deleteUser($user_name);
        }
    }

    /**
     * Test the show flag.
     */
    public function testShowFlag()
    {
        $this->loginToSWAndGoTo('/siteworx/mysql/db');

        $this->assertTrue($this->_isDisplayNoneSet('user-tablerow'));
        $this->click('create_user');
        $this->assertFalse($this->_isDisplayNoneSet('user-tablerow'));
    }

    /**
     * Tests adding a database with a user.
     */
    public function testAddingADatabaseWithUser()
    {
        $this->loginToSWAndGoTo('/siteworx/mysql/db');

        $this->typeInto('name', self::DB_NAME);
        $this->click('create_user');
        $this->typeInto('user', self::USER_NAME);
        $this->typeInto('password', self::DB_PASS);
        $this->typeInto('confirm_password', self::DB_PASS);

        $this->click('submitbutton');

        $this->waitForMessageBanner();

        $message = '##LG_DATABASE_ADDEDSUCCESSFULLY##';
        $this->assertTextPresent(IWorxTranslate::tr($message));
    }

    /**
     * Tests adding a database with the create_user box checked but no user info.
     */
    public function testAddingDbWithUserCheckAndNoUserInfo()
    {
        $this->loginToSWAndGoTo('/siteworx/mysql/db');

        $this->typeInto('name', self::DB_NAME);
        $this->click('create_user');
        $this->typeInto('user', '');
        $this->typeInto('password', '');
        $this->typeInto('confirm_password', '');
        $this->typeInto('host', '');

        $this->click('submitbutton');

        $this->waitForMessageBanner();

        $message = '##LG_INPUT_FORM_DID_NOT_VALIDATE##';
        $this->assertTextPresent(IWorxTranslate::tr($message));
    }

    /**
     * Checks to see if the display:none style is set.
     *
     * @param string $id
     * @return boolean
     */
    private function _isDisplayNoneSet($id)
    {
        $element = "window.document.getElementById( '{$id}' )";
        $style   = '.style.display;';
        $display = $this->getEval($element . $style);

        return $display == 'none';
    }
}
