<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jan 16, 2009
 */

/**
 * Tests to ensure Bug# 949 was corrected.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class Bug_949_FixTest extends IWorxSeleniumTestCase
{

    /**
     * TestHotlinkProtection.
     */
    public function testHotlinkProtection()
    {
        $this->addSWAccountIfNecessary();
        $this->loginToSWAndGoTo('/siteworx/overview');
        $this->assertPageRenderedWithoutErrors();

        $this->click('iw-menu-features');
        $this->click('iw-menu-features-htaccess');
        $this->waitForPageToLoad();

        $path = '//a[@href="/siteworx/htaccess?action=editCommit&dir=/home/addedbyt/addedbytest.com/html"]';
        if (!$this->isElementPresent($path)) {
            $this->clickAndWaitForPopupForm('htaccess-add-link');
            $this->click('htaccess_add-submit');
            $this->waitForMessageBanner();
            $this->assertTranslatedTextPresent('##LG_HTACCESSMGR_ADD_SUCCEEDED##');
            $this->closeMessageBanner();
            $this->assertPageRenderedWithoutErrors();
            sleep(5); // waiting for payload to refresh
        }

        $this->click($path);
        $this->waitForPageToLoad();

        /*
         * Turn hotlink protection off and remove all extensions.
         */
        $path = '//a[text()="Hotlink Protection"]';
        $this->clickAndWaitForPopupForm($path);
        $this->select('status', 'label=Off');
        $this->typeInto('extensions', '');
        $this->click("//input[@value='Update']");
        $this->waitForMessageBanner();

        $message = IWorxTranslate::tr('##LG_HTACCESS_HOTLINK_UPDATE_SUCCEEDED##');
        $this->assertTextPresent($message);
        $this->closeMessageBanner();

        $path = '//a[text()="Hotlink Protection"]';
        $this->clickAndWaitForPopupForm($path);

        $this->assertElementValueEquals('status', 'off');
        $this->assertElementNotContainsText('extensions', '.jpeg');

        /*
         * Turn hotlink protection on and add extensions
         */
        $this->select('status', 'label=On');
        $this->typeInto('extensions', '.jpeg');
        $this->click("//input[@value='Update']");
        $this->waitForMessageBanner();

        $this->assertTextPresent($message);
        $this->closeMessageBanner();

        $path = '//a[text()="Hotlink Protection"]';
        $this->clickAndWaitForPopupForm($path);
        $this->assertElementValueEquals('status', 'on');
        $this->assertElementContainsText('extensions', '.jpeg');

        /*
         * Leave hotlink protection on and remove all extensions.
         */
        $this->typeInto('extensions', '');
        $this->click("//input[@value='Update']");
        $this->waitForMessageBanner();
        $this->closeMessageBanner();

        $path = '//a[text()="Hotlink Protection"]';
        $this->clickAndWaitForPopupForm($path);

        $this->assertElementValueEquals('status', 'on');
        $this->assertElementNotContainsText('extensions', '.jpeg');
        $this->click("//input[@value='Cancel']");

        /*
         * Get me out of here.
         */
        $this->click('iw-menu-features-htaccess');
        $this->waitForPageToLoad();

        /*
         * Remove the global htaccess file
         */
        $path = '//a[@href="/siteworx/htaccess?action=delete&htaccess_dirs=/home/addedbyt/addedbytest.com/html"]';
        $this->clickAndWaitForPopupForm($path);

        $this->assertPageRenderedWithoutErrors();

        $this->click('submitbutton');
        $this->waitForMessageBanner();

        $this->assertTranslatedTextPresent('##LG_HTACCESS_DELETE_SUCCEEDED##');
    }
}
