<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Jan 19, 2010
 */

/**
 * Tests for Backward Compatibility.
 *
 * NEVER CHANGE ANYTHING IN AN INPUT ARRAY IN THIS FILE.  IT IS HERE TO ENSURE
 * WE DON'T BREAK BACKWARD COMPATIBILITY WITH OUR API.  DON'T BE FULL OF FAIL
 * LIKE ME.
 *                      -Jon
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class API_Backwards_XMLRPCTest extends IWorx_API_Framework_TestCase
{

    /**
     * Does a hard coded test trying to add a SiteWorx account using the API.
     *
     * This is hard coded for a reason, we're testing to see if we break the
     * API by making any changes to it.
     */
    public function testAddingASiteWorxAccount()
    {
        $this->deleteSWAccountIfNecessary();

        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        //    $ul = Ini::get( Ini::CONSTANTS, 'unlimited' );
        $ul = '999999999';

        $input = array ( 'domainname'               => self::SW_DOMAIN,
                     'ipaddress'                => self::_getFirstSharedIP(),
                     'uniqname'                 => self::SW_UNIQNAME,
                     'nickname'                 => self::SW_NICK,
                     'email'                    => self::SW_EMAIL,
                     'password'                 => self::SW_PASS,
                     'confirm_password'         => self::SW_PASS,
                     'OPT_STORAGE'              => $ul,
                     'OPT_BANDWIDTH'            => $ul,
                     'OPT_EMAIL_ALIASES'        => $ul,
                     'OPT_EMAIL_AUTORESPONDERS' => $ul,
                     'OPT_EMAIL_BOXES'          => $ul,
                     'OPT_EMAIL_GROUPS'         => $ul,
                     'OPT_FTP_ACCOUNTS'         => $ul,
                     'OPT_MYSQL_DBS'            => $ul,
                     'OPT_MYSQL_DB_USERS'       => $ul,
                     'OPT_POINTER_DOMAINS'      => $ul,
                     'OPT_SLAVE_DOMAINS'        => $ul,
                     'OPT_SUBDOMAINS'           => $ul,
                     'OPT_BACKUP'               => 1,
                     'OPT_CGI_ACCESS'           => 1,
                     'OPT_CRONTAB'              => 1,
                     'OPT_DNS_RECORDS'          => 1,
                     'OPT_RESOLVE_XFERLOG_DNS'  => 1,
                     'OPT_SSL'                  => 1,
                     'OPT_BURSTABLE'            => 1,
                     'OPT_SAVE_XFER_LOGS'       => 1 );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'add',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);

        $this->assertEquals(API::SUCCESS, $result['status']);
    }

    /**
     * Tests that suspending a siteworx account works via the API.
     */
    public function testSuspendingASiteWorxAccount()
    {
        self::addSWAccountIfNecessary();

        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        $input = array( 'domain' => self::SW_DOMAIN,
                    'status' => '0' );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'edit',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);

        $this->assertEquals(API::SUCCESS, $result['status']);
    }

    /**
     * Tests that deleting a siteworx account works via the API.
     */
    public function testDeletingASiteWorxAccount()
    {
        self::addSWAccountIfNecessary();

        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        $input = array( 'domain' => self::SW_DOMAIN );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'delete',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);

        $this->assertEquals(API::SUCCESS, $result['status']);
    }

    /**
     * Test to ensure bug #1385 (MySQL Server Pooling redactor broke backwards.
     *
     * compatibility) has been fixed.
     */
    public function testSiteWorxAddActionBug1385()
    {
        $this->deleteSWAccountIfNecessary();

        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        $input                    = self::_getSWAccountInput();
        $input['database_server'] = 'localhost';

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'add',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);

        $this->assertEquals(API::SUCCESS, $result['status']);
    }

    /**
     * Test to ensure bug #1385 (MySQL Server Pooling redactor broke backwards.
     *
     * compatibility) has been fixed.
     */
    public function testResellerAddActionBug1385()
    {
        self::deleteResellerAccountIfNecessary();

        $NW    = self::getMasterNodeWorx();
        $input = $this->_getResellerInput();

        $input['database_servers'] = array( 'localhost' );

        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/reseller',
                     'action'    => 'add',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);

        $this->assertEquals(API::SUCCESS, $result['status']);
    }

    /**
     * Test to ensure bug #1385 (MySQL Server Pooling redactor broke backwards.
     *
     * compatibility) has been fixed.
     */
    public function testResellerEditActionBug1385()
    {
        $NW = self::getMasterNodeWorx();

        $input = array( 'reseller_id' => self::getResellerNodeWorx()->getId(),
                    'database_servers' => array( 'localhost' ) );

        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/reseller',
                     'action'    => 'edit',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);

        $this->assertEquals(API::SUCCESS, $result['status']);
    }

    /**
     * Test to ensure that they database list datasouce is what SimpleScripts.
     *
     * is expecting.
     */
    public function testSimplescriptsIntegrationJan2010Break()
    {
        self::addSWAccountIfNecessary();

        $Driver = self::getSWAccount()->getMySQLDriver();

        $Driver->uninstall();
        $Driver->addDatabase('testdb');

        $key = array( 'email'    => self::SW_EMAIL,
                  'password' => self::SW_PASS,
                  'domain'   => self::SW_DOMAIN);

        $input  = [];
        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/mysql/db',
                     'action'    => 'list',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);

        $expected = array( 0 => 'testdb',
                       1 => 'addedbyt_testdb' );

        $this->assertEquals(API::SUCCESS, $result['status']);
        $this->assertEquals($expected, $result['payload'][0]);
    }
}
