<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2012 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jun 19, 2012
 */

/**
 * Tests the getip function.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class API_GetipTest extends IWorx_API_Framework_TestCase
{
    /**
     * Data provider.
     *
     * @return array
     */
    public function dataProviderRules()
    {
        return array(
                 array( 'first' ),
                 array( 'last' ),
                 array( 'least' ),
                 array( 'most' ),
                 );
    }

    /**
     * Helper to do the api call itself.
     *
     * @param string $function Function name to call
     * @param array  $args     Arguments to pass in
     * @return mixed
     */
    protected function _apiCall($function, array $args = [])
    {
        $args['apikey'] = $this->_getGoodApiKey();

        $result = $this->_xml->call("iworx.{$function}", array( $args ));

        return $result;
    }

    /**
     * Make sure we get a result with no arguments.
     */
    public function testGetipDefaults()
    {
        $result = $this->_apiCall('getip');

        $this->assertTrue(IWorxValidate::ipv4($result));
    }

    /**
     * Tests the different rules.
     *
     * @param string $rule
     * @dataProvider dataProviderRules
     */
    public function testRules($rule)
    {
        $args = array( 'rule' => $rule );

        $result = $this->_apiCall('getip', $args);

        $this->assertTrue(IWorxValidate::ipv4($result));
    }
}
