<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Jun 4, 2013
 */

/**
 * Tests variations on using the packagetemplate option when adding/editing account.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class API_Siteworx_PackageOptionsTest extends IWorx_API_Framework_TestCase
{

    const PACKAGE_NAME  = 'unlimited';
    const PACKAGE_NAME2 = 'fifteens';

    /**
     * Does a hard coded test trying to add a SiteWorx account using the API.
     */
    public function testAddingASiteWorxAccountWithTemplate()
    {
        $this->deleteSWAccountIfNecessary();
        $this->_addUnlimPackage();
        $this->_addFifteensPackage();

        $this->_addSWUsingPackageTemplate();
        $this->resetSWAccount();
        $this->_editSWUsingDifferentTemplate();
    }

    /**
     * Add unlimited package.
     */
    private function _addUnlimPackage()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'package_name' => self::PACKAGE_NAME );
        $input = array_merge($input, self::_getPackageOptionsInput());
        $Reply = IW::FC()->routeFromPHP('Ctrl_Nodeworx_Packages', 'add', $input, $NW);
    }

    /**
     * Add fifteens package.
     */
    private function _addFifteensPackage()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'package_name' => self::PACKAGE_NAME2 );
        $options = self::_getPackageOptionsInput();
        foreach ($options as &$option) {
            if ($option === Ini::get(Ini::CONSTANTS, 'unlimited')) {
                $option = '15';
            }
        }
        $input = array_merge($input, $options);
        $Reply = IW::FC()->routeFromPHP('Ctrl_Nodeworx_Packages', 'add', $input, $NW);
    }

    /**
     * Add sw account using package template.
     */
    private function _addSWUsingPackageTemplate()
    {
        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        $input = array ( 'domainname'       => self::SW_DOMAIN,
                     'ipaddress'        => self::_getFirstSharedIP(),
                     'uniqname'         => self::SW_UNIQNAME,
                     'nickname'         => self::SW_NICK,
                     'email'            => self::SW_EMAIL,
                     'password'         => self::SW_PASS,
                     'confirm_password' => self::SW_PASS,
                     'packagetemplate'  => self::PACKAGE_NAME
                     );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'add',
                     'input'     => $input );

        $result = $this->_apiCall('route', $params);
        $this->assertEquals(API::SUCCESS, $result['status']);
        $b = $this->getSWAccount()->getPackage()->getOptionValue(Option::EMAIL_BOXES);
        $this->assertEquals(Ini::get(Ini::CONSTANTS, 'unlimited'), $b);
    }

    /**
     * Edit sw account using different package template.
     */
    private function _editSWUsingDifferentTemplate()
    {
        $key = array( 'email'    => self::NW_USER,
                  'password' => self::NW_PASS );

        $input = array( 'domain' => self::SW_DOMAIN,
                    'packagetemplate' => self::PACKAGE_NAME2,
                    substr(Option::EMAIL_ALIASES, 3) => '14' );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'edit',
                     'input'     => $input );
        $result = $this->_apiCall('route', $params);

        $this->assertEquals(API::SUCCESS, $result['status']);

        $boxes =
        $this->getSWAccount()
        ->getPackage()
        ->getOptionValue(Option::EMAIL_BOXES);

        $this->assertEquals('15', $boxes);

        $aliases =
        $this->getSWAccount()
        ->getPackage()
        ->getOptionValue(Option::EMAIL_ALIASES);

        $this->assertEquals('14', $aliases);
    }
}
