<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 02, 2015
 */

/**
 * Class BackupRestore_ConfigTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class BackupRestore_ConfigTest extends IWorx_Framework_TestCase
{
    /**
     * Test restoring a domain restores config values.
     */
    public function testDomainConfigRestore()
    {
        $this->deleteSWAccountIfNecessary();

        $SW = $this->getSWAccount();
        $SW->addSlaveDomain('backthatup.com');

        $MasterDomain = $SW->getMasterDomain();
        $MasterDomain->setConfig('a_test_value', 'test value 1');
        $MasterDomain->setConfig('another_test', [1, 2, 3]);

        $SecondaryDomain = new Domain('backthatup.com');
        $SecondaryDomain->setConfig('secondary_domain_test', 2.5);
        $SecondaryDomain->setConfig(
            'secondary_domain_test2',
            ['one' => 3, 'two' => 4]
        );

        $SW->setConfig('siteworx_config', 'value');

        $this->doBackupAndImport();

        $NewMasterDomain    = new Domain(self::SW_DOMAIN);
        $NewSecondaryDomain = new Domain('backthatup.com');
        $NewSW              = new SiteWorx(self::SW_DOMAIN);

        // Yeah, multiple assertions is bad, but this stupid test takes forever to set up

        $this->assertEquals(
            'test value 1',
            $NewMasterDomain->getConfig('a_test_value')
        );
        $this->assertEquals([1, 2, 3], $NewMasterDomain->getConfig('another_test'));

        $this->assertEquals(
            2.5,
            $NewSecondaryDomain->getConfig('secondary_domain_test')
        );
        $this->assertEquals(
            ['one' => 3, 'two' => 4],
            $NewSecondaryDomain->getConfig(
                'secondary_domain_test2',
                true
            )
        );

        $this->assertEquals('value', $NewSW->getConfig('siteworx_config'));
    }
}
