<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Oct 8, 2010
 */

/**
 * Test that horde identities are properly restored on backup/restore.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_HordeIdentitiesTest extends IWorx_Framework_TestCase
{

    /**
     * Tests to make sure horde identities are restored correctly.
     */
    public function testHordeIdentitiesRestore()
    {
        $this->deleteSWAccountIfNecessary();
        self::addSWAccountIfNecessary();

        $SW    = $this->getSWAccount();
        $input = array( 'username'         => 'test',
                    'password'         => 'pass123hahahohoxxxddaaa44',
                    'confirm_password' => 'pass123hahahohoxxxddaaa44',
                    'diskspacequota'   => '1234' );

        $Reply = IW::FC()->routeFromPHP('Ctrl_Siteworx_EmailBox', 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $test_data = 'a:1:{i:0;a:3:{s:2:"id";s:16:"Default Identity";' .
        's:8:"fullname";s:4:"test";s:9:"from_addr";s:13:"test@test.com";}}';

        $q_ident = q($test_data);

        $sql = "INSERT IGNORE INTO `horde_prefs` (
              `pref_uid`, `pref_scope`, `pref_name`, `pref_value` )
            VALUES ( 'test@addedbytest.com', 'horde', 'identities', {$q_ident} )";

        $db = IWorxDbUtil::db_connect(Ini::get(Ini::HORDE, 'dsn'));
        IWorxDbUtil::db_query($db, $sql);

        $this->doBackupAndImport('_removeHordeIdentitiy');

        $sql = "SELECT `pref_value` FROM `horde_prefs`
            WHERE `pref_uid` = 'test@addedbytest.com' AND
                  `pref_name` = 'identities'";

        $db    = IWorxDbUtil::db_connect(Ini::get(Ini::HORDE, 'dsn'));
        $value = IWorxDbUtil::db_getOne($db, $sql);

        $this->assertEquals($test_data, $value);
    }

    /**
     * Deletes the identitiy, cause it is not removed when the acccount is deleted.
     */
    protected function _removeHordeIdentitiy()
    {
        $sql = "DELETE FROM `horde_prefs`
            WHERE `pref_uid` = 'test@addedbytest.com' AND
                  `horde_prefs`.`pref_name` = 'identities' LIMIT 1";

        $db = IWorxDbUtil::db_connect(Ini::get(Ini::HORDE, 'dsn'));
        IWorxDbUtil::db_query($db, $sql);
    }
}
