<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Apr 3, 2010
 */

/**
 * Class that tests importing of secondary domains from a full backup works.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class BackupRestore_SecondaryDomainTest extends IWorx_Framework_TestCase
{

    /**
     * Test.
     */
    public function testSecondaryDomainsGetImported()
    {
        $this->deleteSWAccountIfNecessary();
        $this->addSWAccountIfNecessary();

        $this->_addSlaveDomain();
        $backup = $this->backupAccount(self::SW_DOMAIN);
        $this->deleteSWAccount();
        $this->_restoreBackup($backup);
        IWorxFileSys::rm($backup);

        $SW = new SiteWorx(self::SW_DOMAIN);
        $this->assertTrue($SW->exists());

        $slaves = $SW->getDomainNameList(Domain::SLAVE);
        $this->assertContains(self::SW_SECONDARY_DOMAIN, $slaves);
    }

    /**
     * Helper.
     */
    private function _addSlaveDomain()
    {
        $SW = $this->getSWAccount();

        $slaves = $SW->getDomainNameList(Domain::SLAVE);
        if (false === in_array(self::SW_SECONDARY_DOMAIN, $slaves)) {
            $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );
            $Reply = IW::FC()->routeFromPHP(
                'Ctrl_Siteworx_DomainsSlave',
                'add',
                $input,
                $SW
            );
            $this->assertReplySuccessful($Reply);
        }
    }

    /**
     * Helper.
     *
     * @param string $backup
     */
    private function _restoreBackup($backup)
    {
        $NW = self::getMasterNodeWorx();

        $ip        = self::_getFirstSharedIP();
        $db_server = $NW->getMySQLDriver()->getLocalServer();

        $input = array( 'archive'     => $backup,
                    'cp'          => 'siteworx',
                    'dbserver'    => $db_server->getNickname(),
                    'reseller_id' => $NW->getId(),
                    'ipaddr'      => $ip );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Import',
            'import',
            $input,
            $NW
        );

        $this->assertReplySuccessful($Reply);
    }
}
