<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Oct 18, 2012
 */

/**
 * Test that Secondary SW User login is properly restored on backup/restore.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_SecondarySWUserLoginTest extends IWorx_Framework_TestCase
{

    /**
     * Tests the Secondary SW User Login backup and restore.
     */
    public function testSecondarySWUserLoginRestore()
    {
        $domain = self::SW_DOMAIN;
        $email  = 'user@email.com';
        $password = 'rootroot';

        $this->deleteSWAccountIfNecessary();
        self::addSWAccountIfNecessary();

        //add secondary SW user
        $input = array( 'nickname'           => 'Secondary User',
                    'email'              => $email,
                    'password'           => $password,
                    'confirm_password'   => $password,
                    'language'           => 'en-us',
                    'menu_style'         => 'big',
                    'perms'              => array( 'LOGIN' ) );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_Users',
            'add',
            $input,
            $this->getSWAccount()
        );

        $this->assertReplySuccessful($Reply);

        //assert secondary SW user exists
        $user = $this->getSWAccount()->getUser($email);
        $this->assertNotNull($user);
        $this->_attemptLogin();

        //do backup
        $this->doBackupAndImport();

        //assert secondary SW user exists
        $user = $this->getSWAccount()->getUser($email);
        $this->assertNotNull($user);

        $this->_attemptLogin();
    }

    /**
     * Attempt to login as the user.
     */
    private function _attemptLogin()
    {
        $domain = self::SW_DOMAIN;
        $email  = 'user@email.com';
        $password = 'rootroot';

        //attempt login as user
        $api = 'http://localhost:2080/nodeworx/xmlrpc-api.php';
        $client = new Zend_XmlRpc_Client($api);
        $key = array( 'email'    => $email,
                  'domain'   => $domain,
                  'password' => $password );

        $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/overview',
                     'action'    => 'win',
                     'input'     => array() );

        $result  = $client->call('iworx.route', $params);
        $code    = ( $result['status'] === 0 ) ? Reply::OK : Reply::ERROR;
        $code    = ( $result['status'] === 401 ) ? Reply::PERM_FORBIDDEN : $code;
        $payload = $result['payload'];

        $Reply = new Reply($code, $payload);
        $this->assertReplySuccessful($Reply);
    }
}
