<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Nathan Roth <nroth@interworx.com>
 * @date       Oct 24, 2018
 */

/**
 * Class BackupRestore_ShellImportCollisionTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class BackupRestore_ShellImportCollisionTest extends IWorx_Framework_TestCase
{
    /**
     * Test restoring a domain restores shell accounts.
     */
    public function testShellImportCollision()
    {
        //initialize $generated_shell.
        $generated_shell='';
        $this->deleteSWAccountIfNecessary();
        $this->deleteSWAccountStatic('addedbytusername.com');
        $SW = $this->getSWAccount();
        $shell_users = ShellAccount::getSiteworxUsers();

        foreach ($shell_users as $shell_user) {
            if ($shell_user['user'] == $SW->getUniqname()) {
                $generated_shell = $shell_user['user'];
            }
        }

        //Make sure we find the generated shell.
        $this->assertNotEmpty($generated_shell);

        //for testing Nexcess side of things. Just confirm that the username changes after import.
        $generated_ssh_username = $SW->getMasterUser()->toAssocArray()['ssh_username'];

        // backup doesn't contain the ssh_username column.
        // Shell user is addedbyt
        // ssh_username doesn't exist.
        // From 5.1.52-1375 with no hotfixes.
        $backup = '/home/interworx/TESTS/_resources/addedbytusername.com-full-Oct.23.2018-14.58.30.tgz';
        IWorxFileSys::cp($backup, '/tmp/backup.tgz');

        // trigger import
        $ctrl = 'Ctrl_Nodeworx_Import';
        $action = 'importCommit';
        $input = array( 'domains' => 'addedbytusername.com',
                        'reseller_id' => '1',
                        'archive' => '/tmp/backup.tgz',
                        'cp' => 'siteworx',
                        'ipaddr' => self::_getFirstSharedIP(),
                        'dbserver' => 'localhost' );

        $NW = $this->getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP($ctrl, $action, $input, $NW);
        $this->assertReplySuccessful($Reply);

        $imported_account = new SiteWorx('addedbytusername.com');
        $imported_ssh_username = $imported_account->getMasterUser()->toAssocArray()['ssh_username'];


        //Tests that shell accounts tied to master SiteWorx user isn't the same.
        //$generated_ssh_username = addedbyt_0
        //$imported_ssh_username = addedby1_0
        $this->assertNotEquals($generated_ssh_username, $imported_ssh_username);
        $imported_shell = $imported_account->getUniqname();

        //$imported_shell = addedby1
        $this->assertTrue(ShellAccount::userExists($imported_shell));

        $imported_pass = ShellAccount::getShadow($imported_shell)['pass'];
        $generated_pass = ShellAccount::getShadow($generated_shell)['pass'];

        //$generated_shell = addedbyt
        //$imported_shell = addedby1
        $this->assertNotEquals($generated_shell, $imported_shell);

        //$generated_pass = !!
        //$imported_pass = $6$j5t9LosnkuV1/3Km$P.XJtagSvcJNjE.QlRykQntiMyBR54kR4LKaIOrChJIhmxXx8YLRJzWkMFLfPlXTrgPQVwMl3VKS8wGS/G.d50
        $this->assertNotEquals($generated_pass, $imported_pass);
    }
}
