<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Dec 12, 2010
 */

/**
 * Test class for Backup.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class BackupTest extends IWorx_Framework_TestCase
{

    /**
     * Setup.
     */
    public function setUp()
    {
        self::addSWAccountIfNecessary();

        IWorxBR::setLogToConsole(false);
    }

    /**
     * Sets the Temp Dir.
     */
    public function testSetTmpDir()
    {
        $Backup   = new Backup();
        $temp_dir = Ini::get(Ini::DIR, 'tmp');

        $this->assertTrue($Backup->setTempDir($temp_dir));
    }

    /**
     * Test setSiteWorx.
     *
     * @depends testSetTmpDir
     */
    public function testSetSiteWorx()
    {
        $Backup   = new Backup();
        $temp_dir = Ini::get(Ini::DIR, 'tmp');
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);

        $Backup->setTempDir($temp_dir);
        $Backup->setSiteWorx($SiteWorx);

        $this->assertEquals($SiteWorx->getId(), $Backup->getSiteWorx()->getId());
    }

    /**
     * Test hasEnoughFreeSpace.
     *
     * @depends testSetSiteWorx
     */
    public function testhasEnoughFreeSpace()
    {
        $Backup   = new Backup();
        $temp_dir = Ini::get(Ini::DIR, 'tmp');
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);
        $options  = IWorxBR::MYSQL_DATA | IWorxBR::EMAIL_DATA | IWorxBR::WEB_DATA;

        $Backup->setTempDir($temp_dir);
        $Backup->setSiteWorx($SiteWorx);
        $Backup->setBackupOptions($options);

        $this->assertTrue($Backup->hasEnoughFreeSpace());
    }
}
