<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2020 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Nov 12, 2020
 */

/**
 * Test restore.pex
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CLI_RestoreTest extends IWorx_Framework_TestCase
{

    /**
     * Setup.
     */
    public static function setUpBeforeClass()
    {
        self::deleteSWAccountIfNecessaryStatic();
        self::addSWAccountIfNecessary();
    }

    /**
     * Test.
     */
    public function testSimpleFileRestore()
    {
        $SW = $this->getSWAccount();
        $uniqname = $SW->getUniqname();
        $domroot = $SW->getDomainRoot();
        $domain = $SW->getMasterDomainName();
        $htmldir = "{$domroot}/html";
        $testfile = "{$htmldir}/test-file.txt";
        $testfile_contents = "this is a test, only a test\n";
        file_put_contents($testfile, $testfile_contents);
        chown($testfile, $uniqname);

        $tgz_path = $this->backupAccount(self::SW_DOMAIN);

        unlink($testfile);
        $this->assertFileNotExists($testfile, 'unlink testfile failed?');
        $restore = Ini::get(Ini::IWORX_BIN, 'restore');
        $cmd = "{$restore} --archive={$tgz_path} --domain={$domain} --path={$domain}/html/test-file.txt";
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals(0, $retval, implode("\n", $result));
        $this->assertFileContains($testfile, $testfile_contents);
    }

    /**
     * Test.
     */
    public function testSimpleDirectoryRestore()
    {
        $SW = $this->getSWAccount();
        $uniqname = $SW->getUniqname();
        $domroot = $SW->getDomainRoot();
        $domain = $SW->getMasterDomainName();
        $htmldir = "{$domroot}/html";
        $testfile = "{$htmldir}/test-file2.txt";
        $testfile_contents = "this is a test2, only a test\n";
        file_put_contents($testfile, $testfile_contents);
        chown($testfile, $uniqname);

        $tgz_path = $this->backupAccount(self::SW_DOMAIN);

        unlink($testfile);
        $this->assertFileNotExists($testfile, 'unlink testfile failed?');
        $restore = Ini::get(Ini::IWORX_BIN, 'restore');
        $cmd = "{$restore} --archive={$tgz_path} --domain={$domain} --path={$domain}/html";
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals(0, $retval, implode("\n", $result));
        $this->assertFileContains($testfile, $testfile_contents);
    }

    /**
     * Test.
     */
    public function testTwoFileRestore()
    {
        $SW = $this->getSWAccount();
        $uniqname = $SW->getUniqname();
        $domroot = $SW->getDomainRoot();
        $domain = $SW->getMasterDomainName();
        $htmldir = "{$domroot}/html";
        $testfile = "{$htmldir}/test-file3.txt";
        $testfile_contents = "this is a test3, only a test\n";
        file_put_contents($testfile, $testfile_contents);
        $testfile2 = "{$htmldir}/test-file4.txt";
        $testfile2_contents = "this is a test4, only a test\n";
        file_put_contents($testfile2, $testfile2_contents);
        chown($testfile, $uniqname);
        chown($testfile2, $uniqname);

        $tgz_path = $this->backupAccount(self::SW_DOMAIN);

        unlink($testfile);
        $this->assertFileNotExists($testfile, 'unlink testfile failed?');
        $restore = Ini::get(Ini::IWORX_BIN, 'restore');
        $cmd = "{$restore} --archive={$tgz_path} --domain={$domain} --path={$domain}/html/test-file3.txt --path={$domain}/html/test-file4.txt";
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals(0, $retval, implode("\n", $result));
        $this->assertFileContains($testfile, $testfile_contents);
        $this->assertFileContains($testfile2, $testfile2_contents);
    }

    /**
     * Test.
     */
    public function testInvalidFileRestore()
    {
        $SW = $this->getSWAccount();
        $domain = $SW->getMasterDomainName();

        $tgz_path = $this->backupAccount(self::SW_DOMAIN);

        $restore = Ini::get(Ini::IWORX_BIN, 'restore');
        $cmd = "{$restore} --archive={$tgz_path} --domain={$domain} --path={$domain}/html/missing-file.txt";
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertNotEquals(0, $retval, implode("\n", $result));
    }

    /**
     * Test.
     */
    public function testInvalidDomainRestore()
    {
        $SW = $this->getSWAccount();
        $domain = 'bad-domain.com';

        $tgz_path = $this->backupAccount(self::SW_DOMAIN);

        $restore = Ini::get(Ini::IWORX_BIN, 'restore');
        $cmd = "{$restore} --archive={$tgz_path} --domain={$domain} --path={$domain}/html/missing-file.txt";
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertNotEquals(0, $retval, implode("\n", $result));
    }

    /**
     * Test.
     */
    public function testSecondaryDomainFileRestore()
    {
        $SW = $this->getSWAccount();
        self::deleteSecondaryDomainIfNecessary();
        self::addSecondaryDomainIfNecessary();

        $uniqname = $SW->getUniqname();
        $domroot = $SW->getDomainRoot(self::SW_SECONDARY_DOMAIN);
        $domain = self::SW_SECONDARY_DOMAIN;
        $htmldir = "{$domroot}/html";
        $testfile = "{$htmldir}/test-file.txt";
        $testfile_contents = "this is a test, only a test\n";
        file_put_contents($testfile, $testfile_contents);
        chown($testfile, $uniqname);

        $tgz_path = $this->backupAccount(self::SW_DOMAIN);

        unlink($testfile);
        $this->assertFileNotExists($testfile, 'unlink testfile failed?');
        $restore = Ini::get(Ini::IWORX_BIN, 'restore');
        $cmd = "{$restore} --archive={$tgz_path} --domain={$domain} --path={$domain}/html/test-file.txt";
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals(0, $retval, implode("\n", $result));
        $this->assertFileContains($testfile, $testfile_contents);
    }

    /**
     * Test.
     */
    public function testDatabaseRestore()
    {
        $SW = $this->getSWAccount();
        $uniqname = $SW->getUniqname();
        $domroot = $SW->getDomainRoot();
        $domain = $SW->getMasterDomainName();
        $db_name = 'restore_test';
        $db_user = 'test';
        $db_user_pw = 'testing123';

        $input = array( 'name'        => $db_name,
                        'create_user' => '1',
                        'user'        => $db_user,
                        'password'    => $db_user_pw,
                        'confirm_password' => $db_user_pw );

        $Reply = IW::FC()->routeFromPHP('Ctrl_Siteworx_MysqlDb', 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);
        $this->assertTrue($SW->getMySQLDriver()->databaseExists($db_name));
        $cmd = "mysql -u {$uniqname}_{$db_user} -p{$db_user_pw} {$uniqname}_$db_name < /home/interworx/TESTS/_resources/sample.sql";
        exec($cmd, $result, $retval);
        $this->assertEquals(0, $retval, implode("\n", $result));

        $tgz_path = $this->backupAccount(self::SW_DOMAIN);

        $dsn = "mysqli://{$uniqname}_{$db_user}:{$db_user_pw}@unix(/var/lib/mysql/mysql.sock)/{$uniqname}_{$db_name}";
        $udb = IWorxDbUtil::db_connect($dsn);
        IWorxDbUtil::db_query($udb, 'DROP TABLE `tasks`');
        $this->assertFalse(IWorxDbUtil::db_tableExists($udb, 'tasks'));

        $restore = Ini::get(Ini::IWORX_BIN, 'restore');
        $cmd = "{$restore} --archive={$tgz_path} --domain={$domain} --database {$uniqname}_{$db_name}";
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals(0, $retval, implode("\n", $result));
        $this->assertTrue(IWorxDbUtil::db_tableExists($udb, 'tasks'));
    }
}
