<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Jon Daniel <jdaniel@interworx.com>
 */

/**
 * Test Suite for Ctrl_Nodeworx_ClusterLb.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_ClusterLbTest extends IWorx_Cluster_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_ClusterLb';

    /**
     * Set up some stuff before the test.
     */
    public static function setUpBeforeClass()
    {
        parent::setUpBeforeClass();

        if (TestConfig::get('cluster-tests') == '1') {
            self::_addAllNodesToCluster();
            self::_removeAllLBServices();
        }
    }

    /**
     * Tests the assigned vars.
     */
    public function testIndexAssignedVars()
    {
        $NodeWorx = self::getMasterNodeWorx();

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NodeWorx);
        $data  = $Reply->getData();

        $this->assertArrayHasKey('iw_nodes', $data);
        $this->assertArrayHasKey('iw_service_actions', $data);
        $this->assertArrayHasKey('iw_lb', $data);
        $this->assertArrayHasKey('iw_form', $data);

        $Sob_Form        = $data['iw_form']['startOnBoot'];
        $Ldirectord_Form = $data['iw_form']['settings'];

        $this->assertTrue($Sob_Form instanceof Form_NW_ClusterLb_StartOnBoot);
        $this->assertTrue($Ldirectord_Form instanceof Form_NW_ClusterLb_Settings);
    }

    /**
     * Test the settings action.
     */
    public function testSettingsAction()
    {
        $NodeWorx = self::getMasterNodeWorx();

        $Config         = new Ldirectord_Config();
        $old_email      = $Config->getEmailAlert();
        $expected_email = 'test@interworx.com';

        $input = array( 'emailalert'       => $expected_email,
                    'emailalertstatus' => array() );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'settings',
            $input,
            $NodeWorx
        );

        $this->assertReplySuccessful($Reply);

        $Config       = new Ldirectord_Config();
        $actual_email = $Config->getEmailAlert();

        $Config->setEmailAlert($old_email);
        $Config->write();

        $this->assertEquals($expected_email, $actual_email);
    }

    /**
     * Tests the add service action.
     */
    public function testAddService()
    {
        $NodeWorx = self::getMasterNodeWorx();
        $input    = array( 'service_port' => '443',
                       'virtual_ip'   => '0.0.0.0',
                       'policy'       => 'rr',
                       'persistent'   => '0' );

        $nodes = $this->_getNodes();
        foreach ($nodes as $node) {
            $input["weight_{$node}"] = '1';
        }

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'addService',
            $input,
            $NodeWorx
        );

        //    sleep( 4 );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the start on boot action.
     */
    public function testStartonbootOnAction()
    {
        $initial  = IW::DM()->getLoadBalancer()->isStartedOnBoot($reset = true);
        $NodeWorx = self::getMasterNodeWorx();
        $input    = array( 'startonboot' => '1' );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'startOnBoot',
            $input,
            $NodeWorx
        );

        $this->assertReplySuccessful($Reply);
        $this->assertTrue(IW::DM()->getLoadBalancer()->isStartedOnBoot($reset = true));

        IW::DM()->getLoadBalancer()->startOnBoot($initial);
    }

    /**
     * Tests the start on boot action.
     */
    public function testStartonbootOffAction()
    {
        $initial  = IW::DM()->getLoadBalancer()->isStartedOnBoot($reset = true);
        $NodeWorx = self::getMasterNodeWorx();
        $input    = array( 'startonboot' => '0' );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'startOnBoot',
            $input,
            $NodeWorx
        );

        $this->assertReplySuccessful($Reply);
        $this->assertTrue(!IW::DM()->getLoadBalancer()->isStartedOnBoot(true));

        IW::DM()->getLoadBalancer()->startOnBoot($initial);
    }

    /**
     * Tests the edit service action.
     */
    public function testEditService()
    {
        $NodeWorx = self::getMasterNodeWorx();
        $lb_info  = IW::DM()->getLoadBalancer()->getLBInfo($reset = true);

        $first    = array_shift($lb_info['services']);
        $input    = array( 'service'    => "{$first['info']['vip']}:443",
                       'policy'     => 'wrr',
                       'persistent' => '300' );

        $nodes = $this->_getNodes();
        foreach ($nodes as $node) {
            $input["weight_{$node}"] = '2';
        }

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'editService',
            $input,
            $NodeWorx
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the delete service action.
     */
    public function testDeleteService()
    {
        $NodeWorx = self::getMasterNodeWorx();
        $lb_info  = IW::DM()->getLoadBalancer()->getLBInfo($reset = true);
        $services = array_keys($lb_info['services']);
        $input    = array( 'services' => $services );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'deleteService',
            $input,
            $NodeWorx
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Get all nodes in the cluster.
     *
     * @return array
     */
    private function _getNodes()
    {
        $DsNodes = new DataSource_LoadBalance_Nodes();
        return $DsNodes->getValues();
    }

    /**
     * Remove all services from the cluster.
     */
    private static function _removeAllLBServices()
    {
        $NodeWorx = self::getMasterNodeWorx();
        $Ds       = new DataSource_CurrentlyLoadBalancedServices();
        $values   = $Ds->getValues();

        if (!empty($values)) {
            foreach ($values as $service) {
                list( $vip, $port ) = explode(':', $service);
                IW::DM()->getLoadBalancer()->deleteService($port, $vip);
            }
        }

        $Ldirectord = new Ldirectord();
        $Ldirectord->sync();
    }
}
