<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jan 20, 2020
 */
require_once(__DIR__ . '/Map.php');

/**
 * Tests for the Mysql Map controller.
 *
 * @package Interworx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_MapDbTest extends Ctrl_Nodeworx_Map
{
    /**
     * Make sure unmapped dbs are added before each test is run.
     */
    public function setUp()
    {
        parent::setUp();
        $this->addUnmappedDbs();
    }

    /**
     * Cleanup after each test.
     */
    public function tearDown()
    {
        $this->deleteDbs();
    }


    /**
     * Test that operation fails without sw id.
     */
    public function testMapDbActionShouldFailWithoutDomain()
    {
        $invalidDomain = '';
        $db = $this->getUnmappedDb();
        $Reply = $this->_getReplyMapDatabaseAction($invalidDomain, $db);

        $this->assertReplyNotSuccessful($Reply, "mapDb should fail when id is empty string.");
    }

    /**
     * Test that operation fails with invalid args.
     */
    public function testMapDbActionShouldFailWithInvalidDomain()
    {
        $invalidDomain = 'veryinvalidid';
        $db = $this->getUnmappedDb();
        $Reply = $this->_getReplyMapDatabaseAction($invalidDomain, $db);

        $this->assertReplyNotSuccessful($Reply, "Map should fail with invalid sw id.");
    }

    /**
     * Test that operation fails with invalid args.
     */
    public function testMapDbActionShouldFailWithoutDb()
    {
        $db = '';
        $SW = $this->getSWAccount();
        $Reply = $this->_getReplyMapDatabaseAction($SW->getMasterDomainName(), $db);

        $this->assertReplyNotSuccessful($Reply, "Map should fail with empty string as db.");
    }

    /**
     * Test that operation fails with invalid args.
     */
    public function testMapDbActionShouldFailWithInvalidDb()
    {
        $db = 'faksfuireuivnuhysbjkfgb';
        $SW = $this->getSWAccount();
        $Reply = $this->_getReplyMapDatabaseAction($SW->getMasterDomainName(), $db);

        $this->assertReplyNotSuccessful($Reply, "Map should fail with non-existing db.");
    }

    /**
     * Test successful reply with valid arguments &
     * that databases are mapped accordingly.
     */
    public function testMapDbActionShouldMapDbsWithValidArgs()
    {
        $SW = $this->getSWAccount();
        $db = $this->getUnmappedDb();
        $Reply = $this->_getReplyMapDatabaseAction($SW->getMasterDomainName(), $db);

        $this->assertReplySuccessful($Reply);

        $userDbs = SiteworxDbMapper::getInstance()->getDatabases($SW);
        $this->assertContains($db, $userDbs);
    }

    /**
     * get Reply object from mapDb action.
     *
     * @param string $masterDomain
     * @param string $db
     * @return Reply
     */
    private function _getReplyMapDatabaseAction($masterDomain, $db)
    {
        $NW = self::getMasterNodeWorx();
        $nickname = MySQL::getLocalServer()->getNickname();

        $input = array(
            'server' => $nickname,
            'siteworx_account' => $masterDomain,
            'database' => $db
        );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'mapDatabase',
            $input,
            $NW
        );

        return $Reply;
    }
}
