<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Nov 11, 2009
 */

/**
 * Test Suite for Ctrl_Nodeworx_DnsZone.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_DnsRecordTest extends IWorx_Framework_TestCase
{

    const CTRL          = 'Ctrl_Nodeworx_DnsRecord';
    const DUMMY_IP      = '1.2.3.4';
    const DUMMY_IP2     = '1.2.3.3';
    const DUMMY_CONTACT = 'nobody@nowhere.com';

    /**
     * Setup.
     */
    public static function setUpBeforeClass()
    {
        //parent::setup();
        self::addSWAccountIfNecessary(); // inits needed sw account

        $Zone = new Dns_Zone('domain = ' . q(self::SW_DOMAIN));
        $Zone->delete();

        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain' => self::SW_DOMAIN,
                    'ipaddr' => self::DUMMY_IP,
                    'contact' => self::DUMMY_CONTACT );
        IW::FC()->routeFromPHP('Ctrl_Nodeworx_DnsZone', 'add', $input, $NW);
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodAddressRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( "a.{$domain}", 'A', self::DUMMY_IP, 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function badAddressRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( 'a', 'A', self::DUMMY_IP, 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodCnameRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( "cname.{$domain}", 'CNAME', 'apples.com', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function badCnameRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( 'cname', 'CNAME', 'apples.com', 43200 ),
                  array( 'some.other.com', 'CNAME', 'apples.com', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodNsRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( $domain, 'NS', 'ns5.unittests.com', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function badNsRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( $domain, 'NS', '5.4.3.2', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodMxRecordData()
    {
        return array( array( 'mail.interworx.com', 'MX', 10, 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function badMxRecordData()
    {
        return array( array( 'mail.interworx.com', 'MX', -50, 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodPtrRecordData()
    {
        return array( array( '4.3.2.1.in-addr.arpa',
                         'PTR',
                         'oh.' . self::SW_DOMAIN,
                         43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function badPtrRecordData()
    {
        return array( array( 'no', 'PTR', 'oh.my.com', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodTxtRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( $domain, 'TXT', 'whatEVER goes here', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function badTxtRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( -5, 'TXT', 'whatEVER goes here', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodSpfRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( $domain, 'SPF', 'stuff', 43200 ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function badSpfRecordData()
    {
        $domain = self::SW_DOMAIN;
        return array( array( 0, 'SPF', 'stuff', 43200 ),
                  );
    }

    /**
     * Add record helper.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @return Reply
     */
    private function _addRecord($host, $type, $target, $ttl)
    {
        $NW      = self::getMasterNodeWorx();
        $Zone    = new Dns_Zone('domain = ' . q(self::SW_DOMAIN));
        $zone_id = $Zone->getId();
        $input   = array( 'zone_id' => $zone_id,
                      'host'    => $host,
                      'type'    => $type,
                      'target'  => $target,
                      'ttl'     => $ttl );
        return IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider goodAddressRecordData()
     */
    public function testAddGoodAddressRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('A', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider goodCnameRecordData()
     */
    public function testAddGoodCnameRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('CNAME', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider goodNsRecordData()
     */
    public function testAddGoodNsRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('NS', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider goodMxRecordData()
     */
    public function testMxGoodCnameRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('MX', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider goodPtrRecordData()
     */
    public function testAddGoodPtrRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('PTR', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider goodTxtRecordData()
     */
    public function testAddGoodTxtRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('TXT', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider goodSpfRecordData()
     */
    public function testAddGoodSpfRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('SPF', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider badAddressRecordData()
     */
    public function testAddBadAddressRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('A', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider badCnameRecordData()
     */
    public function testAddBadCnameRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('CNAME', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider badNsRecordData()
     */
    public function testAddBadNsRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('NS', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider badMxRecordData()
     */
    public function testMxBadCnameRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('MX', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider badPtrRecordData()
     */
    public function testAddBadPtrRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('PTR', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider badTxtRecordData()
     */
    public function testAddBadTxtRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('TXT', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider badSpfRecordData()
     */
    public function testAddBadSpfRecords($host, $type, $target, $ttl)
    {
        $this->assertEquals('SPF', $type, 'wrong type dumbass');
        $Reply = $this->_addRecord($host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Very basic test of editing records.
     */
    public function testEditActionReturnsTrue()
    {
        $Zone = new Dns_Zone('domain = ' . q(self::SW_DOMAIN));

        $A_Records = $Zone->getRecordObjects(Dns_Record::ADDRESS);

        if (count($A_Records) === 0) {
            $this->fail('There were somehow no A Records to test with');
        }

        /* @var $Record Dns_Record_A */
        $Record = $A_Records[0];

        $input           = $Record->toAssocArray();
        $input['target'] = self::DUMMY_IP2;

        $NW = self::getMasterNodeWorx();

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Verifies the previous test.
     *
     * @depend testEditActionReturnsTrue
     */
    public function testEditActionSeemedToWork()
    {
        $Zone = new Dns_Zone('domain = ' . q(self::SW_DOMAIN));

        $A_Records = $Zone->getRecordObjects(Dns_Record::ADDRESS);

        if (count($A_Records) === 0) {
            $this->fail('There were somehow no A Records to test with');
        }

        /* @var $Record Dns_Record_A */
        $Record = $A_Records[0];

        $this->assertEquals(self::DUMMY_IP2, $Record->getIpAddress());
    }

    /**
     * Confirm editing an existing cname with no changes works as expected.
     */
    public function testCnameEdit()
    {
        $Zone = new Dns_Zone('domain = ' . q(self::SW_DOMAIN));
        $CNAME_Records = $Zone->getRecordObjects(Dns_Record::CNAME);
        if (count($CNAME_Records) === 0) {
            $this->fail('There were somehow no CNAME Records to test with');
        }
        /* @var $Record Dns_Record_CNAME */
        $Record = $CNAME_Records[0];
        $input  = $Record->toAssocArray();
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'editCNAME', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Confirm adding a cname with collisions fails.
     */
    public function testCnameDuplicateAddFails()
    {
        $Zone = new Dns_Zone('domain = ' . q(self::SW_DOMAIN));
        $CNAME_Records = $Zone->getRecordObjects(Dns_Record::CNAME);
        if (count($CNAME_Records) === 0) {
            $this->fail('There were somehow no CNAME Records to test with');
        }
        /* @var $Record Dns_Record_CNAME */
        $Record = $CNAME_Records[0];
        $input  = $Record->toAssocArray();
        $input['alias'] = 'www.google.com';
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'addCNAME', $input, $NW);
        $this->assertReplyNotSuccessful($Reply, 'Duplicate CNAME allowed');
    }

    /**
     * Tests deleting a single record.
     */
    public function testDeleteActionReturnsTrue()
    {
        $Zone = new Dns_Zone('domain = ' . q(self::SW_DOMAIN));

        $A_Records = $Zone->getRecordObjects(Dns_Record::ADDRESS);
        $old_count = count($A_Records);

        if ($old_count === 0) {
            $this->fail('There were somehow no A Records to test with');
        }

        /* @var $Record Dns_Record_A */
        $Record = $A_Records[0];

        $input = array( 'record_id' => $Record->getRecordId() );

        $NW = self::getMasterNodeWorx();

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $NW);

        $this->assertReplySuccessful($Reply);
        $this->assertCount(
            $old_count - 1,
            $Zone->getRecordObjects(Dns_Record::ADDRESS)
        );
    }
}
