<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Dns.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_DnsTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_Dns';

    /**
     * Test assigned vars of index action.
     */
    public function testIndexActionAssignedVars()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NW);
        $result = $Reply->getData();
        $this->assertArrayHasKey('iw_dns', $result);
        $this->assertArrayHasKey('iw_dns_settings', $result);

        $forms = $result['iw_form'];

        $this->assertArrayHasKey('servers', $forms);
        $class_name = 'Form_NW_Dns_Servers';
        $this->assertTrue($forms['servers'] instanceof $class_name);

        $this->assertArrayHasKey('zone_defaults', $forms);
        $class_name = 'Form_NW_DnsZone_Defaults';
        $this->assertTrue($forms['zone_defaults'] instanceof $class_name);
    }

    /**
     * Test.
     */
    public function testActiveServerAction()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'activeServers', array(), $NW);
        $result = $Reply->getData();
        $this->assertInternalType('array', $result);
        $this->assertEquals('##LG_NOTHING_TO_DO##', $result[ 0 ]);
    }

    /**
     * Test.
     */
    public function testZoneDefaultsAction()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'zoneDefaults', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testDefaultServersAction()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'defaultServers', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }
}
