<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_DnsZone.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_DnsZoneTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_DnsZone';

    const DNS_ZONE        = 'zone-added-by-test.com';
    const DUMMY_IP        = '1.2.3.4';
    const DUMMY_CONTACT   = 'nobody@nowhere.com';
    const DUMMY_CONTACT_2 = 'somebodyelse@nowhere.com';

    /**
     * Serial bump data provider.
     *
     * @return array
     */
    public function serialBumpData()
    {
        $today = date('Ymd');
        $yesterday = date('Ymd', IW::now() - (60*60*24));
        return array( array( 'unix', '', IW::now() ),
                  array( 'unix', 1234, IW::now() ),
                  array( 'ymd', '', "{$today}00" ),
                  array( 'ymd', "{$today}00", "{$today}01" ),
                  array( 'ymd', "{$today}01", "{$today}02" ),
                  array( 'ymd', "{$today}09", "{$today}10" ),
                  array( 'ymd', "{$today}10", "{$today}11" ),
                  array( 'ymd', "{$today}19", "{$today}20" ),
                  array( 'ymd', "{$today}29", "{$today}30" ),
                  array( 'ymd', "{$today}39", "{$today}40" ),
                  array( 'ymd', "{$today}49", "{$today}50" ),
                  array( 'ymd', "{$today}59", "{$today}60" ),
                  array( 'ymd', "{$today}69", "{$today}70" ),
                  array( 'ymd', "{$today}79", "{$today}80" ),
                  array( 'ymd', "{$today}98", "{$today}99" ),
                  array( 'ymd', "{$today}99", "{$today}99" ),
                  array( 'ymd', "{$today}ZZZZZZZZZZ", "{$today}01" ),
                  array( 'ymd', "{$yesterday}00", "{$today}00" ),
                  array( 'ymd', "{$yesterday}98", "{$today}00" ),
                  );
    }

    /**
     * Test.
     *
     * @param string $mode
     * @param string $serial
     * @param string $expected
     * @dataProvider serialBumpData()
     */
    public function testSerialBumping($mode, $serial, $expected)
    {
        $this->assertEquals($expected, Dns_Zone::bumpSerial($serial, $mode));
    }

    /**
     * Tests to make sure resellers cannot add a DNS Zone.
     */
    public function testResellerCannotAdd()
    {
        self::addResellerIfNecessary();
        $NW = self::getResellerNodeWorx();

        $input  = array( 'domain' => self::DNS_ZONE,
                     'ipaddr' => self::DUMMY_IP,
                     'contact' => self::DUMMY_CONTACT );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
        $msg   = 'was able to add a dns zone as a reseller :(';

        $this->assertReplyNotSuccessful($Reply, $msg);
    }

    /**
     * Test.
     */
    public function testAddActionReturnsTrue()
    {
        $NW     = self::getMasterNodeWorx();
        $input  = array( 'domain' => self::DNS_ZONE,
                     'ipaddr' => self::DUMMY_IP );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @depends testAddActionReturnsTrue
     */
    public function testSuspendActionReturnsTrue()
    {
        $NW = self::getMasterNodeWorx();

        $where   = 'domain = ' . q(self::DNS_ZONE);
        $Zone    = new Dns_Zone($where);
        $zone_id = $Zone->getId();

        $input = array( 'zone_id' => $zone_id );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'suspend', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @depends testSuspendActionReturnsTrue
     */
    public function testUnsuspendActionReturnsTrue()
    {
        $NW = self::getMasterNodeWorx();

        $where   = 'domain = ' . q(self::DNS_ZONE);
        $Zone    = new Dns_Zone($where);
        $zone_id = $Zone->getId();

        $input = array( 'zone_id' => $zone_id );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'unsuspend', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @depends testAddActionReturnsTrue
     */
    public function testEditActionReturnsTrue()
    {
        $this->assertTrue(Dns_Zone::exists('domain = ' . q(self::DNS_ZONE)));
        $Zone    = new Dns_Zone('domain = ' . q(self::DNS_ZONE));
        $zone_id = $Zone->getId();
        $NW      = self::getMasterNodeWorx();
        $input   = array( 'zone_id' => $zone_id,
                      'contact' => self::DUMMY_CONTACT_2 );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests adding a resource record.
     *
     * @depends testAddActionReturnsTrue
     */
    public function testAddResourceRecord()
    {
        $this->assertTrue(Dns_Zone::exists('domain = ' . q(self::DNS_ZONE)));
        $Zone = new Dns_Zone('domain = ' . q(self::DNS_ZONE));

        $node  = self::DNS_ZONE;
        $type  = Dns_Record::TXT;
        $ttl   = 42300;
        $rdata = 'This is a dummy text record, added for fun';

        $expected_count = count($Zone->getRecords()) + 1;

        $this->assertTrue($Zone->addResourceRecord($node, $type, $ttl, $rdata));
        $this->assertEquals($expected_count, count($Zone->getRecords()));
    }

    /**
     * Tests adding a RR with bad data.
     *
     * @depends testAddResourceRecord
     */
    public function testAddResourceRecordBadData()
    {
        $this->assertTrue(Dns_Zone::exists('domain = ' . q(self::DNS_ZONE)));
        $Zone = new Dns_Zone('domain = ' . q(self::DNS_ZONE));

        $node  = self::DNS_ZONE;
        $type  = Dns_Record::MX;
        $ttl   = 42300;
        $rdata = 'missing.preference.com';

        $expected_count = count($Zone->getRecords());

        $this->assertFalse($Zone->addResourceRecord($node, $type, $ttl, $rdata));
        $this->assertNotEmpty($Zone->getErrors());
        $this->assertEquals($expected_count, count($Zone->getRecords()));
    }

    /**
     * Test.
     */
    public function testDeleteActionReturnsTrue()
    {
        $this->assertTrue(Dns_Zone::exists('domain = ' . q(self::DNS_ZONE)));
        $Zone    = new Dns_Zone('domain = ' . q(self::DNS_ZONE));
        $zone_id = $Zone->getId();
        $NW      = self::getMasterNodeWorx();
        $input   = array( 'zone_id' => $zone_id );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testEditActionErrorsWithNoInput()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'edit', array(), $NW);
        $result = $Reply->getData();
        $this->assertIsFormErrorArray($result);
    }

    /**
     * Test.
     */
    public function testDeleteActionErrorsWithNoInput()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'delete', array(), $NW);
        $result = $Reply->getData();
        $this->assertIsFormErrorArray($result);
    }

    /**
     * Test.
     */
    public function testAddActionErrorsWithNoInput()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'add', array(), $NW);
        $result = $Reply->getData();
        $this->assertIsFormErrorArray($result);
    }
}
