<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Brandon Locke <blocke@liquidweb.com>
 * @date    Mar 08, 2021
 */

/**
 * Tests the NodeWorx MailDovecot Controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_MailDovecotTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Nodeworx_MailDovecot';

    public function testEmptyChainPemFile()
    {
        $this->deleteSWAccountIfNecessary();
        $SW = $this->getSWAccount();
        $this->_addSslKey($SW);
        $this->_addCsr($SW);
        $this->_addCert($SW);
        IWorxExec::quick_exec('nodeworx -u -n -c MailDovecot -a syncDovecotSniHosts');
        $this->_removeChainPem($SW);
        IWorxExec::quick_exec('nodeworx -u -n -c MailDovecot -a syncDovecotSniHosts');
        $Dovecot = new Service_Dovecot();
        $this->assertTrue($Dovecot->isRunning());
    }

    private function _addSslKey($SW)
    {
        IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslKey',
            'generate',
            array( 'key_length' => '2048' ),
            $SW
        );
    }

    private function _addCsr($SW)
    {
        $input = array( 'countryName'            => 'US',
                    'stateOrProvinceName'    => 'Pennsylvania',
                    'localityName'           => 'Pittsburgh',
                    'organizationName'       => 'InterWorx, LLC',
                    'organizationalUnitName' => 'SE',
                    'commonName'             => self::SW_DOMAIN,
                    'emailAddress'           => self::SW_EMAIL );
        IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslCsr',
            'generate',
            $input,
            $SW
        );
    }

    private function _addCert($SW)
    {
        IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslCrt',
            'generate',
            array(),
            $SW
        );
    }

    private function _removeChainPem($SW)
    {
       $Ssldir = $SW->getHTTPDriver()->getSSLDriver()->_ssldir;
       $domain = $SW->getMasterDomainName();
       unlink("$Ssldir/$domain.chain.pem");
    }
}