<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_MailMda.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_MailMdaTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_MailMda';

    /**
     * Test assigned vars of index action.
     */
    public function testIndexActionAssignedVars()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NW);
        $result = $Reply->getData();
        $this->assertArrayHasKey('iw_form', $result);

        $forms = $result['iw_form'];

        if (IW::DM()->getImail()->isDovecot()) {
            $this->assertArrayHasKey('dovecot_sob', $forms);
            $class_name = 'Form_NW_MailMda_StartOnBootDovecot';
            $this->assertTrue($forms['dovecot_sob'] instanceof $class_name);

            $this->assertArrayHasKey('dovecot_info', $forms);
            $class_name = 'Form_NW_MailMda_InfoDovecot';
            $this->assertTrue($forms['dovecot_info'] instanceof $class_name);
        } else {
            $this->assertArrayHasKey('pop3_sob', $forms);
            $class_name = 'Form_NW_MailMda_StartOnBootPOP3';
            $this->assertTrue($forms['pop3_sob'] instanceof $class_name);

            $this->assertArrayHasKey('spop3_sob', $forms);
            $class_name = 'Form_NW_MailMda_StartOnBootSPOP3';
            $this->assertTrue($forms['spop3_sob'] instanceof $class_name);

            $this->assertArrayHasKey('imap_sob', $forms);
            $class_name = 'Form_NW_MailMda_StartOnBootIMAP4';
            $this->assertTrue($forms['imap_sob'] instanceof $class_name);

            $this->assertArrayHasKey('simap_sob', $forms);
            $class_name = 'Form_NW_MailMda_StartOnBootSIMAP4';
            $this->assertTrue($forms['simap_sob'] instanceof $class_name);

            $this->assertArrayHasKey('pop3_info', $forms);
            $class_name = 'Form_NW_MailMda_InfoPOP3';
            $this->assertTrue($forms['pop3_info'] instanceof $class_name);

            $this->assertArrayHasKey('imap_info', $forms);
            $class_name = 'Form_NW_MailMda_InfoIMAP4';
            $this->assertTrue($forms['imap_info'] instanceof $class_name);
        }
    }

    /**
     * Test.
     */
    public function testUpdatePop3ActionReturnsTrue()
    {
        if (IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('dovecot active, test skipped');
        }

        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updatePop3', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testUpdateImapActionReturnsTrue()
    {
        if (IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('dovecot active, test skipped');
        }

        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateImap', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testStartOnBootPOP3ActionReturnsTrue()
    {
        if (IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('dovecot active, test skipped');
        }

        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'startOnBootPOP3', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testStartOnBootSPOP3ActionReturnsTrue()
    {
        if (IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('dovecot active, test skipped');
        }

        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'startOnBootSPOP3', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testStartOnBootIMAP4ActionReturnsTrue()
    {
        if (IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('dovecot active, test skipped');
        }

        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'startOnBootIMAP4', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testStartOnBootSIMAP4ActionReturnsTrue()
    {
        if (IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('dovecot active, test skipped');
        }

        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'startOnBootSIMAP4',
            array(),
            $NW
        );
        $this->assertReplySuccessful($Reply);
    }
}
