<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Nathan Roth <nroth@interworx.com>
 * @date       Mar 09, 2020
 */

/**
 * Test Suite for Ctrl_Nodeworx_MailMsa.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_MailMsaTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_MailMsa';

    /**
     * Test assigned vars of index action.
     */
    public function testIndexActionAssignedVars()
    {
        if (! IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('Running in legacy mode, skip.');
        }

        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NW);
        $result = $Reply->getData();
        $this->assertArrayHasKey('iw_form', $result);

        $forms = $result['iw_form'];

        if (IW::DM()->getImail()->dovecot()->isSubmission()) {
            $this->assertArrayHasKey('submission', $forms);
            $class_name = 'Form_NW_MailMsa_Dovecot';
            $this->assertTrue($forms['submission'] instanceof $class_name);
        } else {
            $this->assertArrayHasKey('convert_submission', $forms);
            $class_name = 'Form_NW_MailMsa_Dovecot';
            $this->assertTrue($forms['convert_submission'] instanceof $class_name);
        }
    }

    /**
     * Test.
     */
    public function testEnableSubmission()
    {
        if (! IW::DM()->getImail()->isDovecot()) {
            $this->markTestSkipped('Dovecot not enabled, skip.');
        }

        IWorxExec::quick_exec('nodeworx -u -n -c MailMsa -a convertToSubmission');
        $path = '/etc/dovecot/conf.d/95-iworx-submission.conf';
        $conf = file_get_contents($path);
        $match = preg_match("/587/", $conf);
        $this->assertEquals(1, $match);
    }

    /**
     * Test.
     *
     */
    public function testSetActiveSubmission()
    {
        if (! IW::DM()->getImail()->isDovecot() || ! IW::DM()->getImail()->dovecot()->isSubmission()) {
            $this->markTestSkipped('MSA server not enabled, skip.');
        }

        $proto = ['submission-legacy'];
        $max_mail = 50;

        IW::DM()->getImail()->dovecot()->setActiveSubmission($proto, $max_mail);

        $path = '/etc/dovecot/conf.d/95-iworx-submission.conf';
        $conf = file_get_contents($path);
        $match = preg_match("/465/", $conf);
        $this->assertEquals(1, $match);

        $match = preg_match("/587/", $conf);
        $this->assertEquals(0, $match);
    }
}
