<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_MailMta.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_MailMtaTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_MailMta';

    /**
     * Test assigned vars of index action.
     */
    public function testIndexActionAssignedVars()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NW);
        $result = $Reply->getData();
        $this->assertArrayHasKey('iw_mta', $result);
        $this->assertArrayHasKey('iw_form', $result);

        $forms = $result['iw_form'];

        $this->assertArrayHasKey('alt', $forms);
        $class_name = 'Form_NW_MailMta_AltSmtpSettings';
        $this->assertTrue($forms['alt'] instanceof $class_name);

        $this->assertArrayHasKey('rrd', $forms);
        $class_name = 'Form_NW_MailMta_RrdGraph';
        $this->assertTrue($forms['rrd'] instanceof $class_name);

        $this->assertArrayHasKey('opts', $forms);
        $class_name = 'Form_NW_MailMta_ServerOptions';
        $this->assertTrue($forms['opts'] instanceof $class_name);

        $this->assertArrayHasKey('bounce', $forms);
        $class_name = 'Form_NW_MailMta_BounceOptions';
        $this->assertTrue($forms['bounce'] instanceof $class_name);

        $this->assertArrayHasKey('in', $forms);
        $class_name = 'Form_NW_MailMta_InboundSmtpOptions';
        $this->assertTrue($forms['in'] instanceof $class_name);

        $this->assertArrayHasKey('out', $forms);
        $class_name = 'Form_NW_MailMta_OutboundSmtpOptions';
        $this->assertTrue($forms['out'] instanceof $class_name);
    }

    /**
     * Test.
     */
    public function testUpdateBounceActionReturnsTrue()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateBounce', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Gets the double bounce input.
     *
     * @return array
     */
    private function _getDoubleBounceInput()
    {
        $mta = DriverManager::getDefaultDriver('omail')->getServerInfo();

        $bouncefrom    = $mta['controls']['bouncefrom'][ 0 ];
        $bouncehst     = $mta['controls']['bouncehost'][ 0 ];
        $doublebounce  = $mta['controls']['doublebounce'];
        $dblbounceto   = $mta['controls']['doublebounceto'][ 0 ];
        $dblbouncehost = $mta['controls']['doublebouncehost'][ 0 ];

        $input = array( 'bounce_from' => $bouncefrom,
                    'bounce_host' => $bouncehst,
                    'double_bounce_messages' => $doublebounce,
                    'double_bounce_to' => $dblbounceto,
                    'double_bounce_host' => $dblbouncehost );

        return $input;
    }

    /**
     * Tests the enabling of double bounce.
     */
    public function testDoubleBounceEnable()
    {
        $NW                              = self::getMasterNodeWorx();
        $input                           = $this->_getDoubleBounceInput();
        $input['double_bounce_messages'] = '1';
        $input['double_bounce_to']       = 'postmaster';

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateBounce', $input, $NW);
        $this->assertReplySuccessful($Reply);

        $mta = DriverManager::getDefaultDriver('omail')->getServerInfo(true);

        $this->assertEquals('1', $mta['controls']['doublebounce']);
        $this->assertEquals('postmaster', $mta['controls']['doublebounceto'][0]);
    }

    /**
     * Tests that if the user sets the double bounce to disabled but sets the.
     * doublebounceto option to doublebounce that it will actually disable.
     */
    public function testDoubleBounceEnbableAlwaysEnables()
    {
        $NW                              = self::getMasterNodeWorx();
        $input                           = $this->_getDoubleBounceInput();
        $input['double_bounce_messages'] = '1';
        $input['double_bounce_to']       = 'doublebounce';

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateBounce', $input, $NW);
        $this->assertReplySuccessful($Reply);

        $mta = DriverManager::getDefaultDriver('omail')->getServerInfo(true);

        $this->assertEquals('1', $mta['controls']['doublebounce']);
        $this->assertNotEquals('doublebounce', $mta['controls']['doublebounceto'][0]);
    }

    /**
     * Tests the disabling of double bounce.
     */
    public function testDoubleBounceDisable()
    {
        $NW                              = self::getMasterNodeWorx();
        $input                           = $this->_getDoubleBounceInput();
        $input['double_bounce_messages'] = 0;

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateBounce', $input, $NW);
        $this->assertReplySuccessful($Reply);

        $mta = DriverManager::getDefaultDriver('omail')->getServerInfo(true);

        $this->assertEquals(0, $mta['controls']['doublebounce']);
        $this->assertEquals('doublebounce', $mta['controls']['doublebounceto'][0]);
    }

    /**
     * Test.
     */
    public function testUpdateGeneralActionReturnsTrue()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateGeneral', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testUpdateInboundActionReturnsTrue()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateInbound', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testUpdateOutboundActionReturnsTrue()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateOutbound', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testUpdateRrdActionReturnsTrue()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'rrd' => 0 );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateRrd', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function updateSmtp2portActionReturnsTrue()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'updateSmtp2port', array(), $NW);
        $this->assertReplySuccessful($Reply);
    }
}
