<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Apr 1, 2011
 */

/**
 * Tests for the Mysql Remote controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_MysqlRemoteTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_MysqlRemote';

    /**
     * Tests the index assigned vars.
     */
    public function testIndexAssignedVars()
    {
        $NodeWorx = self::getMasterNodeWorx();

        $input = [];
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'index',
            $input,
            $NodeWorx
        );

        $result = $Reply->getData();

        $this->assertArrayHasKey('iw_payload', $result);
    }

    /**
     * Add action test.
     */
    public function testAddAction()
    {
        if (TestConfig::get('mysqlpool-tests') != '1') {
            $this->markTestSkipped('skipping mysqlpool-tests');
        }

        $nickname = TestConfig::get('mysqlpool-server', 'nickname');
        $host     = TestConfig::get('mysqlpool-server', 'hostname');
        $username = TestConfig::get('mysqlpool-server', 'username');
        $password = TestConfig::get('mysqlpool-server', 'password');

        $NW    = self::getMasterNodeWorx();
        $input = array( 'nickname'      => $nickname,
                    'host'          => $host,
                    'root_username' => $username,
                    'root_password' => $password );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Activate action test.
     *
     * @depends testAddAction
     */
    public function testActivateAction()
    {
        if (TestConfig::get('mysqlpool-tests') != '1') {
            $this->markTestSkipped('skipping mysqlpool-tests');
        }

        $server   = TestConfig::get('mysqlpool-server', 'nickname');
        $nickname = TestConfig::get('mysqlpool-server', 'nickname');
        $host     = TestConfig::get('mysqlpool-server', 'hostname');
        $username = TestConfig::get('mysqlpool-server', 'username');
        $password = TestConfig::get('mysqlpool-server', 'password');

        $turddsn = "mysql://{$username}:notrealpass@{$host}/mysql";

        $Server = MySQL::getServer($nickname);
        $Server->setDsn($turddsn);
        $Server->save();

        $NW    = self::getMasterNodeWorx();
        $input = array( 'server'        => $server,
                    'nickname'      => $nickname,
                    'host'          => $host,
                    'root_username' => $username,
                    'root_password' => $password );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'activate', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Delete action test.
     *
     * @depends testAddAction
     */
    public function testDeleteAction()
    {
        if (TestConfig::get('mysqlpool-tests') != '1') {
            $this->markTestSkipped('skipping mysqlpool-tests');
        }

        $NW    = self::getMasterNodeWorx();
        $input = array(
                   'server' => TestConfig::get('mysqlpool-server', 'nickname')
                   );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testListServersAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listServers',
            array(),
            $NW
        );

        $this->assertEquals(Reply::OK_PAYLOAD, $Reply->getCode());
    }

    /**
     * Test.
     */
    public function testListActiveServersAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listActiveServers',
            array(),
            $NW
        );

        $this->assertEquals(Reply::OK_PAYLOAD, $Reply->getCode());
    }

    /**
     * Test.
     */
    public function testListDeletableServersAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listDeletableServers',
            array(),
            $NW
        );

        $this->assertEquals(Reply::OK_PAYLOAD, $Reply->getCode());
    }

    /**
     * Test.
     */
    public function listInactiveServersAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listInactiveServers',
            array(),
            $NW
        );

        $this->assertEquals(Reply::OK_PAYLOAD, $Reply->getCode());
    }
}
