<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_NfsExport.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_NfsExportTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_NfsExport';

    /**
     * SetUp function.
     */
    public function setUp()
    {
        IWorxFileSys::mkdir('/tmp/testdir', 0777);
    }

    /**
     * TearDown function.
     */
    public function tearDown()
    {
        IWorxFileSys::rm('/tmp/testdir', true, true);
    }

    /**
     * Test assigned vars of index action.
     */
    public function testIndexActionAssignedVars()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NW);
        $result = $Reply->getData();
        $this->assertArrayHasKey('iw_payload', $result);
        $this->assertArrayHasKey('iw_form', $result);

        $this->assertInstanceOf('Form_NW_NfsExport_Add', $result['iw_form']);
        $this->assertInstanceOf('Payload', $result['iw_payload']);
    }

    /**
     * Test the add action when no input is given.
     */
    public function testAddActionErrorsWithNoInput()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'add', array(), $NW);
        $result = $Reply->getData();
        $this->assertIsFormErrorArray($result);
    }

    /**
     * Test the add action.
     */
    public function testAddAction()
    {
        $NW = self::getMasterNodeWorx();

        $input = array ( 'directory_to_export' => '/tmp/testdir',
                     'ip_allow_list'       => array ( 0 => '127.0.0.1' ),
                     'read_write_access'   => 'ro',
                     'root_user_access'    => 'root_squash',
                     'sync_options'        => 'async' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test the delete action when no input is given.
     */
    public function testDeleteActionErrorsWithNoInput()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'delete', array(), $NW);
        $result = $Reply->getData();
        $this->assertIsFormErrorArray($result);
    }

    /**
     * Test the delete action.
     *
     * @depends testAddAction
     */
    public function testDeleteAction()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'exports' => array( '/tmp/testdir:127.0.0.1' ) );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }
}
