<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Packages.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */

class Ctrl_Nodeworx_PackagesTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_Packages';

    /**
     * Test assigned vars of index action.
     */
    public function testIndexActionAssignedVars()
    {
        $NW     = self::getMasterNodeWorx();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NW);
        $result = $Reply->getData();
        $this->assertInstanceOf('Payload', $result);
    }

    /**
     * Test add action with no input (error case).
     */
    public function testAddActionReturnsErrorWithNoInput()
    {
        $NW    = self::getMasterNodeWorx();
        $input = [];
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test add action returns true with good data.
     */
    public function testAddActionReturnsTrue()
    {
        $NW     = self::getMasterNodeWorx();
        $input  = array( 'package_name' => self::SW_PACKAGE );
        $input  = array_merge($input, self::_getPackageOptionsInput());
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test add action fails if package name already exists.
     */
    public function testAddActionAgainFails()
    {
        $NW = self::getMasterNodeWorx();
        $this->assertTrue($NW->packageTemplateExists(self::SW_PACKAGE));
        $input = array( 'package_name' => self::SW_PACKAGE );
        $input = array_merge($input, self::_getPackageOptionsInput());
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test edit action returns true for existing package.
     */
    public function testEditActionReturnsTrue()
    {
        $NW        = self::getMasterNodeWorx();
        $packageid = $this->_getPackageId();
        $input     = array( 'id' => $packageid );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test edit action, change package name, returns true.
     */
    public function testEditActionChangeNameReturnsTrue()
    {
        $NW        = self::getMasterNodeWorx();
        $packageid = $this->_getPackageId();
        $input     = array( 'id' => $packageid,
                        'package_name' => str_rot13(self::SW_PACKAGE) );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test edit action change successfully changed the package name.
     */
    public function testEditActionChangeNameWorked()
    {
        $NW        = self::getMasterNodeWorx();
        $packageid = $this->_getPackageIdRot13();
        $this->assertFalse(is_null($packageid));
    }

    /**
     * Test delete action returns true.
     */
    public function testDeleteActionReturnsTrue()
    {
        $NW        = self::getMasterNodeWorx();
        $packageid = $this->_getPackageIdRot13();
        $input     = array( 'id' => $packageid );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test delete action worked (package is gone).
     */
    public function testDeleteActionWorked()
    {
        $NW    = self::getMasterNodeWorx();
        $rot13 = str_rot13(self::SW_PACKAGE);
        $NW->loadPackageTemplates();
        $this->assertFalse(
            $NW->packageTemplateExists($rot13),
            'Package delete did not delete'
        );
    }

    /**
     * Get the package id of a siteworx package, named SW_PACKAGE_NAME.
     *
     * @return string
     */
    private function _getPackageId()
    {
        $NW        = self::getMasterNodeWorx();
        $this->assertTrue($NW->packageTemplateExists(self::SW_PACKAGE));
        $templates = $NW->getPackageTemplates();
        foreach ($templates as $template) {
            if ($template['name'] == self::SW_PACKAGE) {
                break;
            }
        }
        return $template['id'];
    }

    /**
     * Get the package id of a siteworx package, named rot13( SW_PACKAGE_NAME ).
     *
     * @return string
     */
    private function _getPackageIdRot13()
    {
        $NW    = self::getMasterNodeWorx();
        $NW->loadPackageTemplates();
        $rot13 = str_rot13(self::SW_PACKAGE);
        $this->assertTrue($NW->packageTemplateExists($rot13));
        $templates = $NW->getPackageTemplates();
        foreach ($templates as $template) {
            if ($template['name'] == $rot13) {
                break;
            }
        }
        return $template['id'];
    }
}
