<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Siteworx
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_SiteworxTest extends IWorx_Framework_TestCase
{

    const CTRL = 'Ctrl_Nodeworx_Siteworx';

    const NEW_NICKNAME   = 'New Nickname Here';
    const NEW_BILLINGDAY = '13';

    private static $_first_setup = false;
    private static $_first_teardown = false;

    /**
     * Make sure the SiteWorx account has been deleted.
     */
    public function setUp()
    {
        if (self::$_first_setup === false) {
            $this->deleteSWAccountIfNecessary();

            self::$_first_setup = true;
        }
    }

    /**
     * Make sure the SiteWorx account has been deleted.
     */
    public function tearDown()
    {
        if (self::$_first_teardown === false) {
            $this->deleteSWAccountIfNecessary();

            self::$_first_teardown = true;
        }
    }

    /**
     * Tests the index action.
     */
    public function testIndexAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the list action.
     */
    public function testListAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'listAccounts', array(), $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the search action.
     */
    public function testSearchAction()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'reseller' => $NW->getId() );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'search', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the listMasterDomains action.
     */
    public function testListMasterDomainsAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listMasterDomains',
            array(),
            $NW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the listFreeIps action.
     */
    public function testListFreeIpsAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listFreeIps',
            array(),
            $NW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the listDedicatedFreeIps action.
     */
    public function testListDedicatedFreeIpsAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listDedicatedFreeIps',
            array(),
            $NW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the listSharedFreeIps action.
     */
    public function testListSharedFreeIpsAction()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listSharedFreeIps',
            array(),
            $NW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests the add action.
     */
    public function testAddAction()
    {
        $NW    = self::getMasterNodeWorx();
        $input = self::_getSWAccountInput();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that the siteworx account was added with all the various params.
     *
     * @depends testAddAction
     */
    public function testAddActionWorkedCorrectly()
    {
        $SW    = $this->getSWAccount();
        $input = self::_getSWAccountInput();

        extract($input);

        $this->assertEquals($master_domain, $SW->getWorkingDomainName());
        $this->assertEquals($master_domain_ipv4, $SW->getIP());
        $this->assertEquals($uniqname, $SW->getUniqname());
        $this->assertEquals($nickname, $SW->getWorkingUser()->getNickName());
        $this->assertEquals($email, $SW->getWorkingUser()->getEmail());
    }

    /**
     * Tests that adding the same account will not work.
     *
     * @depends testAddAction
     */
    public function testAddingSameAccountFails()
    {
        $NW    = self::getMasterNodeWorx();
        $input = self::_getSWAccountInput();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);

        $msg = 'could add the same account twice :(';

        $this->assertReplyNotSuccessful($Reply, $msg);
    }

    /**
     * Tests the toggle favorite actions.
     *
     * @depends testAddAction
     */
    public function testToggleFavoriteAction()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain' => self::SW_DOMAIN );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'toggleFavorite', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests to make sure is favorite is false.
     *
     * @depends testToggleFavoriteAction
     */
    public function testFavoriteIsTrue()
    {
        $user_id = self::getMasterNodeWorx()->getWorkingUser()->getId();
        $domain  = self::SW_DOMAIN;

        $is_favorite = Model_Siteworx::isFavorite($user_id, $domain);

        $this->assertTrue($is_favorite);
    }

    /**
     * Tests the toggle favorite actions.
     *
     * @depends testToggleFavoriteAction
     */
    public function testToggleFavoriteAgain()
    {
        $this->testToggleFavoriteAction();
    }

    /**
     * Tests to make sure is favorite is false.
     *
     * @depends testToggleFavoriteAgain
     */
    public function testFavoriteIsFalse()
    {
        $user_id = self::getMasterNodeWorx()->getWorkingUser()->getId();
        $domain  = self::SW_DOMAIN;

        $is_favorite = Model_Siteworx::isFavorite($user_id, $domain);

        $this->assertFalse($is_favorite);
    }

    /**
     * Tests the edit action.
     *
     * @depends testAddAction
     */
    public function testEditAction()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain' => self::SW_DOMAIN );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that the empty edit action didn't change anything.
     *
     * @depends testEditAction
     */
    public function testEditActionChangedNothing()
    {
        Form::resetInstances();
        IW::Env()->addInput('domain', self::SW_DOMAIN);

        $Form = Form::getInstance('Form_NW_Siteworx_Edit');
        $this->assertTrue($Form->validateForm());

        $ip = self::_getFirstSharedIP();

        $this->assertEquals(self::SW_DOMAIN, $Form->getValue('master_domain'));
        $this->assertEquals($ip, $Form->getValue('master_domain_ipv4'));
        $this->assertEquals(self::SW_UNIQNAME, $Form->getValue('uniqname'));
        $this->assertEquals(self::SW_NICK, $Form->getValue('nickname'));
        $this->assertEquals(self::SW_EMAIL, $Form->getValue('email'));

        $options = Option::getOptionList();

        $unlimited = Ini::get(Ini::CONSTANTS, 'unlimited');
        foreach ($options as $opt) {
            if ($opt['displayGroup'] !== '1') {
                continue;
            }
            $input_name = substr($opt['name'], 3);
            $this->assertEquals($unlimited, $Form->getValue($input_name));
        }

        foreach ($options as $opt) {
            $input_name = substr($opt['name'], 3);
            if ($opt['displayGroup'] !== '2') {
                continue;
            }
            if ($input_name != 'OPT_SUPHP' && $input_name != 'OPT_SSI') {
                $this->assertEquals(1, $Form->getValue($input_name));
            } else {
                $this->assertEquals(0, $Form->getValue($input_name));
            }
        }
    }

    /**
     * Tests that edit works when the nickname is changed.
     *
     * @depends testEditAction
     */
    public function testEditActionChangeNicknameReturnsTrue()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain'   => self::SW_DOMAIN,
                    'nickname' => self::NEW_NICKNAME );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests edit action, nickname change worked.
     *
     * @depends testEditActionChangeNicknameReturnsTrue
     */
    public function testEditActionChangeNicknameWorked()
    {
        Form::resetInstances();
        IW::Env()->addInput('domain', self::SW_DOMAIN);

        $Form = Form::getInstance('Form_NW_Siteworx_Edit');
        $this->assertTrue($Form->validateForm());

        $this->assertEquals(self::NEW_NICKNAME, $Form->getValue('nickname'));
    }

    /**
     * Tests that edit works when the billing day is changed.
     *
     * @depends testEditAction
     */
    public function testEditActionChangeBillingDayReturnsTrue()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain'      => self::SW_DOMAIN,
                    'billing_day' => self::NEW_BILLINGDAY );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests edit action, billing day change worked.
     *
     * @depends testEditActionChangeBillingDayReturnsTrue
     */
    public function testEditActionChangeBillingDayWorked()
    {
        Form::resetInstances();
        IW::Env()->addInput('domain', self::SW_DOMAIN);

        $Form = Form::getInstance('Form_NW_Siteworx_Edit');
        $this->assertTrue($Form->validateForm());

        $this->assertEquals(self::NEW_BILLINGDAY, $Form->getValue('billing_day'));
    }

    /**
     * Test.
     *
     * @depends testAddAction
     */
    public function testEditActionChangeStorageQuota()
    {
        $quota_value = '1000';
        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain'   => self::SW_DOMAIN,
                    'OPT_STORAGE' => $quota_value );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);

        $this->assertReplySuccessful($Reply);
        $ov = Ini::get(Ini::QUOTA, 'quota-overflow-pct');
        $soft = IWorxFileSys::getQuota(self::SW_UNIQNAME, 'soft');
        $hard = IWorxFileSys::getQuota(self::SW_UNIQNAME, 'hard');

        $quota = $quota_value * 1024;
        $this->assertEquals($quota, $soft);
        $this->assertEquals($quota + ($quota * ($ov/100)), $hard);
    }


    /**
     * Test edit action - shell password should be updated
     */
    public function testEditActionShouldUpdateShellPassword()
    {
        // enable shell if not enabled
        $this->_enableShell();
        $encryptedPassword = IWorxUtil::hashWithSalt('Otherpass456', IWorxUtil::SHA512);

        $NW    = self::getMasterNodeWorx();
        $input = array(
            'domain'   => self::SW_DOMAIN,
            'password' => $encryptedPassword,
            'confirm_password' => $encryptedPassword,
            'encrypted' => 'y',
            Form_SW_Users_Edit::INPUT_PASSWORD_OPTIONS => ['SHELL']
        );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);
        $this->assertReplySuccessful($Reply);

        $this->assertSame($encryptedPassword, $this->_getShellPassword());

        // ensure ftp & postmaster password isn't affected
        $this->assertNotSame($encryptedPassword, $this->_getFtpPassword());
        $this->assertNotSame($encryptedPassword, $this->_getPostmasterPassword());
    }


    /**
     * enable shell.
     */
    private function _enableShell()
    {
        $passwd = ShellAccount::getCombinedPasswd();
        $user   = self::SW_UNIQNAME;

        if ($passwd[$user]['pass'] === '!!') {
            ShellAccount::changePasswd($user, 'pws3456');
        }

        if ($passwd[ $user ]['disabled']) {
            ShellAccount::enableAccount($user);
        }
    }

    /**
     * Get shadow psw for master sw shell user.
     *
     * @return string
     */
    private function _getShellPassword()
    {
        $passwds = ShellAccount::getCombinedPasswd();
        return $passwds[self::SW_UNIQNAME]['pass'];
    }

    /**
     * Test that edit action updates FTP password.
     */
    public function testEditActionShouldUpdateFtpPassword()
    {
        $encryptedPassword = IWorxUtil::hashWithSalt('Somepass456', IWorxUtil::SHA512);

        $NW    = self::getMasterNodeWorx();
        $input = array(
            'domain'   => self::SW_DOMAIN,
            'password' => $encryptedPassword,
            'confirm_password' => $encryptedPassword,
            'encrypted' => 'y',
                Form_SW_Users_Edit::INPUT_PASSWORD_OPTIONS => ['FTP']
        );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);
        $this->assertReplySuccessful($Reply);

        $ftpPassword = $this->_getFtpPassword();
        $this->assertSame($encryptedPassword, $ftpPassword);

        // ensure shell & postmaster passwords are not updated
        $this->assertNotSame($encryptedPassword, $this->_getShellPassword());
        $this->assertNotSame($encryptedPassword, $this->_getPostmasterPassword());
    }

    /**
     * Get FTP password for main ftp user.
     *
     * @return string
     */
    private function _getFtpPassword()
    {
        $SW    = $this->getSWAccount();
        $Driver = $SW->getFTPDriver();

        $ftpInfo = $Driver->getAccount('ftp');
        return $ftpInfo->password;
    }

    /**
     * test edit action should update postmaster password.
     */
    public function testEditActionShouldUpdatePostmasterPassword()
    {
        $encryptedPassword = IWorxUtil::hashWithSalt('anotherPass123', IWorxUtil::SHA512);

        $NW    = self::getMasterNodeWorx();
        $input = array(
            'domain'   => self::SW_DOMAIN,
            'password' => $encryptedPassword,
            'confirm_password' => $encryptedPassword,
            'encrypted' => 'y',
            Form_SW_Users_Edit::INPUT_PASSWORD_OPTIONS => ['EMAIL']
        );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);
        $this->assertReplySuccessful($Reply);

        $this->assertSame($encryptedPassword, $this->_getPostmasterPassword());

        // ensure ftp & shell not updated
        $this->assertNotSame($encryptedPassword, $this->_getShellPassword());
        $this->assertNotSame($encryptedPassword, $this->_getFtpPassword());
    }

    /**
     * @return string
     */
    private function _getPostmasterPassword()
    {
        $SW    = $this->getSWAccount();
        $boxes = $SW->getIncomingMailDriver()->getBoxes();

        foreach ($boxes as $box) {
            if ($box->username === 'postmaster') {
                return $box->crypt_password;
            }
        }
    }

    /**
     * test edit action should update postmaster password.
     */
    public function testEditActionPasswordChangeWhenNoDefaultAccounts()
    {
        $this->_deletePostmaster();
        $this->_deleteDefaultFtp();
        $this->_disableShell();

        $encryptedPassword = IWorxUtil::hashWithSalt('anotherPass123', IWorxUtil::SHA512);

        $NW    = self::getMasterNodeWorx();
        $input = array(
            'domain'   => self::SW_DOMAIN,
            'password' => $encryptedPassword,
            'confirm_password' => $encryptedPassword,
            'encrypted' => 'y',
        );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $NW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Deletes the postmaster mailbox
     */
    private function _deletePostmaster()
    {
        $SW = $this->getSWAccount();
        $input = array('username' => 'postmaster');
        IW::FC()->routeFromPHP('Ctrl_Siteworx_EmailBox', 'delete', $input, $SW);
    }

    /**
     * Deletes the default FTP user
     */
    private function _deleteDefaultFtp()
    {
        $SW = $this->getSWAccount();
        $input = array('user' => 'ftp');
        IW::FC()->routeFromPHP('Ctrl_Siteworx_Ftp', 'delete', $input, $SW);
    }

    /**
     * Disables the shell account
     */
    private function _disableShell()
    {
        $user   = self::SW_UNIQNAME;
        ShellAccount::disableAccount($user);
    }

    /**
     * Tests bulk deactivate action.
     *
     * @depends testAddAction
     */
    public function testBulkdeactivateAction()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'domains' => array( self::SW_DOMAIN ) );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'bulkdeactivate', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests results of bulk deactivate action.
     *
     * @depends testBulkdeactivateAction
     */
    public function testBulkdeactivateActionWorked()
    {
        $NW = self::getMasterNodeWorx();
        $SW = new SiteWorx(self::SW_DOMAIN);

        $this->assertEquals('inactive', $SW->getStatus());
    }

    /**
     * Tests bulk activate action.
     *
     * @depends testAddAction
     * @depends testBulkdeactivateAction
     */
    public function testBulkactivateAction()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'domains' => array( self::SW_DOMAIN ) );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'bulkactivate', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests results of bulk activate action.
     *
     * @depends testBulkactivateAction
     */
    public function testBulkactivateActionWorked()
    {
        $NW = self::getMasterNodeWorx();
        $SW = new SiteWorx(self::SW_DOMAIN);

        $this->assertEquals('active', $SW->getStatus());
    }

    /**
     * Tests delete action.
     */
    public function testDeleteAction()
    {
        self::addSWAccountIfNecessary();

        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain' => self::SW_DOMAIN );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $NW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests results of delete action.
     *
     * @depends testDeleteAction
     */
    public function testDeleteActionWorked()
    {
        $NW = self::getMasterNodeWorx();
        $SW = new SiteWorx(self::SW_DOMAIN);

        $this->assertFalse($SW->exists());
    }

    /**
     * Tests that the delete action removes databases.
     */
    public function testDeleteActionRemovesDatabases()
    {
        self::addSWAccountIfNecessary();

        $input = array( 'name' => 'sampledb' );
        $SW    = $this->getSWAccount();
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_MysqlDb',
            'add',
            $input,
            $SW
        );

        $this->assertReplySuccessful($Reply);

        $NW    = self::getMasterNodeWorx();
        $input = array( 'domain' => self::SW_DOMAIN );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $NW);

        $this->assertReplySuccessful($Reply);
        $this->assertFalse(is_dir('/var/lib/mysql/addedbyt_sampledb'));
    }

    /**
     * Test.
     */
    public function testAddOnDedicatedIPMakesIPUnavailable()
    {
        $NW    = self::getMasterNodeWorx();
        $input = self::_getSWAccountInput();

        $input['master_domain_ipv4'] = self::_getFirstAvailableDedicatedIP();

        IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $NW);

        $available_ips = $NW->getUnusedIPList();

        $this->assertFalse(
            isset($available_ips[ $input['master_domain_ipv4'] ]),
            'Dedicated IP used but still shown as available!'
        );
    }

    /**
     * Test.
     */
    public function testEditIPAffectsPointerDomains()
    {
        $SW    = $this->getSWAccount();

        $this->_addPointerDomain();
        $pointers = $SW->getMasterDomain()->getChildDomains();

        $Master_IP = $SW->getMasterDomain()->getIPv4AddressObj();
        $Pointr_IP = $pointers[0]->getIPv4AddressObj();

        $this->assertEquals($Master_IP->getExternalIP(), $Pointr_IP->getExternalIP());
        $this->assertEquals($Master_IP->getIP(), $Pointr_IP->getIP());

        $new_ip   = self::_getFirstSharedIP();
        $IWorx_IP = IWorx_IP::factoryFromDatabase($new_ip);

        $SW->getMasterDomain()->changeIPv4AddressTo($IWorx_IP);
        $pointers = $SW->getMasterDomain()->getChildDomains();

        $New_Master_IP = $SW->getMasterDomain()->getIPv4AddressObj();
        $New_Pointr_IP = $pointers[0]->getIPv4AddressObj();

        $this->assertEquals($New_Master_IP->getExternalIP(), $New_Pointr_IP->getExternalIP());
        $this->assertEquals($New_Master_IP->getIP(), $New_Pointr_IP->getIP());
    }

    /**
     * Test changing to an IPv6 subpool.
     */
    public function testChangeToIPv6Subpool()
    {
        self::deleteSWAccountIfNecessary();

        $SW = $this->getSWAccount();
        $NW = new NodeWorx(NodeWorx::MASTER_ID);

        $this->assertEquals('0', $SW->getIPv6Pool());

        $Pool = $this->_setupIPv6Pool();

        $parent_pool = $Pool->getRangeStartWithCIDR();

        $input = array( 'domain' => $SW->getMasterDomainName(),
                    'ipv6_pool' => $parent_pool );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'editCommit',
            $input,
            $NW
        );

        $this->assertReplySuccessful($Reply);
        $this->assertContains('/112', $SW->getIPv6Pool());
    }

    /**
     * Test.
     */
    public function testSetConfigSingle()
    {
        $NW    = self::getMasterNodeWorx();
        $this->deleteSWAccountIfNecessary();
        $SW    = $this->getSWAccount();
        $domain = self::SW_DOMAIN;
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $pre_config, $retval);

        $input = array( 'master_domain' => $domain,
                        'config_name' => 'z_IWORX_TEST_CONFIG_NAME',
                        'config_value' => 'iworx-test-config-value' );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'setConfig',
            $input,
            $NW
        );

        $expected = $pre_config;
        $expected[] = "z_IWORX_TEST_CONFIG_NAME\tiworx-test-config-value";
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $post_config, $retval);
        $this->assertEquals($expected, $post_config);
    }

    /**
     * Test.
     */
    public function testSetConfigSingleEmpty()
    {
        $NW    = self::getMasterNodeWorx();
        $this->deleteSWAccountIfNecessary();
        $SW    = $this->getSWAccount();
        $domain = self::SW_DOMAIN;
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $pre_config, $retval);

        $input = array( 'master_domain' => $domain,
                        'config_name' => 'z_IWORX_TEST_CONFIG_NAME',
                        'config_value' => '' );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'setConfig',
            $input,
            $NW
        );

        $this->assertReplySuccessful($Reply);

        $expected = $pre_config;
        $expected[] = "z_IWORX_TEST_CONFIG_NAME";
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $post_config, $retval);
        $this->assertEquals($expected, $post_config);
    }

    /**
     * Test.
     */
    public function testSetConfigMultiple()
    {
        $NW    = self::getMasterNodeWorx();
        $this->deleteSWAccountIfNecessary();
        $SW    = $this->getSWAccount();
        $domain = self::SW_DOMAIN;
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $pre_config, $retval);

        $data = ['z_IWORX_TEST_1' => 'test-value-1',
                 'z_IWORX_TEST_2' => 'test-value-2',
                 'z_IWORX_TEST_3' => 'test-value-3',
                 'z_IWORX_TEST_4' => '0',
                 'z_IWORX_TEST_5' => '1',
                 'z_IWORX_TEST_6' => '',
                ];
        $input = array('master_domain' => $domain,
                       'config_name'   => array_keys($data),
                       'config_value'  => array_values($data));

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'setConfig',
            $input,
            $NW
        );

        $expected = $pre_config;
        foreach ($data as $key => $value) {
            $expected[] = trim("{$key}\t{$value}");
        }
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $post_config, $retval);
        $this->assertEquals($expected, $post_config);
    }

    /**
     * Test.
     */
    public function testApiSetConfig()
    {
        $NW    = self::getMasterNodeWorx();
        $this->deleteSWAccountIfNecessary();
        $SW    = $this->getSWAccount();
        $domain = self::SW_DOMAIN;
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $pre_config, $retval);


        $key = ['email'   => self::NW_USER,
                'password' => self::NW_PASS];

        $api_controller = '/nodeworx/siteworx';
        $action         = 'setConfig';
        // Be aware that even actions that require no input still require the parameter
        // Just pass in an empty array
        $input = array( 'master_domain' => $domain,
                        'config_name' => 'z_IWORX_TEST_CONFIG_NAME',
                        'config_value' => 'iworx-test-config-value' );

        $params = ['apikey'    => $key,
                   'ctrl_name' => $api_controller,
                   'action'    => $action,
                   'input'     => $input];

        $client = new Zend_XmlRpc_Client('http://127.0.0.1:2080/xmlrpc');
        $result = $client->call('iworx.route', $params);

        if ($result['status'] !== 0) {
            throw new Exception($result['payload']);
        }

        $expected = $pre_config;
        $expected[] = "z_IWORX_TEST_CONFIG_NAME\tiworx-test-config-value";
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $post_config, $retval);
        $this->assertEquals($expected, $post_config);
    }

    /**
     * Test.
     */
    public function testMultiApiSetConfig()
    {
        $NW    = self::getMasterNodeWorx();
        $this->deleteSWAccountIfNecessary();
        $SW    = $this->getSWAccount();
        $domain = self::SW_DOMAIN;
        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $pre_config, $retval);


        $key = ['email'   => self::NW_USER,
                'password' => self::NW_PASS];

        $api_controller = '/nodeworx/siteworx';
        $action         = 'setConfig';

        $data = [
            'z_IWORX_TEST_1' => 'test-value-1',
            'z_IWORX_TEST_2' => 'test-value-2',
            'z_IWORX_TEST_3' => 'test-value-3',
            'z_IWORX_TEST_4' => '0',
            'z_IWORX_TEST_5' => '1',
            'z_IWORX_TEST_6' => '',
        ];
        $input = array('master_domain' => $domain,
                       'config_name'   => array_keys($data),
                       'config_value'  => array_values($data));

        $params = ['apikey'    => $key,
                   'ctrl_name' => $api_controller,
                   'action'    => $action,
                   'input'     => $input];

        $client = new Zend_XmlRpc_Client('http://127.0.0.1:2080/xmlrpc');
        $result = $client->call('iworx.route', $params);

        if ($result['status'] !== 0) {
            throw new Exception($result['payload']);
        }

        $expected = $pre_config;
        foreach ($data as $key => $value) {
            $expected[] = trim("{$key}\t{$value}");
        }

        exec("/home/interworx/bin/config.pex --list --siteworx={$domain} | sort", $post_config, $retval);
        $this->assertEquals($expected, $post_config);
    }

    /**
     * Sets up the IPv6 Pool.
     *
     * @return IPv6_Pool
     */
    private function _setupIPv6Pool()
    {
        $Pool = IPv6_Pool::factoryFromDatabase('2001:db8::/96');
        if (!$Pool) {
            $Pool = IPv6_Pool::factoryFromAddress('2001:db8::/96');
        }
        $Pool->setNickname('testrange');
        $Pool->setDistributionPolicy(IPv6_Pool::POLICY_SEQUENTIAL);
        $Pool->setNetmaskCIDR('128');
        $Pool->setDevice('eth0');
        $Pool->setSubpoolSize(112);

        $Pool->save();

        return $Pool;
    }

    /**
     * @dataProvider pointerProvider
     * @param string $domain_name
     * @param string $pointer_type
     * @param string $points_to
     */
    public function testListAllDomainsIncludePointers($domain_name, $pointer_type, $points_to)
    {
        if ($points_to === self::SW_SECONDARY_DOMAIN) {
            $this->addSecondaryDomainIfNecessary();
        }

        $this->_addPointerDomain($domain_name, $pointer_type, $points_to);
        $domain = $this->_getDomainFromListDomainsAction($domain_name);
        $Pointer = new Domain($domain_name);
        $Parent = $Pointer->getParentDomain();

        $this->_testDocRoot($Parent->getDocRoot(), $domain);
        $this->_testDomainRoot($Parent->getDomainRoot(), $domain);
        $this->_testIpv4($Parent->getIPv4Address(), $domain);
        $this->_testDomainType(Domain::POINTER, $domain);
        $this->_testDomainStatus('active', $domain);
        $this->_testUnixUser(self::SW_UNIQNAME, $domain);
        $this->_testPointsTo($points_to, $domain);
        $this->_testPointerType($pointer_type, $domain);
        $this->_testPhpVersion($Parent->getPhpVersion(), $domain);
    }

    /**
     * Test listAllDomainsAction - redirect domains
     *
     * @dataProvider redirectProvider
     * @param string $domain_name
     * @param string $pointer_type
     * @param string $points_to
     */
    public function testListAllDomainsIncludeRedirects($domain_name, $pointer_type, $points_to)
    {
        $this->_addPointerDomain($domain_name, $pointer_type, $points_to);
        $domain = $this->_getDomainFromListDomainsAction($domain_name);
        $Domain = new Domain($domain_name);
        $Parent = $Domain->getParentDomain();
        $this->_testIpv4($Parent->getIPv4Address(), $domain);
        $this->_testDomainRoot('', $domain);
        $this->_testDocRoot('', $domain);
        $this->_testPhpVersion('', $domain);
        $this->_testDomainType(Domain::REDIRECT, $domain);
        $this->_testDomainStatus(Domain::ACTIVE, $domain);
        $this->_testUnixUser(self::SW_UNIQNAME, $domain);
        $this->_testPointsTo("http://$points_to", $domain);
        $this->_testPointerType($pointer_type, $domain);
        $this->_testMasterDomain(self::SW_DOMAIN, $domain);
    }

    /**
     * Test listAllDomainsAction - master domain
     */
    public function testListAllDomainsMasterDomain()
    {
        $this->addSWAccountIfNecessary();
        $SW = $this->getSWAccount();
        $domain = $this->_getDomainFromListDomainsAction(self::SW_DOMAIN);

        $this->_testMasterDomain($SW->getMasterDomainName(), $domain);
        $this->_testDomainType(Domain::MASTER, $domain);
        $this->_testDomainStatus('active', $domain);
        $this->_testDomainRoot($SW->getPackageRoot() . '/addedbytest.com', $domain);
        $this->_testDocRoot($SW->getPackageRoot() .'/addedbytest.com/html', $domain);
        $this->_testIpv4($SW->getMasterDomain()->getIPv4Address(), $domain);
        $this->_testUnixUser(self::SW_UNIQNAME, $domain);
        $this->_testPointsTo('', $domain);
        $this->_testPointerType('', $domain);
        $this->_testPhpVersion('system-php', $domain);
    }

    /**
     * Test listAllDomainsAction - master domain
     */
    public function testListAllDomainsSubdomain()
    {
        $prefix = 'mysub';
        $domain_name = 'mysub.addedbytest.com';

        $this->_addSubdomainIfNecessary($prefix);

        $SW = $this->getSWAccount();
        $domain = $this->_getDomainFromListDomainsAction($domain_name);
        $Domain = new Domain($domain_name);
        $Parent = $Domain->getParentDomain();

        $this->_testDomainType(Domain::SUBDOMAIN, $domain);
        $this->_testDomainStatus('active', $domain);
        $this->_testDomainRoot($SW->getPackageRoot() . '/addedbytest.com', $domain);
        $this->_testDocRoot($SW->getPackageRoot() .'/addedbytest.com/html/mysub', $domain);
        $this->_testIpv4($Parent->getIPv4Address(), $domain);
        $this->_testMasterDomain(self::SW_DOMAIN, $domain);
        $this->_testUnixUser(self::SW_UNIQNAME, $domain);
        $this->_testPointsTo('', $domain);
        $this->_testPointerType('', $domain);
        $this->_testPhpVersion($Parent->getPhpVersion(), $domain);
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testMasterDomain($expected, array $domain)
    {
        $this->assertSame($expected, $domain['master_domain'], 'unexpected master domain for type ' . $domain['domain_type']);
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testPhpVersion($expected, array $domain)
    {
        $this->assertSame($expected, $domain['php_version'], 'php_version not expected');
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testPointsTo($expected, array $domain)
    {
        $this->assertSame($expected, $domain['points_to'], 'points_to not expected');
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testPointerType($expected, array $domain)
    {
        $this->assertSame($expected, $domain['pointer_type'], 'pointer_type not expected');
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testUnixUser($expected, array $domain)
    {
        $this->assertSame($expected, $domain['unix_user'], 'unix user not expected');
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testDocRoot($expected, array $domain)
    {
        $this->assertSame($expected, $domain['domain_docroot'], 'docroot not expected for type ' . $domain['domain_type']);
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testDomainRoot($expected, array $domain)
    {
        $this->assertSame($expected, $domain['domain_root'], 'domainroot not expected for type ' . $domain['domain_type']);
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testDomainType($expected, array $domain)
    {
        $this->assertSame($expected, $domain['domain_type'], 'domain_type not expected');
    }


    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testDomainStatus($expected, array $domain)
    {
        $this->assertSame($expected, $domain['domain_status'], 'domain_status not expected');
    }

    /**
     * @param string $expected
     * @param array  $domain
     */
    private function _testIpv4($expected, array $domain)
    {
        $this->assertSame($expected, $domain['domain_ipv4'], 'ipv4 not expected for type ' . $domain['domain_type']);
    }

    /**
     * @param string $prefix
     */
    private function _addSubDomainIfNecessary($prefix)
    {
        $master = self::SW_DOMAIN;
        $SW = $this->getSWAccount();
        if ($SW->subDomainExists("$prefix.$master")) {
            return '';
        }
        $SW->setWorkingDomain(new Domain(self::SW_DOMAIN));
        $ctrl  = Ctrl_Siteworx_DomainsSub::class;
        $input = array('prefix' => $prefix);

        $Reply = IW::FC()->routeFromPHP($ctrl, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * test listAllDomains action should include inactive domains
     */
    public function testListAllDomainsIncludesInactiveDomain()
    {
        $this->_testDomainStatusChange(Domain::INACTIVE);
    }

    /**
     * test listAllDomains action should include pending domains
     */
    public function testListAllDomainsIncludesPendingDomain()
    {
        $this->_testDomainStatusChange(Domain::PENDING);
    }

    /**
     * test listAllDomains action - status
     *
     * @param string $expected_status
     */
    private function _testDomainStatusChange($expected_status)
    {
        $secondary = 'secondary.com';
        $this->addSecondaryDomainIfNecessary($secondary);

        $Domain = new Domain($secondary);
        $Domain->setStatus($expected_status);
        $Domain->save();

        $domain = $this->_getDomainFromListDomainsAction($secondary);
        $this->assertSame($expected_status, $domain['domain_status']);
    }

    /**
     * get domain by name from action listAllDomains
     *
     * @param string $domain_name
     */
    private function _getDomainFromListDomainsAction($domain_name): array
    {
        $domains = $this->_listDomains();
        $key = array_search($domain_name, array_column($domains, 'domain'));
        $this->assertNotFalse($key, 'domain not included in listAllDomainsAction: ' . $domain_name);

        return $domains[$key];
    }

    /**
     * @return array
     */
    private function _listDomains()
    {
        $NW    = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'listAllDomains', array(), $NW);
        $this->assertReplySuccessful($Reply);
        return $Reply->getPayloadData();
    }

    /**
     * provider for pointer domains (aliases)
     */
    public function pointerProvider()
    {
        // domain name, pointer type, points to
        return [
            ['pointersecondary.com', Domain::PTR_ALIAS, self::SW_SECONDARY_DOMAIN],
            ['pointeralias.com', Domain::PTR_ALIAS, self::SW_DOMAIN],
        ];
    }

    /**
     * provider for redirect domains (301/302)
     */
    public function redirectProvider()
    {
        return [
            ['testredir301.com', Domain::PTR_301, 'testtarget301.com'],
            ['testredir302.com', Domain::PTR_302, 'testtarget302.com'],
        ];
    }
}
