<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Nov 29, 2010
 */

/**
 * Tests for the scheduled backup controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_BackupScheduleTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_BackupSchedule';

    /**
     * Tests the index action.
     */
    public function testIndexAction()
    {
        $this->deleteSWAccountIfNecessary();
        $SiteWorx = $this->getSWAccount();

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'index',
            array(),
            $SiteWorx
        );

        $Data = $Reply->getData();

        $this->assertTrue($Data instanceof Payload);
    }

    /**
     * Tests that the create action works.
     */
    public function testCreateAction()
    {
        $this->deleteSWAccountIfNecessary();
        $this->getSWAccount();

        $input =
        array( 'frequency' => Model_Scheduled_Backup::FREQUENCY_WEEKLY,
             'day_of_week' => '0',
             'type'      => Model_Scheduled_Backup::TYPE_STRUCTURE,
             'location'  => Model_Scheduled_Backup::LOCATION_SITEWORX,
             'hour'      => '1',
             'rotate'    => '10' );

        $this->_createScheduledBackup($input);
    }

    /**
     * Tests that the delete action works.
     */
    public function testDeleteAction()
    {
        $SiteWorx  = $this->getSWAccount();
        $find_args = array( 'siteworx_id' => $SiteWorx->getId() );
        $scheduled = Model_Scheduled_Backup::find($find_args);

        $schedule_ids = [];
        foreach ($scheduled as $Schedule) {
            $schedule_ids[] = $Schedule->getPrimaryId();
        }

        $input = array( 'scheduled' => $schedule_ids );
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'delete',
            $input,
            $SiteWorx
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Does a simple edit to test that the action works now.
     */
    public function testEditLocal()
    {
        $this->deleteSWAccountIfNecessary();
        $SiteWorx = $this->getSWAccount();

        $input =
        ['frequency'   => Model_Scheduled_Backup::FREQUENCY_WEEKLY,
         'day_of_week' => '0',
         'type'        => Model_Scheduled_Backup::TYPE_STRUCTURE,
         'location'    => Model_Scheduled_Backup::LOCATION_LOCAL,
         'hour'        => '1',
         'rotate'      => '10',
         'remotepath'  => '/home/addedbyt'];

        $this->_createScheduledBackup($input);

        $find = ['frequency'   => $input['frequency'],
             'siteworx_id' => $SiteWorx->getId()];

        $Backup = Model_Scheduled_Backup::find($find);

        $this->assertInstanceOf('Model_Scheduled_Backup', $Backup[0]);

        $input = ['id'          => $Backup[0]->getPrimaryId(),
              'frequency'   => Model_Scheduled_Backup::FREQUENCY_WEEKLY,
              'day_of_week' => '0',
              'type'        => Model_Scheduled_Backup::TYPE_STRUCTURE,
              'location'    => Model_Scheduled_Backup::LOCATION_LOCAL,
              'hour'        => 2];

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'editCommit',
            $input,
            $SiteWorx
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that setting a local file backup directly in iworx-backup folder
     * doesn't result in empty backup list
     *
     * @return void
     * @throws IWorx_Exception_DoingItWrong
     */
    public function testLocalFileInIworxBackupWorks()
    {
        $this->deleteSWAccountIfNecessary();
        $SW = $this->getSWAccount();
        $backup_dir = $SW->getLocalBackupDir();

        $input = [
            'frequency' => Model_Scheduled_Backup::FREQUENCY_WEEKLY,
            'type' => Model_Scheduled_Backup::TYPE_FULL,
            'location' => Model_Scheduled_Backup::LOCATION_LOCAL,
            'day_of_week' => '0',
            'remotepath' => $backup_dir
        ];
        $this->_createScheduledBackup($input);

        $find = [
            'frequency'   => $input['frequency'],
            'siteworx_id' => $SW->getId()
        ];
        $Backup = Model_Scheduled_Backup::find($find);

        $scheduled_backup_pex = Ini::get(Ini::IWORX_BIN, 'scheduled-backup');
        $cmd = "{$scheduled_backup_pex} --execute {$Backup[0]->getPrimaryId()}";
        $Cmd = new Command($cmd);
        $Cmd->runAsRoot()->exec();
        $this->assertCommandSuccessful($Cmd);

        $Payload = Payload_Factory_SW_Backups::getWeeklyBackups($SW);
        $backups = $Payload->getData();
        $this->assertNotEmpty($backups);
    }

    /**
     * Create a scheduled backup and ensure success
     *
     * @param array $input
     * @return void
     * @throws Exception
     */
    private function _createScheduledBackup(array $input)
    {
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'create',
            $input,
            $this->getSWAccount()
        );
        $this->assertReplySuccessful($Reply);
    }
}
