<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jun 23, 2009
 */

/**
 * Tests for Ctrl_Siteworx_Backup.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_BackupTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_Backup';

    static private $_first_setup = false;

    /**
     * Make sure the SiteWorx account has been deleted.
     */
    public function setUp()
    {
        if (self::$_first_setup === false) {
            $this->deleteSWAccountIfNecessary();
            $this->addSWAccountIfNecessary();

            self::$_first_setup = true;
        }//end if
    }

    /**
     * Tests that listing all backups works.
     */
    public function testListAllBackupsAction()
    {
        $SiteWorx = $this->getSWAccount();

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listAllBackups',
            array(),
            $SiteWorx
        );

        $this->assertTrue($Reply->getData() instanceof Payload);
    }

    /**
     * Tests that listing daily backups works.
     */
    public function testListDailyBackupsAction()
    {
        $SiteWorx = $this->getSWAccount();

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listDailyBackups',
            array(),
            $SiteWorx
        );

        $this->assertTrue($Reply->getData() instanceof Payload);
    }

    /**
     * Tests that listing weekly backups works.
     */
    public function testListWeeklyBackupsAction()
    {
        $SiteWorx = $this->getSWAccount();
        $Reply    = IW::FC()->routeFromPHP(
            self::CTRL,
            'listWeeklyBackups',
            array(),
            $SiteWorx
        );

        $this->assertTrue($Reply->getData() instanceof Payload);
    }

    /**
     * Tests that listing monthly backups works.
     */
    public function testListMonthlyBackupsAction()
    {
        $SiteWorx = $this->getSWAccount();
        $Reply    = IW::FC()->routeFromPHP(
            self::CTRL,
            'listMonthlyBackups',
            array(),
            $SiteWorx
        );

        $this->assertTrue($Reply->getData() instanceof Payload);
    }

    /**
     * Test creating the backup.
     */
    public function testCreateAction()
    {
        $SiteWorx = $this->getSWAccount();
        $input    = array( 'block'    => '1',
                       'type'     => 'full',
                       'location' => 'siteworx' );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'create',
            $input,
            $SiteWorx
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests deleting a backup.
     *
     * @depends testCreateAction
     */
    public function testDeleteAction()
    {
        $SiteWorx   = $this->getSWAccount();
        $domain     = $SiteWorx->getMasterDomainName();
        $DataSource = new DataSource_Backup_Files($domain);

        $input = array( 'backups' => array( $DataSource->getFirstValue() ) );
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'delete',
            $input,
            $SiteWorx
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test creating partial backups.
     */
    public function testCreatePartialBackup()
    {
        $SiteWorx = $this->getSWAccount();
        $input    = array( 'block'    => '1',
                       'type'     => 'partial',
                       'location' => 'siteworx',
                       'options'  => array( 'web', 'mail', 'dbs' ) );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'create',
            $input,
            $SiteWorx
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test restoring a backup.
     *
     * @depends testCreatePartialBackup
     */
    public function testRestoreAction()
    {
        $SiteWorx   = $this->getSWAccount();
        $domain     = $SiteWorx->getMasterDomainName();
        $DataSource = new DataSource_Backup_PartialFiles($domain);
        $input      = array( 'filetype' => 'remote',
                         'file'     => $DataSource->getFirstValue() );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'restore',
            $input,
            $SiteWorx
        );

        $this->assertReplySuccessful($Reply);
    }
}
