<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Jan 9, 2010
 */

/**
 * Test Suite for Ctrl_Nodeworx_DnsZone.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Siteworx_DnsTest extends IWorx_Framework_TestCase
{

    const CTRL      = 'Ctrl_Siteworx_Dns';
    const DUMMY_IP  = '1.2.3.4';
    const DUMMY_TXT = 'dummy TXT record value';

    /**
     * Dataprovider.
     *
     * @return array
     */
    public function getGoodRecordData()
    {
        return
        array( array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, self::DUMMY_IP,
                    1234 ),
             array( self::SW_DOMAIN, 'test2', Dns_Record::CNAME, 'www.google.com',
                    4321 ),
             array( self::SW_DOMAIN, 'test3', Dns_Record::TXT, self::DUMMY_TXT,
                    48300 ) );
    }

    /**
     * Dataprovider.
     *
     * @return array
     */
    public function getBadRecordTypeData()
    {
        return
        array(
            array( self::SW_DOMAIN, 'test1', '', self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, 'test1', null, self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, 'test1', array(), self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::MX, self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::NS, self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::PTR, self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::SOA, self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::SPF, self::DUMMY_IP, 1234 )
            );
    }

    /**
     * Dataprovider.
     *
     * @return array
     */
    public function getBadHostData()
    {
        return
        array(
            array( self::SW_DOMAIN, '', Dns_Record::ADDRESS, self::DUMMY_IP, 1234 ),
            array( self::SW_DOMAIN, null, Dns_Record::ADDRESS, self::DUMMY_IP,
                   1234 ),
            array( self::SW_DOMAIN, array(), Dns_Record::ADDRESS, self::DUMMY_IP,
                   1234 )
            );
    }

    /**
     * Dataprovider.
     *
     * @return array
     */
    public function getBadTargetDataForAddressType()
    {
        return
        array(
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, '', 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, null, 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, array(), 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, 'www.google.com',
                   1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, '1', 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, '1.1', 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, '1.1.1', 1234 ),
            array( self::SW_DOMAIN, 'test1', Dns_Record::ADDRESS, '1.1.1.1.1', 1234 )
            );
    }

    /**
     * Dataprovider.
     *
     * @return array
     */
    public function getBadTargetDataForCnameType()
    {
        return array(
                 array( self::SW_DOMAIN, 'test1', Dns_Record::CNAME, '', 1234 ),
                 array( self::SW_DOMAIN, 'test1', Dns_Record::CNAME, null, 1234 ),
                 array( self::SW_DOMAIN, 'test1', Dns_Record::CNAME, array(), 1234 ),
                 array( self::SW_DOMAIN, 'toolonglabel', Dns_Record::CNAME, 'thislabelistoolongbyfarF6B5D91AA8BDE43F8825D98AC88AAE9276F09EC357E3721A84065625961A34E9.domain.com', 1234 )
                 );
    }

    /**
     * Dataprovider.
     *
     * @return array
     */
    public function getBadTargetDataForTxtType()
    {
        return array( array( self::SW_DOMAIN, 'test1', Dns_Record::TXT, '', 1234 ),
                  array( self::SW_DOMAIN, 'test1', Dns_Record::TXT, null, 1234 ),
                  array( self::SW_DOMAIN, 'test1', Dns_Record::TXT, array(), 1234 )
                  );
    }

    /**
     * Setup func.
     */
    public function testSetup()
    {
        $this->deleteSWAccountIfNecessary();
    }

    /**
     * Test.
     *
     * @param string  $domain
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     *
     * @dataProvider getGoodRecordData
     */
    public function testAddActionReturnsTrueForValidData(
        $domain,
        $host,
        $type,
        $target,
        $ttl
    ) {

        $Reply = $this->_addRecord($domain, $host, $type, $target, $ttl);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testSrvRecordAdd()
    {
        $SW = $this->getSWAccount();

        $input = array( 'type'     => Dns_Record::SRV,
                    'zone_id'  => $SW->getWorkingZone()->getId(),
                    'service'  => 'butt',
                    'protocol' => 'butt',
                    'domain'   => self::SW_DOMAIN,
                    'ttl'      => 1234,
                    'priority' => 12,
                    'weight'   => 2,
                    'port'     => 23456,
                    'target'   => 'google.com' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'addSRV', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testDeleteActionReturnsTrueForValidData()
    {
        $SW = $this->getSWAccount();

        $records = $SW->getSimpleDnsRecordsFor(self::SW_DOMAIN);

        $cnt = count($records);
        $this->assertGreaterThan(0, $cnt);

        foreach ($records as $record) {
            if ($record['type'] === Dns_Record::ADDRESS &&
            $record['host'] === self::SW_DOMAIN ) {
                continue;
            }

            $id = $record['record_id'];
            break;
        }//end foreach record

        $input = array( 'record_id' => $id );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @param string  $domain
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider getBadRecordTypeData
     */
    public function testAddActionErrorsForBadData(
        $domain,
        $host,
        $type,
        $target,
        $ttl
    ) {

        $Reply = $this->_addRecord($domain, $host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $domain
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider getBadHostData
     */
    public function testAddActionErrorsForBadHostData(
        $domain,
        $host,
        $type,
        $target,
        $ttl
    ) {

        $Reply = $this->_addRecord($domain, $host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $domain
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider getBadTargetDataForAddressType
     */
    public function testAddActionErrorsForBadAddressTargetData(
        $domain,
        $host,
        $type,
        $target,
        $ttl
    ) {

        $Reply = $this->_addRecord($domain, $host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $domain
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider getBadTargetDataForTxtType
     */
    public function testAddActionErrorsForBadTxtTargetData(
        $domain,
        $host,
        $type,
        $target,
        $ttl
    ) {

        $Reply = $this->_addRecord($domain, $host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Test.
     *
     * @param string  $domain
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @dataProvider getBadTargetDataForCnameType
     */
    public function testAddActionErrorsForBadCnameTargetData(
        $domain,
        $host,
        $type,
        $target,
        $ttl
    ) {

        $Reply = $this->_addRecord($domain, $host, $type, $target, $ttl);
        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Add a record.
     *
     * @param string  $domain
     * @param string  $host
     * @param string  $type
     * @param string  $target
     * @param integer $ttl
     * @return Reply
     */
    private function _addRecord($domain, $host, $type, $target, $ttl)
    {

        $SW = $this->getSWAccount();

        $input = array( 'domain' => $domain,
                    'host' => $host,
                    'type' => $type,
                    'target' => $target,
                    'ttl' => $ttl );

        return IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);
    }
}
