<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_DomainsSlave.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_DomainsSlaveTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_DomainsSlave';

    /**
     * Deletes the SW account if necessary.
     */
    public function testSetup()
    {
        $data = $this->getSWIndexData(self::CTRL);

        if (count($data['iw_payload']->getData()) > 0) {
            $this->deleteSWAccountIfNecessary();
        }
    }

    /**
     * Tests that no secondary domains exist already.
     */
    public function testInitialDomainCount()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(
            1,
            $data['iw_payload']->getData(),
            'Secondary domains already exist'
        );
    }

    /**
     * Tests that addAction fails without input.
     */
    public function testAddFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'add', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that addAction returns true.
     */
    public function testAddReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the add action seemed to work.
     */
    public function testAddActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(2, $data['iw_payload']->getData());
    }

    /**
     * Test adding subdomain on a slave.
     */
    public function testAddingSubdomainOnSlave()
    {
        $SW = $this->getSWAccount();
        $SW->setWorkingDomain(new Domain(self::SW_SECONDARY_DOMAIN));
        $ctrl  = 'Ctrl_Siteworx_DomainsSub';
        $input = array( 'prefix' => 'mysub' );

        $Reply = IW::FC()->routeFromPHP($ctrl, 'add', $input, $SW);

        $SW->setWorkingDomain(new Domain(self::SW_DOMAIN));
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test adding a pointer domain on a slave.
     */
    public function testAddingPointerOnSlave()
    {
        $SW = $this->getSWAccount();
        $SW->setWorkingDomain(new Domain(self::SW_SECONDARY_DOMAIN));
        $ctrl  = 'Ctrl_Siteworx_DomainsPointer';
        $input = array( 'domain' => 'myptr.com' );

        $Reply = IW::FC()->routeFromPHP($ctrl, 'add', $input, $SW);

        $SW->setWorkingDomain(new Domain(self::SW_DOMAIN));
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test deleting subdomain on a slave.
     */
    public function testDeletingSubdomainOnSlave()
    {
        $SW = $this->getSWAccount();
        $SW->setWorkingDomain(new Domain(self::SW_SECONDARY_DOMAIN));
        $ctrl  = 'Ctrl_Siteworx_DomainsSub';
        $input = array( 'prefix' => 'mysub' );

        $Reply = IW::FC()->routeFromPHP($ctrl, 'delete', $input, $SW);

        $SW->setWorkingDomain(new Domain(self::SW_DOMAIN));
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test deleting a pointer domain on a slave.
     */
    public function testDeletingPointerOnSlave()
    {
        $SW = $this->getSWAccount();
        $SW->setWorkingDomain(new Domain(self::SW_SECONDARY_DOMAIN));
        $ctrl  = 'Ctrl_Siteworx_DomainsPointer';
        $input = array( 'domain' => 'myptr.com' );

        $Reply = IW::FC()->routeFromPHP($ctrl, 'delete', $input, $SW);

        $SW->setWorkingDomain(new Domain(self::SW_DOMAIN));
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that deleteAction fails without input.
     */
    public function testDeleteFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'delete', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that deleteAction returns true.
     */
    public function testDeleteReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the delete action seemed to work.
     */
    public function testDeleteActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(1, $data['iw_payload']->getData());
    }

    /**
     * Tests that the add action worked without a DNS Zone present.
     */
    public function testAddActionWorkedWithoutDnsZone()
    {
        $NW = $this->getMasterNodeWorx();
        $where  = 'domain = ' . q(self::SW_DOMAIN);
        $SWZone = new Dns_Zone($where);
        $SWZone->delete();
        $SWZone = new Dns_Zone($where);
        $this->assertEquals(-1, $SWZone->getId());

        $SW    = $this->getSWAccount();
        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);
        $this->assertReplySuccessful($Reply);

        $this->deleteSWAccountIfNecessary();
    }

    /**
     * Tests that changing the IP properly updates the vhost.
     */
    public function testIPChangeUpdatesVhost()
    {
        $NW     = $this->getMasterNodeWorx();
        $SW     = $this->getSWAccount();
        $domain = self::SW_DOMAIN;
        $vhost  = Ini::get(Ini::APACHE_DIR, 'conf.d') . "/vhost_{$domain}.conf";
        $cat    = Ini::get(Ini::BIN, 'cat');
        $grep   = Ini::get(Ini::BIN, 'grep');

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);

        $ip1 = $SW->getIP();
        $ip2 = self::_getFirstAvailableDedicatedIP();

        $input = array( 'domain' => self::SW_DOMAIN,
                    'ipv4'   => $ip2 );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'addIp',
            $input,
            $NW
        );

        $this->assertReplySuccessful($Reply);

        $cmd = "{$cat} {$vhost} | {$grep} {$ip1}";
        $Command = new Command($cmd);
        $Command->allowPipe();
        $Command->exec();

        $this->assertTrue($Command->wasSuccessful());

        $input = array( 'domain' => self::SW_DOMAIN,
                    'ipv4'   => $ip2 );
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_DomainsSlave',
            'edit',
            $input,
            $SW
        );

        $this->assertReplySuccessful($Reply);

        $cmd = "{$cat} {$vhost} | {$grep} {$ip2}";
        $Command = new Command($cmd);
        $Command->allowPipe();
        $Command->exec();

        $this->assertTrue($Command->wasSuccessful());
    }
}
