<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Jul 9, 2009
 */

/**
 * Tests for Ctrl_Siteworx_EmailBox.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_EmailDeletionTest extends IWorx_Framework_TestCase
{
    const CTRL_BOX   = 'Ctrl_Siteworx_EmailBox';
    const CTRL_ALIAS = 'Ctrl_Siteworx_EmailAlias';
    const CTRL_GROUP = 'Ctrl_Siteworx_EmailGroup';
    const CTRL_AUTO  = 'Ctrl_Siteworx_EmailAutorespond';


    const BOX1   = 'box1';
    const PASS1  = 'box1pass';
    const BOX2   = 'box2';
    const PASS2  = 'box2pass';
    const ALIAS1 = 'alias1';
    const ALIAS2 = 'alias2';
    const GROUP1 = 'group1';
    const GROUP2 = 'group2';

    /**
     * This test needs a clean SW account each time.
     */
    public function testSetup()
    {
        $this->deleteSWAccountIfNecessary();
    }

    /**
     * Test.
     */
    public function testDeletingEmailBoxRemovesItselfFromGroupMembership()
    {
        $this->_addEmailBox(self::BOX1, self::PASS1);
        $members = array( self::BOX1 . '@' . self::SW_DOMAIN );
        $this->_addEmailGroup(self::GROUP1, $members);
        $this->_deleteEmailBox(self::BOX1);

        $SW   = $this->getSWAccount();
        $Vpop = $SW->getIncomingMailDriver();

        $this->assertEquals(array(), $Vpop->getGroupMembers(self::GROUP1));
        $this->_deleteEmailGroup(self::GROUP1);
    }

    /**
     * Test.
     */
    public function testDeletingEmailBoxRemovesItselfFromAliases()
    {
        $this->_addEmailBox(self::BOX1, self::PASS1);
        $forward = self::BOX1 . '@' . self::SW_DOMAIN;
        $this->_addEmailAlias(self::ALIAS1, $forward);
        $this->_deleteEmailBox(self::BOX1);

        $SW   = $this->getSWAccount();
        $Vpop = $SW->getIncomingMailDriver();

        $this->assertFalse($Vpop->aliasExists(self::ALIAS1));
    }

    /**
     * Test.
     */
    public function testDeletingEmailBoxRemovesItselfFromChainedAliases()
    {
        $this->markTestSkipped('Test skipped because fixing it is too hard for now');

        $this->_addEmailBox(self::BOX1, self::PASS1);

        $forward = self::BOX1 . '@' . self::SW_DOMAIN;
        $this->_addEmailAlias(self::ALIAS1, $forward);

        $forward2 = self::ALIAS1 . '@' . self::SW_DOMAIN;
        $this->_addEmailAlias(self::ALIAS2, $forward2);

        $this->_deleteEmailBox(self::BOX1);

        $SW   = $this->getSWAccount();
        $Vpop = $SW->getIncomingMailDriver();

        $this->assertFalse($Vpop->aliasExists(self::ALIAS1));
        $this->assertFalse($Vpop->aliasExists(self::ALIAS2));
    }

    /**
     * Test.
     */
    public function testDeletingEmailAliasRemovesItselfFromGroupMembership()
    {
        $this->_addEmailAlias(self::ALIAS1, 'nobody@inparticular.com');
        $members = array( self::ALIAS1 . '@' . self::SW_DOMAIN );
        $this->_addEmailGroup(self::GROUP1, $members);
        $this->_deleteEmailAlias(self::ALIAS1);

        $SW   = $this->getSWAccount();
        $Vpop = $SW->getIncomingMailDriver();

        $this->assertEquals(array(), $Vpop->getGroupMembers(self::GROUP1));
        $this->_deleteEmailGroup(self::GROUP1);
    }

    /**
     * Test.
     */
    public function testDeletingEmailGroupRemovesItselfFromAliases()
    {
        $members = array( 'nobody@inparticular.com' );
        $this->_addEmailGroup(self::GROUP1, $members);
        $forward = self::GROUP1 . '@' . self::SW_DOMAIN;
        $this->_addEmailAlias(self::ALIAS1, $forward);
        $this->_deleteEmailGroup(self::GROUP1);

        $SW   = $this->getSWAccount();
        $Vpop = $SW->getIncomingMailDriver();

        $this->assertFalse($Vpop->aliasExists(self::ALIAS1));
    }

    /**
     * Test.
     */
    public function testDeletingEmailGroupRemovesItselfFromGroupMembership()
    {
        $members = array( 'nobody@inparticular.com' );
        $this->_addEmailGroup(self::GROUP1, $members);
        $members2 = array( self::GROUP1 . '@' . self::SW_DOMAIN );
        $this->_addEmailGroup(self::GROUP2, $members2);
        $this->_deleteEmailGroup(self::GROUP1);

        $SW   = $this->getSWAccount();
        $Vpop = $SW->getIncomingMailDriver();

        $this->assertEquals(array(), $Vpop->getGroupMembers(self::GROUP2));
        $this->_deleteEmailGroup(self::GROUP2);
    }

    /**
     * Add email box helper.
     *
     * @param string $username
     * @param string $password
     */
    private function _addEmailBox($username, $password)
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'         => $username,
                    'password'         => $password,
                    'confirm_password' => $password );

        $Reply = IW::FC()->routeFromPHP(self::CTRL_BOX, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Add email group helper.
     *
     * @param string $username
     * @param array  $members
     */
    private function _addEmailGroup($username, array $members)
    {
        $SW = $this->getSWAccount();

        $input = array( 'username' => $username,
                    'forwardsto' => $members );

        $Reply = IW::FC()->routeFromPHP(self::CTRL_GROUP, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Add email alias helper.
     *
     * @param string $username
     * @param string $forwards_to
     */
    private function _addEmailAlias($username, $forwards_to)
    {
        $SW    = $this->getSWAccount();
        $input = array( 'username' => $username,
                    'forwardsto' => $forwards_to );

        $Reply = IW::FC()->routeFromPHP(self::CTRL_ALIAS, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Delete email box helper.
     *
     * @param string $username
     */
    private function _deleteEmailBox($username)
    {
        $SW     = $this->getSWAccount();
        $input  = array( 'username' => $username );
        $Reply = IW::FC()->routeFromPHP(self::CTRL_BOX, 'delete', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Delete email group helper.
     *
     * @param string $username
     */
    private function _deleteEmailGroup($username)
    {
        $SW     = $this->getSWAccount();
        $input  = array( 'username' => $username );
        $Reply = IW::FC()->routeFromPHP(self::CTRL_GROUP, 'delete', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Delete email alias helper.
     *
     * @param string $username
     */
    private function _deleteEmailAlias($username)
    {
        $SW     = $this->getSWAccount();
        $input  = array( 'username' => $username );
        $Reply = IW::FC()->routeFromPHP(self::CTRL_ALIAS, 'delete', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }
}
