<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_EmailFilters.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailFiltersTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_EmailFilters';

    /**
     * Clear the account if necessary.
     */
    public function testSetup()
    {
        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $fromFilters = $driver->getFromFilters();
        $subjFilters = $driver->getSubjectFilters();

        if (count($fromFilters) > 0 || count($subjFilters) > 0) {
            $this->deleteSWAccountIfNecessary();
        }
    }

    /**
     * Tests the output of the index action.
     */
    public function testIndexActionReturnsArray()
    {
        $output = $this->getSWIndexData(self::CTRL);

        $this->assertInternalType('array', $output);
    }

    /**
     * Tests that there are no from filters initially.
     */
    public function testInitialFromFilterCount()
    {
        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $fromFilters = $driver->getFromFilters();

        $this->assertCount(0, $fromFilters, 'From filters already exist!');
    }

    /**
     * Tests that there are no subject filters initially.
     */
    public function testInitialSubjectFilterCount()
    {
        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $subjectFilters = $driver->getSubjectFilters();

        $this->assertCount(0, $subjectFilters, 'Subject filters already exist!');
    }

    /**
     * Tests that addFromFilterAction fails without input.
     */
    public function testAddFromFilterFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'addFrom', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that addFromFilterAction returns true.
     */
    public function testAddFromFilterReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'from-filter' => 'bob@komta.com' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'addFrom', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that addFromAction actually worked.
     */
    public function testAddFromFilterWorked()
    {
        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $fromFilters = $driver->getFromFilters();

        $this->assertCount(1, $fromFilters, 'From filters not added');
    }

    /**
     * Tests that addSubjFilterAction fails without input.
     */
    public function testAddSubjFilterFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'addSubject', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that addSubjFilterAction returns true.
     */
    public function testAddSubjFilterReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'subj-filter' => 'viagra' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'addSubject', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that addSubjectAction actually worked.
     */
    public function testAddSubjectFilterWorked()
    {
        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $subjFilters = $driver->getFromFilters();

        $this->assertCount(1, $subjFilters, 'Subject filters not added');
    }


    /**
     * Tests that deleteFiltersAction returns true.
     */
    public function testDeleteFiltersReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $fromFilters = $driver->getFromFilters();
        $subjFilters = $driver->getSubjectFilters();

        $input = array( 'delete_from'     => $fromFilters,
                    'delete_subjects' => $subjFilters );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that the subject filter was deleted.
     */
    public function testDeleteFiltersDeletedSubjectFilters()
    {
        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $subjFilters = $driver->getSubjectFilters();

        $this->assertCount(0, $subjFilters, 'Subject Filters not deleted');
    }

    /**
     * Tests that the From filter was deleted.
     */
    public function testDeleteFiltersDeletedFromFilters()
    {
        $driver = $this->getSWAccount()->getIncomingMailDriver();

        $fromFilters = $driver->getFromFilters();

        $this->assertCount(0, $fromFilters, 'From Filters not deleted');
    }

    /**
     * Tests that filterBehaviorAction returns true.
     */
    public function testFilterBehaviorReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'filter_action' => 'drop' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'filterBehavior', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that filterBehaviorAction returns true.
     */
    public function testFilterBehaviorReturnsTrueAgain()
    {
        $SW = $this->getSWAccount();

        $input = array( 'filter_action' => 'spam' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'filterBehavior', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }
}
