<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_EmailGroup.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailGroupTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_EmailGroup';

    const USER = 'forward';

    /**
     * Clears the account if needed.
     */
    public function testSetup()
    {
        $data_group = $this->getSWIndexData(self::CTRL);
        $data_alias = $this->getSWIndexData('Ctrl_Siteworx_EmailAlias');

        if (count($data_group['iw_groups']) !== 0 ||
        count($data_alias['iw_aliases']) !== 0 ) {
            $this->deleteSWAccountIfNecessary();
        }
    }

    /**
     * Tests the output of the index action.
     */
    public function testIndexActionReturnsArray()
    {
        $output = $this->getSWIndexData(self::CTRL);

        $this->assertInternalType('array', $output);
    }

    /**
     * Tests that there are initially no groups.
     */
    public function testInitialGroupCount()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(0, $data['iw_payload']->getData(), 'Groups already exist');
    }

    /**
     * Tests that addAction fails without input.
     */
    public function testAddFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'add', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that addAction returns true.
     */
    public function testAddReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'   => self::USER,
                    'forwardsto' => array( self::SW_EMAIL,
                                           self::SW_SECONDARY_EMAIL ) );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the add action seemed to work.
     */
    public function testAddActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(1, $data['iw_payload']->getData());
    }

    /**
     * Tests that editAction fails without input.
     */
    public function testEditFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'edit', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that editAction returns true.
     */
    public function testEditReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'   => self::USER,
                    'forwardsto' => array( self::SW_EMAIL,
                                           self::SW_SECONDARY_EMAIL,
                                           'tim@komta.com' ) );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the edit action seemed to work.
     */
    public function testEditActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);
        $groups = $data['iw_payload']->getData();
        $this->assertCount(
            3,
            explode(',', $groups[0]->members),
            'Extra user not added'
        );
    }

    /**
     * Tests that deleteAction fails without input.
     */
    public function testDeleteFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'delete', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that deleteAction returns true.
     */
    public function testDeleteReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username' => self::USER );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the delete action seemed to work.
     */
    public function testDeleteActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(0, $data['iw_payload']->getData());
    }

    /**
     * Test.
     */
    public function testGroupAliasInteraction()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'   => self::USER,
                    'forwardsto' => array( self::SW_EMAIL ) );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);

        $input = array( 'username'   => 'testalias',
                    'forwardsto' => self::USER . '@' . self::SW_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_EmailAlias',
            'add',
            $input,
            $SW
        );

        $this->assertReplySuccessful($Reply);

        $input = array( 'username'   => self::USER,
                    'forwardsto' => array( self::SW_EMAIL ) );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);
        $this->assertReplySuccessful($Reply);

        $group_data = $this->getSWIndexData(self::CTRL);
        $this->assertCount(1, $group_data['iw_payload']->getData());

        $alias_data = $this->getSWIndexData('Ctrl_Siteworx_EmailAlias');
        $this->assertCount(1, $alias_data['iw_payload']->getData());
    }

    /**
     * Test.
     */
    public function testCleanUp()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username' => 'testalias' );
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_EmailAlias',
            'delete',
            $input,
            $SW
        );
        $this->assertReplySuccessful($Reply);

        $input = array( 'username' => self::USER );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }
}
