<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_EmailSpamprefs.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailSpamprefsTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_EmailSpamprefs';

    const PREFERENCE = 'bayes_ignore_to';

    /**
     * Gets the index data about the preference.
     *
     * @return array|false
     */
    private function _getPreferenceData()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $prefs = $data['iw_spamprefs'];

        $preference = false;

        foreach ($prefs as $pref) {
            if ($preference === false) {
                if ($pref['preference'] === self::PREFERENCE) {
                    $preference = $pref;
                }
            }
        }

        return $preference;
    }

    /**
     * Deletes the SW account if the preference exists.
     */
    public function testSetup()
    {
        if ($this->_getPreferenceData() !== false) {
            $this->deleteSWAccountIfNecessary();
        }
    }

    /**
     * Tests the output of the index action.
     */
    public function testIndexActionReturnsArray()
    {
        $output = $this->getSWIndexData(self::CTRL);

        $this->assertInternalType('array', $output);
    }

    /**
     * Tests that editAction works even without input (the defaults are set)
     */
    public function testEditSucceedsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', array(), $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that editAction returns true.
     */
    public function testEditReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'enable_spam' => 1 );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the edit action seemed to work.
     */
    public function testEditActionWorked()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();
        $this->assertTrue($Driver->isSpamEnabled());
    }

    /**
     * Tests that addAdvancedAction fails without input.
     */
    public function testAddAdvancedFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'addAdvanced', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that addAdvancedAction returns true.
     */
    public function testAddAdvancedReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'preference' => self::PREFERENCE,
                    'value'      => self::SW_EMAIL );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'addAdvanced', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Checks for the added preference.
     */
    public function testAddAdvancedWorked()
    {
        $pref = $this->_getPreferenceData();

        $this->assertInternalType('array', $pref);
    }

    /**
     * Tests that editAdvancedAction fails without input.
     */
    public function testEditAdvancedFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'editAdvanced', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that editAdvancedAction returns true.
     */
    public function testEditAdvancedReturnsTrue()
    {
        $pref = $this->_getPreferenceData();
        $id   = $pref['id'];
        $SW   = $this->getSWAccount();

        $input = array( 'pref_id' => $id,
                    'value'   => self::SW_SECONDARY_EMAIL );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'editAdvanced', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Verifies that the preference was edited successfully.
     */
    public function testEditAdvancedWorked()
    {
        $preference = $this->_getPreferenceData();

        $this->assertEquals(self::SW_SECONDARY_EMAIL, $preference['value']);
    }

    /**
     * Tests that deleteAdvancedAction fails without input.
     */
    public function testDeleteAdvancedFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'deleteAdvanced', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that deleteAdvancedAction returns true.
     */
    public function testDeleteAdvancedReturnsTrue()
    {
        $pref = $this->_getPreferenceData();
        $id   = $pref['id'];
        $SW   = $this->getSWAccount();

        $input = array( 'pref_id' => $id );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'deleteAdvanced', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Checks that the preference was deleted.
     */
    public function testDeleteAdvancedWorked()
    {
        $pref = $this->_getPreferenceData();

        $this->assertFalse($pref);
    }
}
