<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 25, 2009
 */

/**
 * Tests for Ctrl_Siteworx_Email.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailTest extends IWorx_Framework_TestCase
{
    const CTRL     = 'Ctrl_Siteworx_Email';
    const CATCHALL = 'dummy@dummydomain.com';

    /**
     * Gets the data from the index action.
     *
     * @return array
     */
    private function _getIndexData()
    {
        $SW = $this->getSWAccount();

        return IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $SW)->getData();
    }

    /**
     * Test has bounce form.
     */
    public function testIndexVarHasBounceForm()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_form', $data);
        $this->assertArrayHasKey('bounce', $data['iw_form']);

        $BounceForm = $data['iw_form']['bounce'];
        $this->assertTrue($BounceForm instanceof Form_SW_Email_Bounce);
    }

    /**
     * Tests for the presence of iw_email_boxes_allowed.
     */
    public function testIndexVarEmailBoxesAllowed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_email_boxes_allowed', $data);
    }

    /**
     * Tests for the presence of iw_alias_allowed.
     */
    public function testIndexVarEmailAliasAllowed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_alias_allowed', $data);
    }

    /**
     * Tests for the presence of iw_group_allowed.
     */
    public function testIndexVarGroupAllowed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_group_allowed', $data);
    }

    /**
     * Tests for the presence of iw_autorespond_allowed.
     */
    public function testIndexVarAutoRespondAllowed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_autorespond_allowed', $data);
    }

    /**
     * Tests for the presence of iw_box_used.
     */
    public function testIndexVarBoxUsed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_box_used', $data);
    }

    /**
     * Tests for the presence of iw_alias_used.
     */
    public function testIndexVarAliasUsed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_alias_used', $data);
    }

    /**
     * Tests for the presence of iw_group_used.
     */
    public function testIndexVarGroupUsed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_group_used', $data);
    }

    /**
     * Tests for the presence of iw_autorespond_used.
     */
    public function testIndexVarAutorespondUsed()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_autorespond_used', $data);
    }

    /**
     * Tests for the presence of iw_box_usedpct.
     */
    public function testIndexVarBoxUsedPct()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_box_usedpct', $data);
    }

    /**
     * Tests for the presence of iw_alias_usedpct.
     */
    public function testIndexVarAliasUsedPct()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_alias_usedpct', $data);
    }

    /**
     * Tests for the presence of iw_group_usedpct.
     */
    public function testIndexVarGroupUsedPct()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_group_usedpct', $data);
    }

    /**
     * Tests for the presence of iw_autorespond_usedpct.
     */
    public function testIndexVarAutorespondUsedPct()
    {
        $data = $this->_getIndexData();
        $this->assertArrayHasKey('iw_autorespond_usedpct', $data);
    }

    /**
     * Tests that iw_box_usedpct is a sane percentage.
     */
    public function testIndexVarBoxUsedPctSanity()
    {
        $data = $this->_getIndexData();
        $this->assertBetween(0, 100, $data['iw_box_usedpct']);
    }

    /**
     * Tests that iw_alias_usedpct is a sane percentage.
     */
    public function testIndexVarAliasUsedPctSanity()
    {
        $data = $this->_getIndexData();
        $this->assertBetween(0, 100, $data['iw_alias_usedpct']);
    }

    /**
     * Tests that iw_group_usedpct is a sane percentage.
     */
    public function testIndexVarGroupUsedPctSanity()
    {
        $data = $this->_getIndexData();
        $this->assertBetween(0, 100, $data['iw_group_usedpct']);
    }

    /**
     * Tests that iw_autorespond_usedpct is a sane percentage.
     */
    public function testIndexVarAutorespondUsedPctSanity()
    {
        $data = $this->_getIndexData();
        $this->assertBetween(0, 100, $data['iw_autorespond_usedpct']);
    }

    /**
     * Tests the initial value of bounce.
     */
    public function testEmailBounceInitialValue()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        $is_bounce_enabled = $Driver->isBounceEnabled();

        $this->assertTrue($is_bounce_enabled, 'Bounce was disabled by default!');
    }

    /**
     * Tests disabling bounce.
     */
    public function testDisableBounce()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        $input = array( 'enablebounce' => '0' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'editBounce', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Checks that disabling bounce turned it off.
     */
    public function testDisableBounceActuallyDisabledBounce()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        $is_bounce_enabled = $Driver->isBounceEnabled();

        $this->assertFalse($is_bounce_enabled, 'Bounce is still enabled!');
    }

    /**
     * Tests re-enabling bounce.
     */
    public function testEnableBounce()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        $input = array( 'enablebounce' => '1' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'editBounce', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that the enabling actually turned it on.
     */
    public function testEnableBounceActuallyEnabledBounce()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        $is_bounce_enabled = $Driver->isBounceEnabled();

        $this->assertTrue($is_bounce_enabled, 'Bounce is not enabled!');
    }

    /**
     * Test enabling of the plugin.
     */
    public function testEnableCatchallPlugin()
    {
        Ini::set(Ini::PLUGINS, 'catchall-account-option', 1);
        Ini::write();
        IW::PluginManager()->clearInstance();
        Ini::reload();
        $this->assertTrue(IW::PluginManager()
                       ->getPlugin('catchall-account-option')->isEnabled());
        $Plugin = IW::PluginManager()->getPlugin('catchall-account-option');
        $uniq   = $this->getSWAccount()->getUniqname();
        $Plugin->unblockUser($uniq);
        $this->assertFalse($Plugin->userIsBlocked($uniq));
    }

    /**
     * Tests setting the catch all.
     */
    public function testSettingCatchAll()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        $input = array( 'enablebounce' => self::CATCHALL );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'editBounce', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Checks to makesure the catchall is properly set.
     */
    public function testCatchallActuallySet()
    {
        $SW      = $this->getSWAccount();
        $Driver  = $SW->getIncomingMailDriver();
        $aliases = $Driver->getAliases();

        $catchall = '';
        foreach ($aliases as $alias) {
            if ($Driver->isCatchAll($alias->username)) {
                $catchall = $alias->dest;
                break;
            }
        }

        $this->assertTrue(
            $catchall == self::CATCHALL,
            'Catchall not set properly'
        );
    }

    /**
     * Tests setting the catch all.
     */
    public function testUnsettingCatchAll()
    {
        $SW     = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        $input = array( 'enablebounce' => '1' );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'editBounce', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Checks to makesure the catchall is properly set.
     */
    public function testCatchallActuallyUnset()
    {
        $SW      = $this->getSWAccount();
        $Driver  = $SW->getIncomingMailDriver();
        $aliases = $Driver->getAliases();

        $catchall = '';
        foreach ($aliases as $alias) {
            if ($Driver->isCatchAll($alias->username)) {
                $catchall = $alias->dest;
                break;
            }
        }

        $this->assertTrue($catchall == '', 'Catchall not set properly');
    }

    /**
     * Test disabling of the plugin.
     */
    public function testDisableCatchallPlugin()
    {
        Ini::set(Ini::PLUGINS, 'catchall-account-option', 0);
        Ini::write();
        IW::PluginManager()->clearInstance();
        Ini::reload();
        $this->assertFalse(IW::PluginManager()
                        ->getPlugin('catchall-account-option')->isEnabled());
    }
}
