<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Jan 21, 2010
 */

/**
 * SiteWorx Htaccess Controller Test.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_HtaccessTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_Htaccess';
    const DIR  = '/home/addedbyt/addedbytest.com/html';

    /**
     * Actions used in routeFromPHP.
     */
    const INDEX = 'index';
    const DELETE = 'delete';
    const ADD = 'add';
    const ENABLE = 'enable';
    const DISABLE = 'disable';
    const HOTLINK = 'hotlink';

    /**
     * Setup before class.
     */
    public static function setUpBeforeClass()
    {
        self::deleteSWAccountIfNecessaryStatic();
        self::addSWAccountIfNecessary();
    }

    /**
     * Tests the assigned index vars.
     */
    public function testIndexActionAssignedVars()
    {
        $action = self::INDEX;
        $input = array();

        $Reply  = $this->_getReply($action, $input);
        $result = $Reply->getData();

        $this->assertArrayHasKey('iw_htaccess_status', $result);
        $this->assertArrayHasKey('iw_htaccess_actions', $result);
        $this->assertArrayHasKey('iw_mod_suphp', $result);
        $this->assertArrayHasKey('iw_htaccess_mgr_info', $result);
    }



    /**
     * Test.
     */
    public function testAddAction()
    {
        $action = self::ADD;
        $input = array('new_dir' => self::DIR);
        $Reply  = $this->_getReply($action, $input);


        if ($Reply->wasSuccessful() === false) {
            $error_message = $Reply->getErrorsAsString();
            echo "\nerror_message: \n";
            var_export($error_message);
            echo "\niworx.log:\n";
            var_export(`cat /home/interworx/var/log/iworx.log`);
            echo "\naddedbytest files:\n";
            var_export(`find /home/addedbyt/ -ls`);
        }
        $this->assertReplySuccessful($Reply);
    }


    /**
     * Test.
     * Must run after addAction & before delete - file must exist.
     */
    public function testHotlinkCommitAction()
    {
        // should accept both http & https
        $urls = ['http://test.com', 'https://test.com'];
        $action = self::HOTLINK;
        $input = array('dir' => self::DIR, 'status' => 'on', 'allow_urls' => $urls);

        $Reply = $this->_getReply($action, $input);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testHtaccessPermissions()
    {
        $expected_perms = decoct(HtaccessManager::PERMS);

        $stat           = IWorxFileSys::stat(self::DIR . '/.htaccess');
        $actual_perms   = decoct($stat['mode'] & 0777);

        $this->assertEquals($expected_perms, $actual_perms);
    }

    /**
     * Test.
     */
    public function testDisableAction()
    {
        $action = self::DISABLE;
        $input = array('dir' => self::DIR);
        $Reply  = $this->_getReply($action, $input);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testEnableAction()
    {
        $action = self::ENABLE;
        $input = array('dir' => self::DIR);
        $Reply  = $this->_getReply($action, $input);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testDeleteAction()
    {
        $action = self::DELETE;
        $input = array('htaccess_dirs' => self::DIR);

        $Reply  = $this->_getReply($action, $input);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Runs a controller action.
     *
     * @param string $action
     * @param array  $input
     * @return Reply
     */
    private function _getReply($action, array $input)
    {
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            $action,
            $input,
            $this->getSWAccount()
        );

        return $Reply;
    }
}
