<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jun 30, 2009
 */

/**
 * TestCase for SslKey controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_SslKeyTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_SslKey';

    /**
     * Test index assigned vars.
     */
    public function testIndexActionReturnsArray()
    {
        $this->deleteSWAccountIfNecessary();

        // add siteworx account on a dedi ip
        $NW    = self::getMasterNodeWorx();
        $input = self::_getSWAccountInput();
        $input['master_domain_ipv4'] = self::_getFirstAvailableDedicatedIP();
        IW::FC()->routeFromPHP('Ctrl_Nodeworx_Siteworx', 'add', $input, $NW);

        $SW     = $this->getSWAccount();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $SW);
        $output = $Reply->getData();

        $this->assertInternalType('array', $output);
        $this->assertInternalType('array', $output['iw_form']);

        $GenerateForm = $output['iw_form']['generate'];
        $this->assertTrue($GenerateForm instanceof Form_SW_SslKey_Generate);
    }

    /**
     * Test generate action with 2048 bit key.
     */
    public function testGenerateAction()
    {
        $SW = $this->getSWAccount();
        $input  = array( 'key_length' => 2048 );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'generate', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', array(), $SW);
        $this->assertReplySuccessful($Reply);

        $input  = array( 'key_length' => 2048 );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'generate', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     */
    public function testCleanup()
    {
        $this->deleteSWAccountIfNecessary();
    }
}
