<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_Ssl.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_SslTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_Ssl';

    /**
     * Setup.
     */
    public static function setUpBeforeClass()
    {
        self::deleteSWAccountIfNecessaryStatic();

        self::addSWAccount($use_dedicated_ip = true);
    }

    /**
     * Tests the output of the index action.
     */
    public function testIndexActionReturnsArray()
    {
        $output = $this->getSWIndexData(self::CTRL);

        $this->assertInternalType('array', $output);
    }

    /**
     * Test.
     */
    public function testSslKeyGenerateReturnsTrue()
    {
        $SW    = $this->getSWAccount();
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslKey',
            'generate',
            array( 'key_length' => '2048' ),
            $SW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @depends testSslKeyGenerateReturnsTrue
     */
    public function testSSKeyCsrReturnsTrue()
    {
        $SW    = $this->getSWAccount();
        $input = array( 'countryName'            => 'US',
                    'stateOrProvinceName'    => 'Pennsylvania',
                    'localityName'           => 'Pittsburgh',
                    'organizationName'       => 'InterWorx, LLC',
                    'organizationalUnitName' => 'SE',
                    'commonName'             => self::SW_DOMAIN,
                    'emailAddress'           => self::SW_EMAIL );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslCsr',
            'generate',
            $input,
            $SW
        );

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test.
     *
     * @depends testSSKeyCsrReturnsTrue
     */
    public function testSSLKeyCrtReturnsTrue()
    {
        $SW    = $this->getSWAccount();
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslCrt',
            'generate',
            array(),
            $SW
        );

        $this->assertReplySuccessful($Reply);
    }
}
