<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Aug 30, 2010
 */

/**
 * Regression testing, class that tests that adding custom entries to a domain's.
 * vhost file don't end up breaking the vhost directives.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CustomVhostDoesntBreakApacheTest extends IWorx_Framework_TestCase
{

    /**
     * Setup.
     */
    public function setUp()
    {
        self::deleteSWAccountIfNecessary();
        self::addSWAccount($use_dedicated_ip = true);
    }

    /**
     * Test.
     */
    public function testPointerDomainDoesntBreakVhost()
    {
        $SW = $this->getSWAccount();
        $ip = $SW->getIP();

        $domain = self::SW_DOMAIN;
        $confd  = Ini::get(Ini::APACHE_DIR, 'conf.d');

        $vhost = file_get_contents("{$confd}/vhost_{$domain}.conf");

        $pattern = '\<VirtualHost ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+):80\>';
        $replace = "Listen {$ip}:8000\n" . '<VirtualHost $1:80 $1:8000>';
        $vhost   = preg_replace("/$pattern/", $replace, $vhost);

        file_put_contents("{$confd}/vhost_{$domain}.conf", $vhost);

        $SW->getHTTPDriver()->restart();

        $this->_installSSLCert();

        $vhost   = file_get_contents("{$confd}/vhost_{$domain}.conf");
        $pattern = '(\<VirtualHost .+\>)';

        preg_match_all("/$pattern/", $vhost, $matches);

        $expected_80  = "<VirtualHost $ip:80 $ip:8000>";
        $expected_443 = "<VirtualHost $ip:443>";

        $this->assertEquals($expected_80, $matches[1][0]);
        $this->assertEquals($expected_443, $matches[1][1]);

        $this->assertTrue($SW->getHTTPDriver()->httpdCheckSyntax());
    }

    /**
     * Test.
     */
    private function _installSSLCert()
    {
        $domain = self::SW_DOMAIN;
        $confd  = Ini::get(Ini::APACHE_DIR, 'conf.d');

        $SW    = $this->getSWAccount();
        $input = array( 'countryName'            => 'US',
                    'stateOrProvinceName'    => 'Pennsylvania',
                    'localityName'           => 'Pittsburgh',
                    'organizationName'       => 'InterWorx, LLC',
                    'organizationalUnitName' => 'SE',
                    'commonName'             => self::SW_DOMAIN,
                    'emailAddress'           => self::SW_EMAIL );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslKey',
            'generate',
            array( 'key_length' => '2048' ),
            $SW
        );

        $this->assertReplySuccessful($Reply);

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslCsr',
            'generate',
            $input,
            $SW
        );

        $this->assertReplySuccessful($Reply);

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_SslCrt',
            'generate',
            array(),
            $SW
        );

        $this->assertReplySuccessful($Reply);
    }
}
