<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Oct 22, 2018
 */

/**
 * cPanel DbMapper Import test.
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class Import_CpanelImportDbMapperTest extends IWorx_Framework_TestCase
{
    const CP_DOMAIN = 'cimport.com';
    const NO_PREFIX_DB_NAME = 'nopdb';
    const NO_PREFIX_USER_NAME = 'nopuser';
    const PREFIX_DB_NAME = 'cimport_prefix';
    const PREFIX_USER_NAME = 'cimport_prefix';
    const BACKUP_LOCATION = '/home/interworx/TESTS/_resources/cpanelDbMappertest.tgz';

    /**
     * Test.
     */
    public function testImportingCpanelBackupWithNoPrefixDatabaseAndUserNames()
    {
        $this->deleteSWAccountIfNecessary();
        if ($this->getSWAccountExists(self::CP_DOMAIN)) {
            self::deleteSWAccountStatic(self::CP_DOMAIN);
        }

        $ip      = self::_getFirstSharedIP();
        $type    = 'cpanel';
        $archive = self::BACKUP_LOCATION;

        $cmd = Ini::get(Ini::IWORX_BIN, 'import');
        $cmd .= " --archive={$archive} --ipv4={$ip} --control-panel={$type} --force";

        IWorxExec::exec($cmd, $result, $retval);

        $Driver = $this->_getDriver();

        //databases
        $this->assertTrue($Driver->databaseExistsFQ(self::PREFIX_DB_NAME));
        $this->assertTrue($Driver->databaseExistsFQ(self::NO_PREFIX_DB_NAME));

        //users
        $this->assertTrue($Driver->userExistsFQ(self::NO_PREFIX_USER_NAME));
        $this->assertTrue($Driver->userExistsFQ(self::PREFIX_USER_NAME));

        //should be associated with nopuser and cimport_prefix
        $perms1 = [0 => 'INDEX',
                   1 =>'INSERT'];

        //should be associated with cimport_prefix and nopdb
        $perms2 = [0 => 'CREATE',
                   1 => 'SELECT'];

        $sort_perms1 = sort($perms1);
        $sort_perms2 = sort($perms2);

        //permissions
        $this->assertEquals(sort($Driver->getPermissionsFQ(self::NO_PREFIX_USER_NAME, self::PREFIX_DB_NAME)), $sort_perms1);
        $this->assertEquals(sort($Driver->getPermissionsFQ(self::PREFIX_USER_NAME, self::NO_PREFIX_DB_NAME)), $sort_perms2);
    }
    /**
     * Gets the Database Driver.
     *
     * @return MySQL
     */
    private function _getDriver()
    {

        $SiteWorx = new SiteWorx(self::CP_DOMAIN);
        $Driver   = $SiteWorx->getMySQLDriver();

        return $Driver;
    }
}
