<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Oct 11, 2013
 */

/**
 * Cpanel import dns record tests.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Import_CpanelImportDnsRecordsTest extends IWorx_Framework_TestCase
{

    const BACKUP_FILE = '/home/interworx/TESTS/_resources/cpanel-backup-4.30.2014_18-06-04_bug-3578.tar.gz';
    const MASTER_DOM  = 'notwell.net';

    /**
     * Test.
     */
    public function testImportingDnsRecords()
    {
        $ip   = self::_getFirstSharedIP();
        $type = 'cpanel';
        $archive = self::BACKUP_FILE;

        $cmd = Ini::get(Ini::IWORX_BIN, 'import');
        $cmd .= " --archive={$archive} --ipv4={$ip} --control-panel={$type} --force";

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(0, $retval, "Importing {$archive} failed: " .
                         implode("\n", $result));

        $error = 'An error occurred while importing a slave domain';
        foreach ($result as $line) {
            $this->assertNotContains($error, $line, $line);
        }

        $Zone = new Dns_Zone('domain = ' . q(self::MASTER_DOM));

        $where = "zone_id = {$Zone->getId()}
              AND type = " . q(Dns_Record::SRV) . '
              AND host = ' . q('_sipfederationtls._tcp.notwell.net') . '
              AND target = ' . q('100 1 5061 sipfed.online.lync.com');

        $this->assertCount(
            1,
            $Zone->getRecords($where),
            'Could not find SRV record that should have imported.'
        );

        $where = "zone_id = {$Zone->getId()}
              AND type = " . q(Dns_Record::CNAME) . '
              AND host = ' . q('mtl.notwell.net') . '
              AND target = ' . q('notwell.dyndns.org');

        $this->assertCount(
            1,
            $Zone->getRecords($where),
            'Could not find CNAME record that should have imported.'
        );
    }
}
