<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Oct 11, 2013
 */

/**
 * Cpanel import tests.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Import_CpanelImportTest extends IWorx_Framework_TestCase
{

    const ADDON_BACKUP1 = '/home/interworx/TESTS/_resources/cpmove-granite.tar.gz';
    const ADDON_BACKUP1_MASTER_DOMAIN = 'thegraniteco.net';
    const ADDON_BACKUP1_ADDON_DOMAIN = 'thegraniteco.com';

    const SSL_BACKUP1 = '/home/interworx/TESTS/_resources/cpmove-washes.tar.gz';
    const SSL_BACKUP1_MASTER_DOMAIN = 'fasteddiescarwashes.com';

    const NO_PLAN_BACKUP =
    '/home/interworx/TESTS/_resources/cpmove-washes-no-plan.tar.gz';
    const NO_PLAN_MASTER_DOMAIN = 'fasteddiescarwashes.com';

    /**
     * Test.
     */
    public function testImportWithSpacesInPlanInfo()
    {
        $ip   = self::_getFirstSharedIP();
        $type = 'cpanel';
        $archive = self::NO_PLAN_BACKUP;

        $cmd = Ini::get(Ini::IWORX_BIN, 'import');
        $cmd .= " --archive={$archive} --ipv4={$ip} --control-panel={$type} --force";

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(0, $retval, "Importing {$archive} failed: " .
                         implode("\n", $result));

        $error = 'An error occurred while importing a slave domain';
        foreach ($result as $line) {
            $this->assertNotContains($error, $line, $line);
        }

        $NW = self::getMasterNodeWorx();
        $SW = $NW->getSiteWorx(self::NO_PLAN_MASTER_DOMAIN);
        // cleanup
        self::deleteSWAccountIfNecessaryStatic($SW->getMasterDomainName());
    }

    /**
     * Test.
     */
    public function testImportingCpanelBackupWithAddonHavingMasterDomDocroot()
    {
        $ip   = self::_getFirstSharedIP();
        $type = 'cpanel';
        $archive = self::ADDON_BACKUP1;

        $cmd = Ini::get(Ini::IWORX_BIN, 'import');
        $cmd .= " --archive={$archive} --ipv4={$ip} --control-panel={$type} --force";

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(0, $retval, "Importing {$archive} failed: " .
                         implode("\n", $result));

        $error = 'An error occurred while importing a slave domain';
        foreach ($result as $line) {
            $this->assertNotContains($error, $line, $line);
        }

        $NW = self::getMasterNodeWorx();
        $SW = $NW->getSiteWorx(self::ADDON_BACKUP1_MASTER_DOMAIN);

        $is_link = is_link($SW->getWebRoot(self::ADDON_BACKUP1_ADDON_DOMAIN));
        $this->assertTrue(
            $is_link,
            'Addon domain docroot is not a symlink, but it should be.'
        );

        // cleanup
        self::deleteSWAccountIfNecessaryStatic($SW->getMasterDomainName());
    }

    /**
     * Test.
     */
    public function testImportingCpanelBackupWithSslInstalled()
    {
        $ip = self::_getFirstAvailableDedicatedIP();
        if ($ip == '') {
            // if we can't find a dedi ip, try deleting the siteworx account, it might
            // be hogging it.
            $this->deleteSWAccountIfNecessary();
            $ip = self::_getFirstAvailableDedicatedIP();
        }
        $type = 'cpanel';
        $archive = self::SSL_BACKUP1;

        $cmd = Ini::get(Ini::IWORX_BIN, 'import');
        $cmd .= " --archive={$archive} --ipv4={$ip} --control-panel={$type} --force";

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(0, $retval, "Importing {$archive} failed: " .
                         implode("\n", $result));

        $search1 = 'SSL Domain is fasteddiescarwashes.com';
        $search2 = 'Loaded SSL certificate : fasteddiescarwashes.com';

        $this->assertContains($search1, $result, 'SSL Domain output missing');
        $this->assertContains($search2, $result, 'Loaded SSL cert output missing');

        $NW = self::getMasterNodeWorx();
        $SW = $NW->getSiteWorx(self::SSL_BACKUP1_MASTER_DOMAIN);

        $ssl_info = $SW->getHTTPDriver()->getSSLDriver()->getSSLInfo();
        $this->assertTrue($ssl_info['config_exists']);
        $this->assertFileExists($ssl_info[ SSL::TYPE_KEY ], 'Imported key missing');
        $this->assertFileExists($ssl_info[ SSL::TYPE_CRT ], 'Imported crt missing');
        $dom = ApacheVhostReader::getInstance(true)->getSslDomains($ip);
        $this->assertEquals(
            self::SSL_BACKUP1_MASTER_DOMAIN,
            $dom[0],
            'Expected SSL web server config not found'
        );

        // cleanup
        self::deleteSWAccountIfNecessaryStatic($SW->getMasterDomainName());
    }
}
