<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Oct 22, 2018
 */

/**
 * SiteWorx DbMapper test
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class Import_SiteWorxImportDbMapperTest extends IWorx_Framework_TestCase
{
    const NO_PREFIX_USER = 'nopuser';
    const PREFIX_USER = 'addedbyt_puser';
    const NO_PREFIX_DB = 'nopdb';
    const PREFIX_DB = 'addedbyt_pdb';
    const PASSWORD = 'pass123';

    /**
     * Test DbMapper on Backup/Import
     */
    public function testSiteworxDbMapper()
    {
        $this->_createSiteworxAccount();
        $this->_addMysqlUsers();
        $this->_addMysqlDbs();
        $this->doBackupAndImport();

        $Driver = $this->_getMysqlDriver();
        $this->assertTrue($Driver->userExistsFQ(self::NO_PREFIX_USER));
        $this->assertTrue($Driver->userExistsFQ(self::PREFIX_USER));
        $this->assertTrue($Driver->databaseExistsFQ(self::NO_PREFIX_DB));
        $this->assertTrue($Driver->databaseExistsFQ(self::PREFIX_DB));
    }


    /**
     * Create Siteworx account for testing.
     */
    private function _createSiteworxAccount()
    {
        $this->deleteSWAccountIfNecessary();
        self::addSWAccountIfNecessary();
    }

    /**
     * Add Mysql users to SW account
     *
     */
    private function _addMysqlUsers()
    {
        $Driver = $this->_getMysqlDriver();
        $Driver->addUserFQ(self::NO_PREFIX_USER, self::PASSWORD);
        $Driver->addUserFQ(self::PREFIX_USER, self::PASSWORD);
    }

    /**
     * Add Mysql dbs to SW account
     */
    private function _addMysqlDbs()
    {
        $Driver = $this->_getMysqlDriver();
        $Driver->addDatabaseFQ(self::NO_PREFIX_DB);
        $Driver->addDatabaseFQ(self::PREFIX_DB);
    }

    /**
     * Returns the MySQL Driver for the test SiteWorx account.
     */
    private function _getMysqlDriver()
    {
        $SW = new SiteWorx(self::SW_DOMAIN);
        return $SW->getMySQLDriver();
    }
}
