<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2017 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 28, 2017
 */

/**
 * Class Php_Abstract
 *
 * @package    InterWorx
 * @subpackage TEST
 */
abstract class Php_Abstract extends IWorx_Framework_TestCase
{
    /**
     * Prepare the webroot with test files.
     *
     * @param SiteWorx $SW
     * @param Domain   $Domain
     */
    protected static function _setUpWebroot(SiteWorx $SW, Domain $Domain = null)
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);

        if ($Domain === null) {
            $Domain = $SW->getMasterDomain();
        }

        IWorxFileSys::cp(
            __DIR__ . '/../../_resources/php-mode-test',
            $SW->getWebRoot($Domain->getName()),
            true
        );

        IWorxFileSys::chown(
            $SW->getUniqname(),
            $SW->getWebRoot($Domain->getName()),
            $SW->getUniqname(),
            true
        );
        IWorxFileSys::chmod(
            0711,
            $SW->getWebRoot($Domain->getName()) . '/php-mode-test'
        );
    }

    /**
     * Verify that hello world script works.
     *
     * @param Domain $Domain
     */
    protected function _verifyHelloWorld(Domain $Domain)
    {
        $response = $this->_getUrlContent($Domain, 'hello-world.php');
        $this->assertEquals('Hello, world', trim($response));
    }

    /**
     * Check the php sapi name.
     *
     * @param Domain $Domain
     * @param string $expected
     */
    protected function _verifyPhpSapi(Domain $Domain, $expected)
    {
        $actual = trim($this->_getUrlContent($Domain, 'php-sapi-name.php'));
        $this->assertEquals($expected, $actual);
    }

    /**
     * Verify that the script executes as the expected user.
     *
     * @param Domain $Domain
     * @param string $expected
     */
    protected function _verifyExpectedUser(Domain $Domain, $expected)
    {
        $actual = trim($this->_getUrlContent($Domain, 'whoami.php'));
        $this->assertEquals($expected, $actual);
    }

    /**
     * Verify that the version running is the version expected.
     *
     * @param Domain $Domain
     * @param string $expected
     */
    protected function _verifyVersion(Domain $Domain, $expected)
    {
        $actual = trim($this->_getUrlContent($Domain, 'version.php'));
        $this->assertEquals($expected, $actual);
    }

    /**
     * Gets the content served through apache.
     *
     * @param Domain $Domain
     * @param string $file
     * @return string
     */
    private function _getUrlContent(Domain $Domain, $file)
    {
        $response = IWorxHTTP::httpGet(
            $Domain->getIPv4Address(),
            $Domain->getName(),
            "/php-mode-test/{$file}",
            false
        );
        return $response;
    }
}
