<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jun 8, 2009
 */

/**
 * Tests to make sure the run as user tests are working properly.
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class RunAsUserTest extends IWorx_Framework_TestCase
{

    /**
     * Tests to make sure the script it switching users properly.
     */
    public function testWhoami()
    {
        $this->addSWAccountIfNecessary();
        $domain = self::SW_DOMAIN;

        $SiteWorx = new SiteWorx($domain);
        $uname    = $SiteWorx->getUniqname();

        $cmd = RunAsUser::buildCommand($uname, 'whoami', $params = []);
        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals($uname, implode("\n", $result));
    }

    /**
     * Tests the building a simple command works properly.
     */
    public function testSimpleBuildCommand()
    {
        $this->addSWAccountIfNecessary();
        $domain = self::SW_DOMAIN;

        $SiteWorx = new SiteWorx($domain);
        $uname    = $SiteWorx->getUniqname();

        $runasuser    = Ini::get(Ini::IWORX_BIN, 'runasuser');
        $args         = base64_encode(serialize(array()));
        $expected_cmd = "{$runasuser} $uname whoami \"$args\"";

        $cmd = RunAsUser::buildCommand($uname, 'whoami', $params = []);

        $this->assertEquals($expected_cmd, $cmd);
    }

    /**
     * Tests to make sure tailing as the user is working properly.
     */
    public function testTailAsUser()
    {
        $this->addSWAccountIfNecessary();
        $domain = self::SW_DOMAIN;

        $SiteWorx = new SiteWorx($domain);
        $uname    = $SiteWorx->getUniqname();

        $tmp = Ini::get(Ini::DIR, 'tmp');

        $testfile      = "{$tmp}/dummyfile.txt";
        $dummy_content = 'This is a test file';

        IWorxFileSys::barf($testfile, $dummy_content, $uname, $uname, 0600);

        $params = array( 'file' => $testfile );

        $cmd = RunAsUser::buildCommand($uname, 'tail', $params);
        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals($dummy_content, implode("\n", $result));

        IWorxFileSys::rm($testfile);
    }

    /**
     * Tests the building a more complex command works properly.
     */
    public function testComplexBuildCommand()
    {
        $this->addSWAccountIfNecessary();
        $domain = self::SW_DOMAIN;

        $SiteWorx = new SiteWorx($domain);
        $uname    = $SiteWorx->getUniqname();

        $tmp = Ini::get(Ini::DIR, 'tmp');

        $testfile      = "{$tmp}/dummyfile.txt";
        $dummy_content = 'This is a test file';

        IWorxFileSys::barf($testfile, $dummy_content, $uname, $uname, 0600);

        $params = array( 'file' => $testfile );
        $cmd    = RunAsUser::buildCommand($uname, 'tail', $params);

        $runasuser     = Ini::get(Ini::IWORX_BIN, 'runasuser');
        $expected_args = base64_encode(serialize($params));
        $expected_cmd  = "{$runasuser} $uname tail \"$expected_args\"";

        $this->assertEquals($expected_cmd, $cmd);
    }
}
