<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2017 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jan 24, 2017
 */

/**
 * Class Upgrade_4139Test
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class Upgrade_4139Test extends IWorx_Framework_TestCase
{
    const DIR_RESOURCE = __DIR__ . '/../../_resources/upgrade-4139';

    /**
     * Tests that a "normal" vhost will be processed.
     */
    public function testBasicVhostWillNotBeSkipped()
    {
        $file    = self::DIR_RESOURCE . '/vhost_preupgrade.com.conf';
        $Upgrade = new Upgrade_4139();
        $content = file_get_contents($file);

        $this->assertFalse($Upgrade->shouldBeSkipped($content));
    }

    /**
     * Tests that files will not be converted twice.
     */
    public function testAlreadyProcessedFileWillBeSkipped()
    {
        $file    = self::DIR_RESOURCE . '/vhost_postupgrade.com.conf';
        $Upgrade = new Upgrade_4139();
        $content = file_get_contents($file);

        $this->assertTrue($Upgrade->shouldBeSkipped($content));
    }

    /**
     * Tests that a customized vhost file should be skipped.
     */
    public function testCustomizedFileWillBeSkipped()
    {
        $file    = self::DIR_RESOURCE . '/vhost_custom.com.conf';
        $Upgrade = new Upgrade_4139();
        $content = file_get_contents($file);

        $this->assertTrue($Upgrade->shouldBeSkipped($content));
    }

    /**
     * Tests that a customized vhost file should be skipped.
     */
    public function testCustomizedFileWillBeSkipped2()
    {
        $file    = self::DIR_RESOURCE . '/vhost_custom2.com.conf';
        $Upgrade = new Upgrade_4139();
        $content = file_get_contents($file);

        $this->assertTrue($Upgrade->shouldBeSkipped($content));
    }
}
