<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Apr 09, 2014
 */

/**
 * Test class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Bug3507CrossUserCronJobViewTest extends IWorx_Framework_TestCase
{
    /**
     * Ensure that an iworx cron exists.
     */
    public static function setUpBeforeClass()
    {
        $NW    = self::getMasterNodeWorx();
        $input = array( 'interface' => 'simple',
                    'user'      => 'iworx',
                    'minute'    => '*',
                    'hour'      => '*',
                    'day'       => '*',
                    'month'     => '*',
                    'dayofweek' => '*',
                    'script'    => 'echo hi' );

        IW::FC()->routeFromPHP('Ctrl_Nodeworx_Cron', 'add', $input, $NW);
    }

    /**
     * Tear down after class.
     */
    public static function tearDownAfterClass()
    {
        $NW      = self::getMasterNodeWorx();
        $Lines   = new DataSource_Cron_LineNumbers('iworx');

        foreach ($Lines->getValues() as $linenum) {
            $cron = IWorxCron::getCronjob('iworx', $linenum);
            if ($cron['script'] === 'echo hi') {
                $input = array( 'user' => 'iworx',
                        'jobs' => $linenum );

                IW::FC()->routeFromPHP('Ctrl_Nodeworx_Cron', 'delete', $input, $NW);
            }
        }
    }

    /**
     * Need to check that iworx has crons in the first place.
     */
    public function testIWorxUserHasCrons()
    {
        $iworx_crons = IWorxCron::getUserCrontab('iworx', true);
        $msg = 'IWorx user is missing cron jobs, major bad news.';

        $this->assertGreaterThan(0, count($iworx_crons['jobs']), $msg);
    }

    /**
     * Trac: #3507.
     *
     * @depends testIWorxUserHasCrons
     */
    public function testCrossUserCronJobViewing()
    {
        $this->deleteSWAccountIfNecessary();
        $this->addSWAccount();
        $input = array( 'user' => 'iworx' );
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_Cron',
            'queryJobs',
            $input,
            $this->getSWAccount()
        );

        $this->assertReplySuccessful($Reply);
        $data = $Reply->getPayloadData();

        if (count($data)) {
            $msg = 'Oh no! Was able to query cross-user cron jobs!';
            $this->assertNotEquals('iworx', $data[0]->user, $msg);
        }
    }

    /**
     * Tests cross-user options.
     *
     * @depends testCrossUserCronJobViewing
     */
    public function testCrossUserCronOptionViewing()
    {
        $input = array( 'user' => 'iworx' );
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_Cron',
            'queryOptions',
            $input,
            $this->getSWAccount()
        );

        $this->assertReplySuccessful($Reply);
        $data = $Reply->getPayloadData();

        if (count($data)) {
            $msg = 'Oh no! Was able to query cross-user cron options!';
            $this->assertNotEquals('iworx', $data[0]->user, $msg);
        }
    }
}
