<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Apr 09, 2014
 */

/**
 * Test class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Bug3519CrossUserScheduledBackupEditTest extends IWorx_Framework_TestCase
{
    /**
     * Trac: #3519.
     */
    public function testCrossUserScheduledBackupEditing()
    {
        $this->deleteSWAccountIfNecessary();
        $this->deleteResellerSwAccountIfNecessary();
        $this->deleteResellerAccountIfNecessary();
        $this->addSWAccount();
        $this->addResellerSwAccountIfNecessary();

        $input = array( 'frequency' => Model_Scheduled_Backup::FREQUENCY_MONTHLY,
                    'day_of_month' => '1',
                    'type'      => Model_Scheduled_Backup::TYPE_STRUCTURE,
                    'location'  => Model_Scheduled_Backup::LOCATION_SITEWORX,
                    'hour'      => '1',
                    'rotate'    => '2' );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_BackupSchedule',
            'create',
            $input,
            self::getSWAccount()
        );
        $this->assertReplySuccessful($Reply);

        $SW = self::getResellerNodeWorx()->getSiteWorx(self::RSL_SW_DOMAIN);
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_BackupSchedule',
            'create',
            $input,
            $SW
        );
        $this->assertReplySuccessful($Reply);

        foreach (Model_Scheduled_Backup::find() as $schedule) {
            if ($schedule->siteworx_id === $SW->getId()) {
                $Reply = IW::FC()->routeFromPHP(
                    'Ctrl_Siteworx_BackupSchedule',
                    'edit',
                    array( 'id' => $schedule->id ),
                    self::getSWAccount()
                );
                $msg = 'Cross user scheduled backup edit attack successful!';
                $this->assertReplyNotSuccessful($Reply, '');
            } else {
                $Reply = IW::FC()->routeFromPHP(
                    'Ctrl_Siteworx_BackupSchedule',
                    'edit',
                    array( 'id' => $schedule->id ),
                    $SW
                );
                $msg = 'Cross user scheduled backup edit attack successful!';
                $this->assertReplyNotSuccessful($Reply, 'Cross user');
            }
        }
    }
}
