<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Jan 5, 2011
 */

/**
 * Data Generator Abstract.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class DataGenerator_Abstract extends IWorx_Framework_TestCase
{

    private $_first_names = array( 'Jon',
                                 'Paul',
                                 'Tim',
                                 'Dan',
                                 'Chris',
                                 'Greg',
                                 'Homer',
                                 'Marge',
                                 'Bart',
                                 'Lisa',
                                 'Maggie',
                                 'Ned',
                                 'Rod',
                                 'Todd',
                                 'Bob',
                                 'Barney',
                                 'Stu',
                                 'Stanly',
                                 'Jackson',
                                 'Ira',
                                 'Stephen',
                                 'John',
                                 'Jonathan',
                                 'Jeff',
                                 'Nina',
                                 'Ashley',
                                 'Jessica',
                                 'Roberta' );

    private $_last_names = array( 'Simpson',
                                'Powel',
                                'Homerson',
                                'Buckley',
                                'Burns',
                                'Lovejoy',
                                'Flanders',
                                'Daniel',
                                'Oehler',
                                'College',
                                'Motles',
                                'Wells',
                                'Swaney',
                                'Baggins',
                                'Jackson',
                                'Black',
                                'White',
                                'Gray',
                                'Daniels',
                                'Colins',
                                'Bush',
                                'Obama',
                                'Colbert',
                                'Stewart' );

    private $_domains = array( 'interworx.com',
                             'nexcess.net',
                             'gmail.com',
                             'googlemail.com',
                             'mail.google.com',
                             'yahoo.com',
                             'hotmail.com',
                             'live.com',
                             'earthlink.net',
                             'aol.com',
                             'turdhost.co.ca',
                             'usa.net',
                             'superlongturdemailaddress.com' );

    private $_tlds = array( 'com',
                          'net',
                          'org',
                          'co.uk',
                          'co.ca',
                          'co.au',
                          'co.cn',
                          'co',
                          'us',
                          'cx' );

    private $_random_words = array( 'binary',
                                  'cleric',
                                  'man',
                                  'woman',
                                  'boy',
                                  'girl',
                                  'drummer',
                                  'disco',
                                  'djbdns',
                                  'spanish',
                                  'lyrics',
                                  'adult',
                                  'lan',
                                  'dawgs',
                                  'poker',
                                  'star',
                                  'only',
                                  'diaper',
                                  'admin',
                                  'postmaster',
                                  'big',
                                  'small',
                                  'forward',
                                  'backward',
                                  'football',
                                  'soccer',
                                  'hockey',
                                  'stanleycup',
                                  'mud',
                                  'space',
                                  'my',
                                  'ford',
                                  'chevy',
                                  'cloud',
                                  'honda',
                                  'thingsilike',
                                  'facebook',
                                  'sucks',
                                  'interworx',
                                  'cpanel',
                                  'plesk',
                                  'google' );

    private $_last_random_firstname;
    private $_last_random_lastname;

    /**
     * Gets a random first name.
     *
     * @return string
     */
    protected function _getRandomFirstName()
    {
        $num = count($this->_first_names);
        $firstname = $this->_first_names[rand(0, $num - 1)];

        $this->_last_random_firstname = $firstname;

        return $firstname;
    }

    /**
     * Gets a random last name.
     *
     * @return string
     */
    protected function _getRandomLastName()
    {
        $num      = count($this->_last_names);
        $lastname =  $this->_last_names[rand(0, $num - 1)];

        $this->_last_random_lastname = $lastname;

        return $lastname;
    }

    /**
     * Gets a random TLD.
     *
     * @return string
     */
    protected function _getRandomTld()
    {
        $num = count($this->_tlds);
        return $this->_tlds[rand(0, $num - 1)];
    }

    /**
     * Gets a random word.
     *
     * @return string
     */
    protected function _getRandomWord()
    {
        $num = count($this->_random_words);
        return $this->_random_words[rand(0, $num - 1)];
    }

    /**
     * Gets a random username.
     *
     * @return string
     */
    protected function _getRandomUsername()
    {
        $firstname = strtolower($this->_last_random_firstname);
        $lastname  = strtolower($this->_last_random_lastname);

        $num = rand(2, 11);

        if ($num == 2) {
            //example: jdaniel
            $username = substr($firstname, 0, 1) . $lastname;
        } elseif ($num == 3) {
            //example: j_daniel
            $username = substr($firstname, 0, 1) . '_' . $lastname;
        } elseif ($num == 4) {
            //example: jon-daniel
            $username = $firstname . '-' . $lastname;
        } elseif ($num == 5) {
            //example: daniel
            $username = strtolower($this->_getRandomLastName());
        } elseif ($num == 6) {
            // example: backward
            $username = $this->_getRandomWord();
        } elseif ($num == 7) {
            //example: smalladmin
            $username = $this->_getRandomWord() .
            $this->_getRandomWord();
        } elseif ($num == 8) {
            //example: small-admin
            $username = $this->_getRandomWord() .
            '-' .
            $this->_getRandomWord();
        } elseif ($num == 9) {
            //example: irasexyira
            $username = $this->_getRandomWord() .
            strtolower($this->_getRandomFirstName()) .
            $this->_getRandomWord();
        } elseif ($num == 10) {
            //example: discostu
            $username = $this->_getRandomWord() .
            strtolower($this->_getRandomFirstName());
        } else {
            //example: binarydaniel
            $username = $this->_getRandomWord() .
            strtolower($this->_getRandomLastName());
        }//end if

        return $username;
    }

    /**
     * Gets a random dommain.
     *
     * @param string $force_gen forces the generation of a new domain name
     *                          instead of getting one from the list.
     * @return string
     */
    protected function _getRandomDomain($force_gen = false)
    {
        $num = rand(0, 21);

        if ($num % 3 || $force_gen === true) {
            $count = rand(1, 3);
            $domain = '';

            for ($i = 0; $i < $count; $i++) {
                $domain .= $this->_getRandomWord();
            }

            if (strlen($domain) < 4) {
                $domain .= $this->_getRandomWord();
            }

            return "{$domain}.{$this->_getRandomTld()}";
        } else {
            $num = count($this->_domains);
            return $this->_domains[rand(0, $num)];
        }
    }

    /**
     * Gets a random e-mail address.
     *
     * @return string
     */
    protected function _getRandomEmailAddress()
    {
        $username = $this->_getRandomUsername();
        return $username . '@' . $this->_getRandomDomain();
    }

    /**
     * Gets a random full name.
     *
     * @return string
     */
    protected function _getRandomFullName()
    {
        $firstname = $this->_getRandomFirstname();
        $lastname  = $this->_getRandomLastname();

        return "{$firstname} {$lastname}";
    }

    /**
     * Gets SiteWorx input.
     *
     * @return array
     */
    protected function _getSiteWorxInput()
    {
        $input = self::_getSWAccountInput();

        unset($input['uniqname']);

        $firstname = $this->_getRandomFirstname();
        $lastname  = $this->_getRandomLastname();

        $input['nickname']   = "{$firstname} {$lastname}";
        $input['email']      = $this->_getRandomEmailAddress();
        $input['master_domain'] = $this->_getRandomDomain($force_gen = true);

        return $input;
    }

    /**
     * Gets the SiteWorx user creation input.
     *
     * @return array
     */
    protected function _getSiteWorxUserInput()
    {
        $input['nickname']         = $this->_getRandomUsername();
        $input['email']            = $this->_getRandomEmailAddress();
        $input['password']         = 'abc123';
        $input['confirm_password'] = 'abc123';
        $input['perms']            = array( 'LOGIN' );

        return $input;
    }

    /**
     * Get random reseller siteworx input.
     *
     * @return array
     */
    protected function _getResellerSiteWorxInput()
    {
        $input = $this->_getSiteWorxInput();

        $ips = IWorx_IP::getAvailableResellerIPv4(IWorx_IP::TYPE_SHARED);

        $ip  = array_shift($ips);
        $input['master_domain_ipv4'] = $ip->getIP();

        return $input;
    }

    /**
     * Gets Reseller input.
     *
     * @return array
     */
    protected function _getResellerInput()
    {
        $input = parent::_getResellerInput();

        $firstname = $this->_getRandomFirstname();
        $lastname  = $this->_getRandomLastname();

        $input['nickname'] = "{$firstname} {$lastname}";
        $input['email']    = $this->_getRandomEmailAddress();

        return $input;
    }
}
