<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Jun 17, 2009
 */

/**
 * Tests for DataSource_FreeResellerIps.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class DataSource_FreeResellerIpsTest extends \PHPUnit\Framework\TestCase
{

    /**
     * Sample ip data for helping with the test.
     *
     * This is in the format of IPAddress:getIPMap()
     *
     * @return array
     */
    private function _getSampleIPData()
    {
        return
        array( '1.1.1.1' => array( 'ipaddr'               => '1.1.1.1',
                                 'sys_aware'            => 1,
                                 'iworx_aware'          => 1,
                                 'type'                 => IWorx_IP::STATICIP,
                                 'reseller'             => 'yes',
                                 'assigned_to_reseller' => true,
                                 'vip'                  => 'yes',
                                 'ext_ipaddr'           => '1.1.1.1',
                                 'domains'              => 0,
                                 'ssl_domain'           => false,
                                 'resellers'            => array(),
                                 'is_internal'          => false,
                                 'siteworx_count'       => 0,
                                 'device'               => 'eth0' ),

             '2.2.2.2' => array( 'ipaddr' => '2.2.2.2',
                                 'sys_aware'            => 1,
                                 'iworx_aware'          => 1,
                                 'type'                 => IWorx_IP::SHAREDIP,
                                 'reseller'             => 'yes',
                                 'assigned_to_reseller' => false,
                                 'vip'                  => 'yes',
                                 'ext_ipaddr'           => '2.2.2.2',
                                 'domains'              => 'a.com,b.com,c.com',
                                 'ssl_domain'           => false,
                                 'resellers'            => array(),
                                 'is_internal'          => false,
                                 'siteworx_count'       => 3,
                                 'device'               => 'eth0:0' ),

             '3.3.3.3' => array ( 'ipaddr' => '3.3.3.3',
                                  'sys_aware'            => 1,
                                  'iworx_aware'          => 1,
                                  'type'                 => IWorx_IP::STATICIP,
                                  'reseller'             => 'yes',
                                  'assigned_to_reseller' => false,
                                  'vip'                  => 'yes',
                                  'ext_ipaddr'           => '3.3.3.3',
                                  'domains'              => 'ssltesting.com',
                                  'ssl_domain'           => 'ssltesting.com',
                                  'resellers'            => array(),
                                  'is_internal'          => false,
                                  'siteworx_count'       => 1,
                                  'device'               => 'eth0:1',
                                  )
             );
    }

    /**
     * Test.
     */
    public function testFreeResellerIpsDSFiltersStaticAndInUse()
    {
        $ip_map   = $this->_getSampleIPData();
        $filtered = DataSource_FreeResellerIps::filterIps($ip_map);

        $this->assertArrayNotHasKey(
            '3.3.3.3',
            $filtered,
            'Used ip should not be in datasource'
        );
    }

    /**
     * Test.
     */
    public function testFreeResellerIpsDSFiltersAssignedToReseller()
    {
        $ip_map   = $this->_getSampleIPData();
        $filtered = DataSource_FreeResellerIps::filterIps($ip_map);

        $this->assertArrayNotHasKey(
            '1.1.1.1',
            $filtered,
            'Used ip should not be in datasource'
        );
    }
}
