<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Nov 24, 2020
 */

/**
 * Test class DataSource_SoftwareChannels
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class DataSource_SoftwareChannelsTest extends IWorx_Framework_TestCase
{
    private $_channels = array('release', 'release-candidate', 'beta', 'stable');

    /**
     * ensure correct channels in datasource.
     */
    public function testGetValuesIncludesAllCannels()
    {
        $SUT    = $this->_getSut();
        $values = $SUT->getValues();

        foreach ($this->_channels as $channel) {
            $this->assertContains($channel, $values);
        }

        $this->assertEquals(count($values), count($this->_channels));
    }

    /**
     * ensure default is current channel.
     */
    public function testDefaultIsCurrent()
    {
        foreach ($this->_channels as $channel) {
            $SUT = $this->_getSut();
            $this->assertEquals($channel, $SUT->getDefault($channel));
        }
    }

    /**
     * Ensure equivalent for stable is base.
     */
    public function testGetEquivalentIW7ShouldReturnBase()
    {
        $expected = 'base';
        $actual = $this->_getSut()->getEquivalentIW7('stable');

        $this->assertSame($expected, $actual);
    }

    /**
     * Ensure qquivalent for beta is beta.
     */
    public function testGetEquivalentIW7ShouldReturnBeta()
    {
        $expected = 'beta';
        $actual = $this->_getSut()->getEquivalentIW7('beta');

        $this->assertSame($expected, $actual);
    }

    /**
     * Ensure qquivalent for release is ga.
     */
    public function testGetEquivalentIW7ShouldReturnGa()
    {
        $expected = 'ga';
        $actual = $this->_getSut()->getEquivalentIW7('release');

        $this->assertSame($expected, $actual);
    }

    /**
     * Ensure equivalent for release-candidate is rc.
     */
    public function testGetEquivalentIW7ShouldReturnRc()
    {
        $expected = 'rc';
        $actual = $this->_getSut()->getEquivalentIW7('release-candidate');

        $this->assertSame($expected, $actual);
    }

    /**
     * Ensure getEquivalent throws exception if random value.
     */
    public function testGetEquivalentThrowsExceptionInvalidValue()
    {
        $this->expectException(Exception::class);
        $this->_getSut()->getEquivalentIW7('somerandomvalue');
    }

    /**
     * @return DataSource_SoftwareChannels
     */
    private function _getSut()
    {
        return new DataSource_SoftwareChannels();
    }
}
