<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 31, 2015
 */

/**
 * Class IWorx_ConfigurableTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_ConfigurableTest extends IWorx_Framework_TestCase
{
    /**
     * Delete any leftovers.
     */
    public function setup()
    {
        $sql = 'DELETE FROM config WHERE type_id = 4';
        IWorxDbUtil::db_query(IW::DB(), $sql);
    }

    /**
     * Dataprovider to test return types.
     *
     * @return array
     */
    public static function dataProviderConfirmTypes()
    {
        return [[1],
            ['2'],
            [1.2],
            ['some string'],
            [[1, 2]],
            [[1, 'string']],
            [['one' => 1, 'two' => 'two'], true],
            [(object)['one' => 1, 'two' => 'two']]];
    }

    /**
     * Test that hasConfig returns false by default.
     */
    public function testHasConfigFalse()
    {
        $Tester = new IWorx_Config_Tester(__FUNCTION__);
        $this->assertFalse($Tester->hasConfig('some_config'));
    }

    /**
     * Tests that after setting a value, hasConfig works.
     */
    public function testHasConfigTrue()
    {
        $Tester = new IWorx_Config_Tester(__FUNCTION__);
        $Tester->setConfig('some_config', '1');
        $this->assertTrue($Tester->hasConfig('some_config'));
    }

    /**
     * Tests that returned value from the database is identical to input.
     *
     * @param mixed   $input
     * @param boolean $is_assoc
     * @dataProvider dataProviderConfirmTypes
     */
    public function testGetConfigValue($input, $is_assoc = false)
    {
        $Tester = new IWorx_Config_Tester(__FUNCTION__);
        $Tester->setConfig('some_config', $input);
        $output = $Tester->getConfig('some_config', $is_assoc);

        if (IWorxUtil::isNumericArray($input)) {
            sort($input);
        }
        if (IWorxUtil::isNumericArray($output)) {
            sort($output);
        }

        $this->assertEquals($input, $output);
    }

    /**
     * Tests that changing the value works as expected.
     */
    public function testUpdatingValue()
    {
        $input  = 'Fred';
        $Tester = new IWorx_Config_Tester(__FUNCTION__);
        $Tester->setConfig('some_config', $input);

        $Tester->setConfig('some_config', 'Rita');
        $output = $Tester->getConfig('some_config');

        $this->assertEquals('Rita', $output);
    }
}

/**
 * Class IWorx_Config_Tester
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class IWorx_Config_Tester implements IWorx_Configurable
{
    use IWorx_Configurable_Trait;

    private $_key;

    /**
     * Constructor.
     *
     * @param string $key
     */
    public function __construct($key)
    {
        $this->_key = $key;
    }

    /**
     * Gets the config level.
     *
     * @return integer
     */
    public function getConfigLevel()
    {
        return 0;
    }

    /**
     * Get the fallback config.
     *
     * @return null
     */
    public function getFallbackConfigurable()
    {
        return null;
    }

    /**
     * Gets the unique identifier to associate config values with whatever.
     *
     * For a domain, the config key is the domain name
     * For SiteWorx, the SiteWorx master domain name
     * For NodeWorx, maybe the nw_id
     *
     * @return string
     */
    public function getConfigKey()
    {
        return $this->_key;
    }

    /**
     * Gets the name of the config type - ie, "nodeworx", "siteworx", "domain", etc.
     *
     * @return string
     */
    public function getConfigType()
    {
        return IWorx_Config::TYPE_TEST;
    }
}
