<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2016 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 14, 2016
 */

/**
 * Class IWorx_Filter_VariableReplacementTest
 *
 * @package    InterWorx
 * @subpackage Test
 */
class IWorx_Filter_VariableReplacementTest extends IWorx_Framework_TestCase
{
    /**
     * Test basic replacement.
     */
    public function testBasicReplacement()
    {
        $Filter   = new IWorx_Filter_VariableReplacement(['TEST' => 'RESULT']);
        $string   = 'Replacing TEST string';
        $expected = 'Replacing RESULT string';

        $this->assertEquals($expected, $Filter->filter($string));
    }

    /**
     * Test when multiple matches occur.
     */
    public function testBasicReplacementMultipleMatches()
    {
        $Filter   = new IWorx_Filter_VariableReplacement(['TEST' => 'RESULT']);
        $string   = 'Replacing TEST string TEST';
        $expected = 'Replacing RESULT string RESULT';

        $this->assertEquals($expected, $Filter->filter($string));
    }

    /**
     * Tests replacement with a prefix and suffix.
     */
    public function testReplacementWithPrefixSuffix()
    {
        $Filter = new IWorx_Filter_VariableReplacement(['TEST' => 'RESULT']);
        $Filter->setKeyPrefix('{$');
        $Filter->setKeySuffix('}');
        $string   = 'Replacing {$TEST} string';
        $expected = 'Replacing RESULT string';

        $this->assertEquals($expected, $Filter->filter($string));
    }

    /**
     * Test when more than one variable is present.
     */
    public function testWithMultipleVars()
    {
        $Filter   = new IWorx_Filter_VariableReplacement(['TEST' => 'RESULT',
                                                       'WHAT' => 'EVER' ]);
        $string   = 'Replacing TEST string WHAT';
        $expected = 'Replacing RESULT string EVER';

        $this->assertEquals($expected, $Filter->filter($string));
    }

    /**
     * Tests adding a variable after the initial instantiation.
     */
    public function testAddingVarsAfterInstantiation()
    {
        $Filter = new IWorx_Filter_VariableReplacement([]);
        $Filter->setVar('TEST', 'RESULT');
        $string   = 'Replacing TEST string';
        $expected = 'Replacing RESULT string';

        $this->assertEquals($expected, $Filter->filter($string));
    }
}
