<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2012 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Matt Parmelee <mparmelee@interworx.com>
 * @date       May 30, 2012
 */

/**
 * Test class for IPObject.
 *
 * @package    InterWorx
 * @subpackage IpTest
 */
class IWorx_IpTest extends IWorx_Framework_TestCase
{
    const NEXT_IP = '2001:db8::1';

    /**
     * Data provider.
     *
     * @return array
     */
    public static function dataProviderIPs()
    {
        return
        array(
            array( self::NEXT_IP, '128' ),
            array( '1.2.3.4', '255.255.255.0' )
            );
    }

    /**
     * Setup function.
     */
    public static function setUpBeforeClass()
    {
        foreach (self::dataProviderIPs() as $ip) {
            $DbIP = IWorx_IP::factoryFromDatabase($ip[0]);

            if ($DbIP !== false) {
                $DbIP->delete();
            }
        }
    }

    /**
     * Spot-checks the parsing of IPv4 lines.
     */
    public function testParseIfconfigIPv4Line1()
    {
        $line = 'inet addr:199.184.153.122  P-t-P:199.184.153.122 ' .
        'Bcast:199.184.153.122  Mask:255.255.255.255';

        $IP_Info = IWorx_IP::parseIfconfigIPv4($line);
        $this->assertEquals('199.184.153.122', $IP_Info->ip);
        $this->assertEquals('199.184.153.122', $IP_Info->broadcast);
        $this->assertEquals('255.255.255.255', $IP_Info->netmask);
    }

    /**
     * Spot-checks the parsing of IPv4 lines.
     */
    public function testParseIfconfigIPv4Line2()
    {
        $line = 'inet addr:127.0.0.1  Mask:255.0.0.0';

        $IP_Info = IWorx_IP::parseIfconfigIPv4($line);

        $this->assertEquals('127.0.0.1', $IP_Info->ip);
        $this->assertEquals('', $IP_Info->broadcast);
        $this->assertEquals('255.0.0.0', $IP_Info->netmask);
    }

    /**
     * Spot-checks the parsing of IPv4 lines in newer versions of ifconfig.
     */
    public function testParseAlternateIfconfigIPv4Line1()
    {
        $line = 'inet addr:199.184.153.122  ' .
        'netmask 255.255.255.255  broadcast 199.184.153.122';

        $IP_Info = IWorx_IP::parseIfconfigIPv4($line);
        $this->assertEquals('199.184.153.122', $IP_Info->ip);
        $this->assertEquals('199.184.153.122', $IP_Info->broadcast);
        $this->assertEquals('255.255.255.255', $IP_Info->netmask);
    }

    /**
     * Spot-checks the parsing of IPv4 lines in newer versions of ifconfig.
     */
    public function testParseAlternateIfconfigIPv4Line2()
    {
        $line = 'inet 127.0.0.1  netmask 255.0.0.0';

        $IP_Info = IWorx_IP::parseIfconfigIPv4($line);

        $this->assertEquals('127.0.0.1', $IP_Info->ip);
        $this->assertEquals('', $IP_Info->broadcast);
        $this->assertEquals('255.0.0.0', $IP_Info->netmask);
    }

    /**
     * Spot checks the parsing of IPv6 lines.
     */
    public function testParseIfconfigIPv6Line()
    {
        $line    = 'inet6 addr: 2001:db8::1/128 Scope:Host';
        $IP_Info = IWorx_IP::parseIfconfigIPv6($line);

        $this->assertEquals('2001:db8::1', $IP_Info->ip);
    }

    /**
     * Tests the overall output.
     */
    public function testParseIfconfigOutput()
    {
        $ifconfig_output = 'lo        Link encap:Local Loopback
          inet addr:127.0.0.1  Mask:255.0.0.0
          inet6 addr: ::1/128 Scope:Host
          UP LOOPBACK RUNNING  MTU:16436  Metric:1
          RX packets:9817862 errors:0 dropped:0 overruns:0 frame:0
          TX packets:9817862 errors:0 dropped:0 overruns:0 carrier:0
          collisions:0 txqueuelen:0
          RX bytes:18831746790 (17.5 GiB)  TX bytes:18831746790 (17.5 GiB)

venet0    Link encap:UNSPEC  HWaddr 00-00-00-00-00-00-00-00-00-00-00-00-00-00-00-00
          inet addr:127.0.0.1  P-t-P:127.0.0.1  Bcast:0.0.0.0 Mask:255.255.255.255
          UP BROADCAST POINTOPOINT RUNNING NOARP  MTU:1500  Metric:1
          RX packets:3377699644 errors:0 dropped:0 overruns:0 frame:0
          TX packets:2745647681 errors:0 dropped:27 overruns:0 carrier:0
          collisions:0 txqueuelen:0
          RX bytes:3315378383676 (3.0 TiB)  TX bytes:1213721290471 (1.1 TiB)

venet0:0  Link encap:UNSPEC  HWaddr 00-00-00-00-00-00-00-00-00-00-00-00-00-00-00-00
          inet addr:199.184.153.120  P-t-P:199.184.153.120 ' .
        'Bcast:199.184.153.120  Mask:255.255.255.255
          UP BROADCAST POINTOPOINT RUNNING NOARP  MTU:1500  Metric:1

venet0:1  Link encap:UNSPEC  HWaddr 00-00-00-00-00-00-00-00-00-00-00-00-00-00-00-00
          inet addr:199.184.153.121  P-t-P:199.184.153.121 ' .
        'Bcast:199.184.153.121  Mask:255.255.255.255
          UP BROADCAST POINTOPOINT RUNNING NOARP  MTU:1500  Metric:1

venet0:2  Link encap:UNSPEC  HWaddr 00-00-00-00-00-00-00-00-00-00-00-00-00-00-00-00
          inet addr:199.184.153.122  P-t-P:199.184.153.122 ' .
        'Bcast:199.184.153.122  Mask:255.255.255.255
          UP BROADCAST POINTOPOINT RUNNING NOARP  MTU:1500  Metric:1';

        $ips = IWorx_IP::parseIfconfigOutput($ifconfig_output);

        $this->assertTrue(isset($ips[ '199.184.153.120' ]));
        $this->assertEquals(
            '00-00-00-00-00-00-00-00-00-00-00-00-00-00-00-00',
            $ips['199.184.153.120']->mac_addr
        );

        $this->markTestSkipped();
        $this->fail('On purpose');
    }

    /**
     * Test IP object creation.
     *
     * @param string $ip
     * @param string $netmask
     * @dataProvider dataProviderIPs
     */
    public function testCreateIP($ip, $netmask)
    {
        $IP = IWorx_IP::factoryFromString($ip);
        $IP->setIP($ip);
        $IP->setNetmask($netmask);
        $IP->setDevice('eth0');
        $IP->setIsStatic(true);
        $IP->setIsVIP(false);
        $IP->setPoolId('1');
        $IP->setIsReseller(true);

        $IP->save();

        $DbIP = IWorx_IP::factoryFromDatabase($ip);

        $this->assertInstanceOf('IWorx_IP', $DbIP);
    }

    /**
     * Test IP activation.
     *
     * @param string $ip
     * @param string $netmask
     * @dataProvider dataProviderIPs
     * @depends testCreateIP
     */
    public function testActivateIP($ip, $netmask)
    {
        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);

        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);

        $IWorx_IP->activate();

        $this->assertTrue($IWorx_IP->isActive());
    }

    /**
     * Test getDedicatedSiteworxId.
     *
     * @param string $ip
     * @param string $netmask
     * @dataProvider dataProviderIPs
     */
    public function testGetDedicatedSiteworxIdInitialValue($ip, $netmask)
    {
        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $this->assertEquals('0', $IWorx_IP->getDedicatedSiteworxId());
    }

    /**
     * Test setDedicatedSiteworxId().
     *
     * @param string $ip
     * @param string $netmask
     * @dataProvider dataProviderIPs
     */
    public function testSetDedicatedSiteworxId($ip, $netmask)
    {
        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $IWorx_IP->setDedicatedSiteworxId('1');
        $IWorx_IP->save();

        $this->assertEquals('1', $IWorx_IP->getDedicatedSiteworxId());
    }

    /**
     * Test that the saved siteworx id persists.
     *
     * @param string $ip
     * @param string $netmask
     * @dataProvider dataProviderIPs
     */
    public function testDedicatedSiteworxIdPersists($ip, $netmask)
    {
        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $this->assertEquals('1', $IWorx_IP->getDedicatedSiteworxId());
    }

    /**
     * Test grantAccessToReseller().
     *
     * @param string $ip
     * @param string $netmask
     * @dataProvider dataProviderIPs
     */
    public function testGrantAccessToReseller($ip, $netmask)
    {
        self::addResellerIfNecessary();
        $NW = self::getResellerNodeWorx();

        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $IWorx_IP->grantAccessToReseller($NW->getId());

        $ips = Payload_Factory_IPs::getResellerIpData($NW->getId());

        $found_it = false;
        foreach ($ips as $ip_addr) {
            if ($ip_addr->ip === $ip) {
                $found_it = true;
            }
        }

        $this->assertTrue($found_it);
    }

    /**
     * Test grantAccessToSiteworx( $ip, $netmask ).
     *
     * @param string $ip
     * @param string $netmask
     * @dataProvider dataProviderIPs
     */
    public function testGrantAccessToSiteworx($ip, $netmask)
    {
        self::addSWAccountIfNecessary();
        $sw_id = self::getSWAccountId(self::SW_DOMAIN);

        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $IWorx_IP->grantAccessToSiteWorx($sw_id);

        $ds_class = 'DataSource_SW_IP_IPv' . $IWorx_IP->getVersion();
        $IP_DS    = new $ds_class($sw_id);

        $this->assertTrue($IP_DS->isValidValue($IWorx_IP->getIP()));
    }

    /**
     * Test IP deactivation.
     *
     * @param string $ip
     * @param string $netmask
     * @depends testCreateIP
     * @depends testActivateIP
     * @dataProvider dataProviderIPs
     */
    public function testDeactivateIP($ip, $netmask)
    {
        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $IWorx_IP->deactivate();

        $this->assertFalse($IWorx_IP->isActive());
    }

    /**
     * Test IP object deletion.
     *
     * @param string $ip
     * @param string $netmask
     * @depends testCreateIP
     * @dataProvider dataProviderIPs
     */
    public function testDeleteIP($ip, $netmask)
    {
        $IWorx_IP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $IWorx_IP->delete();

        $DbIP = IWorx_IP::factoryFromDatabase($ip);
        $this->assertInstanceOf('IWorx_IP', $IWorx_IP);
        $this->assertFalse($DbIP, 'Deleting IP failed.');
    }
}
