<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Aug 5, 2014
 */

/**
 * TmpUnwriteable Problem Detector Test.
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class IWorx_Problem_TmpUnwriteableTest extends IWorx_Framework_TestCase
{
    /**
     * Test the problem detector.
     */
    public function testProblem()
    {
        // make sure /tmp is set to 777
        $retval = IWorxFileSys::chmod(0777, '/tmp');
        $this->assertEquals(0, $retval);
        clearstatcache();
        $perms = decoct(fileperms('/tmp'));
        $this->assertEquals('40777', $perms);

        // should be no problem
        $detector = Ini::get(INI::IWORX_BIN, 'problem-detector') .
        ' --run-one --code TMP_UNWRITEABLE';

        $Cmd = new Command($detector);
        $Cmd->exec();
        $output = $Cmd->getOutput();
        $output = json_decode($output[0], true);
        $this->assertFalse($output['is_problem']);

        // make /tmp unwriteable
        $retval = IWorxFileSys::chmod(0750, '/tmp');
        $this->assertEquals(0, $retval);
        clearstatcache();
        $perms = decoct(fileperms('/tmp'));
        $this->assertEquals('40750', $perms);

        // should be problem
        $Cmd = new Command($detector);
        $Cmd->exec();
        $output = $Cmd->getOutput();
        $output = json_decode($output[0], true);
        $this->assertTrue($output['is_problem']);

        $retval = IWorxFileSys::chmod(0777, '/tmp');
        $this->assertEquals(0, $retval);
    }
}
