<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2018 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 17, 2018
 */

/**
 * Class IWorx_Template_EventsTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Template_EventsTest extends IWorx_Framework_TestCase
{
    /**
     * Prepare the test run.
     */
    public static function setUpBeforeClass()
    {
        self::deleteSWAccountIfNecessaryStatic();
    }

    /**
     * Test creation of symlink.
     */
    public function testSiteWorxSymlink()
    {
        $SW     = $this->getSWAccount();
        $Domain = $SW->getMasterDomain();
        $this->assertTrue(is_link(Domain::getTemplateSiteWorxSymlinkPath($Domain->getName())));
    }

    /**
     * Test creation of symlink.
     */
    public function testDomainSymlink()
    {
        $SW     = $this->getSWAccount();
        $Domain = $SW->getMasterDomain();
        $this->assertTrue(is_link(Domain::getTemplateDomainSymlinkPath($Domain->getName())));
    }

    /**
     * Test creation of symlink.
     */
    public function testSecondaryDomainSymlink()
    {
        $this->addSecondaryDomainIfNecessary();
        $this->assertTrue(is_link(Domain::getTemplateDomainSymlinkPath(self::SW_SECONDARY_DOMAIN)));
    }

    /**
     * Test that the symlink is cleaned up.
     */
    public function testSecondaryDomainSymlinkCleanup()
    {
        $this->addSecondaryDomainIfNecessary();
        $this->deleteSecondaryDomainIfNecessary();
        $this->assertFalse(is_link(Domain::getTemplateDomainSymlinkPath(self::SW_SECONDARY_DOMAIN)));
    }

    /**
     * Test that the symlink is cleaned up.
     */
    public function testDomainSymlinkCleanup()
    {
        $this->deleteSWAccountIfNecessary();
        $this->assertFalse(is_link(Domain::getTemplateDomainSymlinkPath(self::SW_DOMAIN)));
    }

    /**
     * Test that the symlink is cleaned up.
     */
    public function testSiteWorxSymlinkCleanup()
    {
        $this->deleteSWAccountIfNecessary();
        $this->assertFalse(is_link(Domain::getTemplateSiteWorxSymlinkPath(self::SW_DOMAIN)));
    }

    /**
     * Test creating the reseller template folder.
     */
    public function testResellerFolderCreation()
    {
        self::deleteResellerAccountIfNecessary();
        self::addResellerIfNecessary();

        $Reseller = new NodeWorx(self::RSL_EMAIL);

        $dir = \IWorx\Template::getTemplateDir();

        $this->assertFileExists("{$dir}/custom-reseller/{$Reseller->getId()}");
    }

    /**
     * Test deleting the reseller template folder.
     */
    public function testResellerFolderCleanup()
    {
        self::addResellerIfNecessary();
        $Reseller = new NodeWorx(self::RSL_EMAIL);
        $id = $Reseller->getId();

        self::deleteResellerAccountIfNecessary();

        $dir = \IWorx\Template::getTemplateDir();

        $this->assertFileNotExists("{$dir}/custom-reseller/{$id}");
    }
}
