<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2018 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 05, 2018
 */

use \IWorx\Template\FileTemplate;

/**
 * Class IWorx_Template_FileTemplateTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Template_FileTemplateTest extends IWorx_Framework_TestCase
{
    /**
     * Test simple variable replacement.
     *
     * @throws SmartyException If smarty is mad.
     */
    public function testVariableReplacement()
    {
        $Template = new FileTemplate('test/test-one');
        $Template->assign('test_var', 'This worked');

        $this->assertEquals("This worked\n", $Template->toString());
    }

    /**
     * Test a simple flow control.
     *
     * @throws SmartyException If smarty is mad.
     */
    public function testBasicLogic()
    {
        $Template = new FileTemplate('test/test-two');
        $Template->assign('true_thing', true);

        $this->assertEquals("True thing was true", $Template->toString());
    }

    /**
     * Test assigning variables pulled from a Filter.
     *
     * @throws SmartyException If smarty is mad.
     */
    public function testAssignFromFilter()
    {
        $Filter = new IWorx_Filter_VariableReplacement(['test_var' => 'Test']);

        $Template = new FileTemplate('test/test-one');
        $Template->assignFromFilter($Filter);

        $this->assertEquals("Test\n", $Template->toString());
    }

    /**
     * Test that including other templates works as expected.
     */
    public function testIncludes()
    {
        $Template = new FileTemplate('test/test-includes');

        $this->assertEquals("INCLUDED\n", $Template->toString());
    }
}
