<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Jun 22, 2011
 */

/**
 * Tests the validation of DataSources given different data possibilities.
 *
 * @package Tests
 */
class IWorx_Validate_DataSourceTest extends IWorx_Framework_TestCase
{

    /**
     * Checks DataSource validity given correct data.
     */
    public function testIsValidWithGoodData()
    {
        $V = new IWorx_Validate_DataSource(['datasource' => $this->_getDataSource()]);
        $this->assertTrue($V->isValid('1'));
    }

    /**
     * Checks DataSource validity given multiple entries of correct data.
     */
    public function testIsValidWithGoodDataMultiple()
    {
        $V = new IWorx_Validate_DataSource(['datasource' => $this->_getDataSource()]);
        $this->assertTrue($V->isValid(array( '1', '2' )));
    }

    /**
     * Checks DataSource validity given incorrect data.
     */
    public function testIsValidWithBadData()
    {
        $V = new IWorx_Validate_DataSource(['datasource' => $this->_getDataSource()]);
        $this->assertFalse($V->isValid('6'));
    }

    /**
     * Checks DataSource validity given multiple entries of incorrect data.
     */
    public function testIsValidWithBadDataMultipleAllBad()
    {
        $V = new IWorx_Validate_DataSource(['datasource' => $this->_getDataSource()]);
        $this->assertFalse($V->isValid(array( '6', '7' )));
    }

    /**
     * Checks DataSource validity given partially incorrect data.
     */
    public function testIsValidWithBadDataMultipleSomeBad()
    {
        $V = new IWorx_Validate_DataSource(['datasource' => $this->_getDataSource()]);
        $this->assertFalse($V->isValid(array( '5', '7' )));
    }

    /**
     * Checks DataSource validity given no data.
     */
    public function testIsValidWithoutData()
    {
        $V = new IWorx_Validate_DataSource(['datasource' => $this->_getDataSource()]);
        $this->assertTrue($V->isValid(array()));
    }

    /**
     * Returns data.
     */
    private function _getDataSource()
    {
        $values = array( 1, 2, 3, 4, 5 );
        return new DataSource_Array($values);
    }
}
