<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Apr 18, 2011
 */

/**
 * Tests for IWorx_Validate_Dns_PtrIpData.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Validate_Dns_PtrIpDataTest extends IWorx_Framework_TestCase
{
    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderGoodInput()
    {
        return array( array( '1.2.3.4' ),
                  array( '4.3.2.1.in-addr.arpa' ),
                  array( 'dead::beef' ),
                  array( 'f.e.e.b.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.d.a.e.d.ip6.arpa' ),
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderBadInput()
    {
        return array( array( '1.2.3.4.5' ),
                  array( '256.3.2.1.in-addr.arpa' ),
                  array( 'dead::beet' ),
                  array( 'f.e.e.b.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.d.a.e.d' ),
                  array( '4.3.2/?29.1.in-addr.arpa' ),
                  array( '4.3.2/29.1.in-addr.arpa' ),
                  array( '4.3.2-29.1.in-addr.arpa' ),
                  );
    }

    /**
     * Tests that good input passes the validator.
     *
     * @param string $input
     * @dataProvider dataProviderGoodInput
     */
    public function testGoodInputPasses($input)
    {
        $V = new IWorx_Validate_Dns_PtrIpData();

        $this->assertTrue($V->isValid($input));
    }

    /**
     * Tests that bullshit is caught by the validator.
     *
     * @param string $input
     * @dataProvider dataProviderBadInput
     */
    public function testBadInputFails($input)
    {
        $V = new IWorx_Validate_Dns_PtrIpData();

        $this->assertFalse($V->isValid($input));
    }
}
