<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Test
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Sep 19, 2013
 */

/**
 * Tests for IWorx_Validate_SpamassassinAddressTest
 *
 * @package    InterWorx
 * @subpackage Test
 */
class IWorx_Validate_SpamassassinAddressTest extends IWorx_Framework_TestCase
{
    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderGoodInput()
    {
        return array( array( 'a@aa.com' ),
                  array( 'aa@aa.com' ),
                  array( '?@a.com' ),
                  array( '*.tld' ),
                  array( '?*.tld' ),
                  array( '?*?@?domain.com' ),
                  array( 'a@a.*' ),
                  array( '*.*' ),
                  array( '*@*.*' ),
                  array( 'mparmelee@interworx.???' ),
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderBadInput()
    {
        return array( array( '?*?' ),
                  array( '@a.com' ),
                  array( '@aa.com' ),
                  array( 'aa.com' ),
                  array( '*' ),
                  array( '?*' ),
                  array( '*?' ),
                  array( '_@_' ),
                  array( '..........' ),
                  array( 'mparmel#e@noo.com' ),
                  );
    }

    /**
     * Tests good addresses.
     *
     * @param string $input
     * @dataProvider dataProviderGoodInput
     */
    public function testGoodSpamassassinAddresses($input)
    {
        $V = new IWorx_Validate_SpamassassinAddress();
        $this->assertTrue($V->isValid($input));
    }

    /**
     * Tests bad addresses.
     *
     * @param string $input
     * @dataProvider dataProviderBadInput
     */
    public function testBadSpamassassinAddresses($input)
    {
        $V = new IWorx_Validate_SpamassassinAddress();
        $this->assertFalse($V->isValid($input));
    }
}
