<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 25, 2008
 * @id  $Id$
 */

/**
 * Test suite for IWorxHTTP.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxHTTPTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Data Provider.
     *
     * @return array
     */
    public function getSomeInputData()
    {
        return array(
                 array( 'test', 'value' ),
                 array( 'action', 'do' ),
                 array( 'whatever_2', 'something else' ),
                 array( 'FOURTH', 'whatever' ),
                 );
    }

    /**
     * Sets up variables so we can test getting them back.
     *
     * @param string $index
     * @param string $value
     */
    protected function _setEnvironmentVars($index, $value)
    {
        $_GET[ $index ]     = 'GET ' . $value;
        $_POST[ $index ]    = 'POST ' . $value;
        $_SESSION[ $index ] = 'SESSION ' . $value;
        $_COOKIE[ $index ]  = 'COOKIE ' . $value;
    }

    /**
     * Tests that GETONLY returns the value from $_GET.
     *
     * @param string $index
     * @param string $value
     * @dataProvider getSomeInputData
     */
    public function testGrabHttpvarWithGetOnly($index, $value)
    {
        $this->_setEnvironmentVars($index, $value);

        $this->assertEquals(
            'GET ' . $value,
            IWorxHTTP::grabHttpvar($index, IWorxHTTP::GETONLY)
        );
    }

    /**
     * Tests that GETPOST returns the value from $_GET.
     *
     * @param string $index
     * @param string $value
     * @dataProvider getSomeInputData
     */
    public function testGrabHttpvarWithGetPost($index, $value)
    {
        $this->_setEnvironmentVars($index, $value);

        $this->assertEquals(
            'GET ' . $value,
            IWorxHTTP::grabHttpvar($index, IWorxHTTP::GETPOST)
        );
    }

    /**
     * Tests that POSTONLY returns the value from $_POST.
     *
     * @param string $index
     * @param string $value
     * @dataProvider getSomeInputData
     */
    public function testGrabHttpvarWithPostOnly($index, $value)
    {
        $this->_setEnvironmentVars($index, $value);

        $this->assertEquals(
            'POST ' . $value,
            IWorxHTTP::grabHttpvar($index, IWorxHTTP::POSTONLY)
        );
    }

    /**
     * Tests that POSTGET returns the value from $_POST.
     *
     * @param string $index
     * @param string $value
     * @dataProvider getSomeInputData
     */
    public function testGrabHttpvarWithPostGet($index, $value)
    {
        $this->_setEnvironmentVars($index, $value);

        $this->assertEquals(
            'POST ' . $value,
            IWorxHTTP::grabHttpvar($index, IWorxHTTP::POSTGET)
        );
    }

    /**
     * Tests that SESSIONONLY returns the value from $_SESSION.
     *
     * @param string $index
     * @param string $value
     * @dataProvider getSomeInputData
     */
    public function testGrabHttpvarWithSessionOnly($index, $value)
    {
        $this->_setEnvironmentVars($index, $value);

        $this->assertEquals(
            'SESSION ' . $value,
            IWorxHTTP::grabHttpvar($index, IWorxHTTP::SESSIONONLY)
        );
    }

    /**
     * Tests that COOKIEONLY returns the value from $_COOKIE.
     *
     * @param string $index
     * @param string $value
     * @dataProvider getSomeInputData
     */
    public function testGrabHttpvarWithCookieOnly($index, $value)
    {
        $this->_setEnvironmentVars($index, $value);

        $this->assertEquals(
            'COOKIE ' . $value,
            IWorxHTTP::grabHttpvar($index, IWorxHTTP::COOKIEONLY)
        );
    }

    /**
     * Tests that conversion works as expected for Nodeworx files.
     */
    public function testConvertUrlToFilePathForNodeworx()
    {
        $input    = '/nodeworx/js/file.js';
        $expected = Ini::get(Ini::IWORX_DIR, 'base') . '/html/nodeworx/js/file.js';

        $this->assertEquals($expected, IWorxHTTP::convertUrlToFilePath($input));
    }

    /**
     * Tests that conversion works as expected for Siteworx files.
     */
    public function testConvertUrlToFilePathForSiteworx()
    {
        $input    = '/siteworx/js/file.js';
        $expected = Ini::get(Ini::IWORX_DIR, 'base') . '/html/siteworx/js/file.js';

        $this->assertEquals($expected, IWorxHTTP::convertUrlToFilePath($input));
    }

    /**
     * Tests that conversion works as expected for files in the HTML directory.
     */
    public function testConvertUrlToFilePathForHtml()
    {
        $input    = '/js/file.js';
        $expected = Ini::get(Ini::IWORX_DIR, 'base') . '/html/js/file.js';

        $this->assertEquals($expected, IWorxHTTP::convertUrlToFilePath($input));
    }
}
