<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2013 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jun 27, 2013
 */

/**
 * Tests for the IworxSecUtil class.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class IWorxSecUtilTest extends IWorx_Framework_TestCase
{
    /**
     * Test this file.
     */
    public function testFilePath()
    {
        $dir = '/home/interworx/TESTS/unit';
        $file = __FILE__;
        $this->assertTrue(IWorxSecUtil::iworx_test_file_path($dir, $file));
    }

    /**
     * Tests the path of a directory.
     */
    public function testDirectoryPath()
    {
        $dir  = '/home/interworx/TESTS';
        $file = '/home/interworx/TESTS/unit';

        $this->assertTrue(IWorxSecUtil::iworx_test_file_path(
            $dir,
            $file,
            false,
            false
        ));
    }

    /**
     * Test a naughty file.
     */
    public function testNaughtyFilePath()
    {
        $dir = '/home/interworx/TESTS/';
        $file = '/etc/passwd';
        $this->assertFalse(IWorxSecUtil::iworx_test_file_path(
            $dir,
            $file,
            false,
            false
        ));
    }

    /**
     * Test a symlink'd naughty file.
     */
    public function testNaughtySymlinkFilePath()
    {
        $dir     = '/home/interworx/tmp/';
        $file    = '/etc/passwd';
        $symlink = '/home/interworx/tmp/naughty_symlink';

        exec("ln -s $file $symlink");

        $this->assertFalse(IWorxSecUtil::iworx_test_file_path(
            $dir,
            $symlink,
            false,
            false
        ));

        IWorxFileSys::rm($symlink);
    }

    /**
     * Test a hardlink'd naughty file.
     */
    public function testNaughtyHardlinkFilePath()
    {
        $dir      = '/home/interworx/tmp/';
        $file     = '/etc/passwd';
        $hardlink = '/home/interworx/tmp/naughty_hardlink';

        exec("ln $file $hardlink");

        $this->assertFalse(IWorxSecUtil::iworx_test_file_path(
            $dir,
            $hardlink,
            false,
            false
        ));

        IWorxFileSys::rm($hardlink);
    }
}
