<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Oct 6, 2009
 */

/**
 * Dummy object for testing IWorxUtil::call()
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class DummyObject
{
    /**
     * Returns the count of arguments.
     *
     * @return integer
     */
    public function returnArgCount()
    {
        return func_num_args();
    }
}

/**
 * Tests for IWorxUtil.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxUtilTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderStringify()
    {
        return array( array( 1, '1' ),
                  array( false, '0' ),
                  array( true, '1' ),
                  array( null, null ),
                  array( array( 25 ), array( '25' ) ),
                  array( 'already a string', 'already a string' )
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderCallArgCount()
    {
        // array of args     Args           Expected return
        return array( array( array(),       0 ),
                  array( range(1, 1), 1 ),
                  array( range(1, 2), 2 ),
                  array( range(1, 3), 3 ),
                  array( range(1, 4), 4 ),
                  array( range(1, 5), 5 ),
                  array( range(1, 6), 6 ),
                  array( range(1, 7), 7 ),
                  array( range(1, 8), 8 ),
                  array( range(1, 9), 9 ),
                  );
    }

    /**
     * Data Provider - data should result in exceptions.
     *
     * @return array
     */
    public static function dataProviderCallArgCountBad()
    {
        // array of args     Args
        return array( array( range(1, 13) ), // Too many arguments
                  array( range(1, 20) ), // Too many arguments
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderDocBlockTagValue()
    {
        $block = '/**
               * Test DocBlock.
               *
               * @author Tim College
               * @param string $tag
               * @return string|false
               * @dataProvider Data
               */';
        return array( array( $block, 'author', 'Tim College' ),
                  array( $block, 'dataProvider', 'Data' ),
                  array( $block, 'return', 'string|false' ),
                  array( $block, 'param', 'string $tag' ),
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderIsAssocArray()
    {
        return array( array( array(), false ),
                  array( array( 'key' => 'value' ), true ),
                  array( array( 1,2,3,4,5 ), false ),
                  array( array( 1,2,3,4, 'five' => 5 ), true ),
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderConvertToArrayOfArrays()
    {
        $obj      = (object) null;
        $obj->key = 'value';

        return array( array( 1, array( array( '1' ) ) ),
                  array( array( 1 ), array( array( '1' ) ) ),
                  array( $obj, array( array( 'key' => 'value') ) ),
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderGenerateUniqname()
    {
        return array( array( 'domain.com', 'domainco' ),
                  array( 'domain.org', 'domainor' ),
                  array( '12three.org', 'onetwoth' ),
                  array( 'seven8.edu', 'seveneig' ),
                  array( 'one.edu', 'oneedu' ),
                  array( 'two-words.com', 'twowords' ),
                  );
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderPathsToStrip()
    {
        return array( array( 'path/to/strip',
                         'path/to/strip/out/of',
                         array( 'out/of' ) ),
                  array( 'path/to/strip',
                         array( 'path/to/strip/out/of/one',
                                'path/to/strip/out/of/two', ),
                         array( 'out/of/one', 'out/of/two' ) )
                  );
    }

    /**
     * Data Provider.
     *
     * @return string
     */
    public static function dataProviderHumanReadableSize()
    {
        return array( array( 1024, '1.00 KB' ),
                  array( pow(1024, 2), '1.00 MB' ),
                  array( pow(1024, 3), '1.00 GB' ),
                  array( pow(1024, 4), '1.00 TB' ),
                  );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public static function dataProviderTimeAgo()
    {
        return [
            [ 'now -10 seconds', '##LG_X_SECONDS_AGO' ],
            [ 'now -10 minutes', '##LG_X_MINUTES_AND_Y_SECONDS_AGO' ],
            [ 'now -10 hours', '##LG_X_HOURS_AND_Y_MINUTES_AGO' ],
            [ 'now -3 days', '##LG_X_DAYS_AND_Y_HOURS_AGO' ],
            [ 'now -10 days', '##LG_X_WEEKS_AND_Y_DAYS_AGO' ],
            [ 'now -10 weeks', '##LG_X_MONTHS_AND_Y_WEEKS_AGO' ],
            [ 'now -2 years', '##LG_X_YEARS_AND_Y_MONTHS_AGO' ],
            [ 'now -12 years', '##LG_X_YEARS_AND_Y_MONTHS_AGO' ] ];
    }

    /**
     * Test the formation of "fuzzy" human readable times.
     *
     * @param string $strtotime
     * @param string $expected_lang_key
     * @dataProvider dataProviderTimeAgo
     */
    public function testTimeAgo($strtotime, $expected_lang_key)
    {
        $actual_lang_key = IWorxUtil::timeAgo(strtotime($strtotime));

        $this->assertContains($expected_lang_key, $actual_lang_key);
    }

    /**
     * Tests the behavior of stringify.
     *
     * @param mixed $in
     * @param mixed $expected
     *
     * @dataProvider dataProviderStringify
     */
    public function testStringify($in, $expected)
    {
        $this->assertEquals(IWorxUtil::stringify($in), $expected);
    }

    /**
     * Tests the call function.
     *
     * @param array   $args
     * @param integer $expected
     *
     * @dataProvider dataProviderCallArgCount
     */
    public function testCall(array $args, $expected)
    {
        $Obj = new DummyObject();

        $actual = IWorxUtil::call($Obj, 'returnArgCount', $args);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests the call function for some cases that should result in exceptions.
     *
     * @param array $args
     *
     * @dataProvider dataProviderCallArgCountBad
     */
    public function testCallExceptions(array $args)
    {
        $Obj = new DummyObject();

        $got_exception = false;

        try {
            $actual = IWorxUtil::call($Obj, 'returnArgCount', $args);
        } catch (IWorx_Exception $e) {
            $got_exception = true;
        }

        $this->assertTrue($got_exception, 'No exception was thrown');
    }

    /**
     * Tests getDocBlockTagValue.
     *
     * @param string $block
     * @param string $tag
     * @param string $expected
     *
     * @dataProvider dataProviderDocBlockTagValue
     */
    public function testGetDocBlockTagValue($block, $tag, $expected)
    {
        $actual = IWorxUtil::getDocBlockTagValue($tag, $block);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test isAssocArray.
     *
     * @param array   $array
     * @param boolean $expected
     *
     * @dataProvider dataProviderIsAssocArray
     */
    public function testIsAssocArray(array $array, $expected)
    {
        $actual = IWorxUtil::isAssocArray($array);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests convertToArrayOfArrays.
     *
     * @param mixed $in
     * @param array $expected
     *
     * @dataProvider dataProviderConvertToArrayOfArrays
     */
    public function testConvertToArrayOfArrays($in, array $expected)
    {
        $this->assertEquals($expected, IWorxUtil::convertToArrayOfArrays($in));
    }

    /**
     * Tests length of password is as expected.
     */
    public function testGeneratePassword()
    {
        $password = IWorxUtil::generate_password(16);

        $this->assertEquals(16, strlen($password));
    }

    /**
     * Tests generating uniqnames.
     *
     * @param string $in
     * @param string $expected
     *
     * @dataProvider dataProviderGenerateUniqname
     */
    public function testGenerateUniqname($in, $expected)
    {
        $this->assertEquals($expected, IWorxUtil::generate_uniqname($in, false));
    }

    /**
     * Tests that conflicting names are changed.
     */
    public function testMakeUniqNameUnique()
    {
        $test = 'root';

        $this->assertNotEquals($test, IWorxUtil::makeUniqNameUnique($test));
    }

    /**
     * Tests extracting email user.
     */
    public function testStripEmailUser()
    {
        $this->assertEquals(
            'tcollege',
            IWorxUtil::strip_email_user('tcollege@interworx.com')
        );
    }

    /**
     * Tests forcing newlines.
     */
    public function testForceNewLines()
    {
        $input  = array( 'Line 1', "Line 2\n" );
        $output = array( "Line 1\n", "Line 2\n" );

        $this->assertEquals($output, IWorxUtil::forceNewLines($input));
    }

    /**
     * Tests stripBlankLines.
     */
    public function testStripBlankLines()
    {
        $input    = array( 'not blank', '', '', 'not blank' );
        $expected = array( 'not blank', 'not blank' );

        $this->assertEquals($expected, IWorxUtil::stripBlankLines($input));
    }

    /**
     * Tests stripping paths.
     *
     * @param string       $prefix
     * @param string|array $input
     * @param array        $expected
     *
     * @dataProvider dataProviderPathsToStrip
     */
    public function testStripPathPrefix($prefix, $input, array $expected)
    {
        $this->assertEquals($expected, IWorxUtil::stripPathPrefix($prefix, $input));
    }

    /**
     * Tests human readable sizes.
     *
     * @param integer $input
     * @param string  $expected
     *
     * @dataProvider dataProviderHumanReadableSize
     */
    public function testHumanReadableSize($input, $expected)
    {
        $this->assertEquals($expected, IWorxUtil::humanReadableSize($input));
    }
}
