<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Oct 20, 2008
 */

/**
 * Tests a few things about the ini file.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IniTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Tests that the goiworx value is set.
     */
    public function testIsSet()
    {
        $this->assertTrue(Ini::is_set(Ini::IWORX, 'goiworx'));
        $this->assertFalse(Ini::is_set('Test_Section', 'Test_Value'));
    }

    /**
     * Tests behavior of is_set on a whole section.
     */
    public function testIsSetSection()
    {
        $this->assertTrue(Ini::is_set(Ini::IWORX));
        $this->assertFalse(Ini::is_set('Test_Section'));
    }

    /**
     * Verifies that the goiworx value is not empty.
     */
    public function testIsEmpty()
    {
        $this->assertFalse(Ini::is_empty(Ini::IWORX, 'goiworx'));
        $this->assertTrue(Ini::is_empty('Test_Section', 'Test_Index'));
    }

    /**
     * Tests the behavior of is_empty on a whole section.
     */
    public function testIsEmptySection()
    {
        $this->assertFalse(Ini::is_empty(Ini::IWORX));
        $this->assertTrue(Ini::is_empty('Test_Section'));
    }

    /**
     * Tests the value of goiworx.
     *
     * Dumb test - just makes sure we can read at all, really
     */
    public function testGetFromIni()
    {
        $this->assertEquals('1', Ini::get(Ini::IWORX, 'goiworx'));
    }

    /**
     * Doublechecks that get returns null if the value is not real.
     */
    public function testGetUnsetValueFromIniReturnsNull()
    {
        $this->assertNull(Ini::get(Ini::IWORX, 'unset_variable_for_testing'));
    }

    /**
     * Verifies that you get an array back if you ask for the whole section.
     */
    public function testGetArrayFromIni()
    {
        $this->assertInternalType('array', Ini::get(Ini::IWORX));
    }

    /**
     * Sets a value and checks that it's still there.
     */
    public function testSet()
    {
        // if you see this message, you need to delete it from the iworx.ini
        $this->assertFalse(
            Ini::is_set('Test_Section', 'Test_Index'),
            'Test_Index is already set - previous test run failed?'
        );
        Ini::set('Test_Section', 'Test_Index', 'Test_Value');
        Ini::write();
        $this->assertEquals('Test_Value', Ini::get('Test_Section', 'Test_Index'));

        // Destruct the ini so it regenerates
        Ini::writeAndResetInstance();
        $this->assertTrue(Ini::is_set('Test_Section', 'Test_Index'));
        $this->assertEquals('Test_Value', Ini::get('Test_Section', 'Test_Index'));

        // Remove the index
        Ini::remove('Test_Section', 'Test_Index');
        $this->assertFalse(Ini::is_set('Test_Section', 'Test_Index'));

        // Destruct the ini so it regenerates
        Ini::writeAndResetInstance();
        $this->assertFalse(Ini::is_set('Test_Section', 'Test_Index'));
        $this->assertTrue(Ini::is_set('Test_Section'));

        // Now remove the whole section
        Ini::remove('Test_Section');

        // Destruct the ini so it regenerates
        Ini::writeAndResetInstance();
        $this->assertFalse(Ini::is_set('Test_Section'));
    }
}
