<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Jun 22, 2011
 */

/**
 * Test for the setting of DataSources.
 *
 * @package Tests
 */
class Input_SetTest extends \PHPUnit\Framework\TestCase
{
    const INPUT_NAME = 'test_input';
    const INPUT_VALUE = 'initial_value';

    /**
     * Gets an input.
     *
     * @param string $initial_value
     * @return Input_Select
     */
    private function _getInput($initial_value = '')
    {
        return new Input_Set(self::INPUT_NAME, $initial_value);
    }

    /**
     * Returns a test DataSource.
     *
     * @return DataSource_Array $values
     */
    private function _getDataSource()
    {
        $values = array( self::INPUT_VALUE, '1', '2', '3' );
        return new DataSource_Array($values);
    }

    /**
     * Test.
     */
    public function testAddDataSourceDefaultFilterWithBadValue()
    {
        $DS = new DataSource_Array(array( 1, 2, 3, 4, 5 ));
        $I = $this->_getInput(1);
        $I->setDataSource($DS)
        ->setCurrentValue(6)
        ->addDataSourceDefaultFilter();

        $this->assertEquals(1, $I->getCurrentValue());
    }

    /**
     * Test.
     */
    public function testAddDataSourceDefaultFilterWithGoodValue()
    {
        $DS = new DataSource_Array(array( 1, 2, 3, 4, 5 ));
        $I = $this->_getInput(1);
        $I->setDataSource($DS)
        ->setCurrentValue(5)
        ->addDataSourceDefaultFilter();

        $this->assertEquals(5, $I->getCurrentValue());
    }

    /**
     * Test.
     */
    public function testValidateWithDataSource()
    {
        $I = $this->_getInput(self::INPUT_VALUE);
        $I->setDataSource($this->_getDataSource());
        $this->assertTrue($I->validate());
    }

    /**
     * Test.
     */
    public function testValidateWithDataSourceBadData()
    {
        $I = $this->_getInput();
        $I->setDataSource($this->_getDataSource());
        $I->setCurrentValue('somethingbogus');
        $this->assertFalse($I->validate());
    }
}
