<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Apr 26, 2010
 */

/**
 * SSL Abstract test.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
abstract class SSL_AbstractTestCase extends IWorx_Framework_TestCase
{

    /**
     * Gets the class name.
     *
     * @return string
     */
    abstract protected function _getClassName();

    /**
     * Gets an object.
     *
     * @return SSL_Abstract
     */
    protected function _getObject()
    {
        $class_name = $this->_getClassName();
        return new $class_name();
    }

    /**
     * Tests that we can validate.
     */
    public function testValidate()
    {
        $Ssl = $this->_getObject();

        $this->assertTrue($Ssl->validate());
    }

    /**
     * Tests that we can write the file.
     */
    public function testFileWrite()
    {
        $Ssl = $this->_getObject();

        $this->assertTrue($Ssl->write());
    }

    /**
     * Tests that we can get the domain.
     */
    public function testGetDomain()
    {
        $Ssl    = $this->_getObject();
        $domain = $Ssl->getDomain();

        $this->assertTrue(!empty($domain));
    }

    /**
     * Tests that we can get the issuer.
     */
    public function testGetIssuer()
    {
        $Ssl    = $this->_getObject();
        $issuer = $Ssl->getIssuer();

        $this->assertTrue(!empty($issuer));
    }

    /**
     * Tests that the parsed crt function is returning an array.
     */
    public function testGetParsedCRT()
    {
        $Ssl       = $this->_getObject();
        $crt_array = $Ssl->getParsedCRT();

        $this->assertArrayHasKey('domain', $crt_array);
        $this->assertArrayHasKey('alt_names', $crt_array);
        $this->assertArrayHasKey('expiry', $crt_array);
        $this->assertArrayHasKey('subject', $crt_array);
        $this->assertArrayHasKey('issuer', $crt_array);
    }
}
