<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Sep 15, 2009
 */

/**
 * Test for Smarty i18n functions.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class Smartyi18nTest extends IWorx_Framework_TestCase
{

    /**
     * Tests that the serialized language file is created.
     */
    public function testSerializedLanguageFileIsCreated()
    {
        $this->addSWAccountIfNecessary();

        $lngfile = $this->_createSerializedLanguageFile();

        $this->assertTrue(IWorxFileSys::fileExists($lngfile), "$lngfile not found");
    }

    /**
     * Tests that the serialized language files is created with proper ownership.
     */
    public function testSerializedLanguageFileHasProperOwnership()
    {
        $this->addSWAccountIfNecessary();

        $lngfile = $this->_createSerializedLanguageFile();
        $stat    = IWorxFileSys::stat($lngfile);
        $pwnam   = posix_getpwuid($stat['uid']);
        $pwgrp   = posix_getgrgid($stat['gid']);

        $this->assertEquals('iworx', $pwnam['name']);
        $this->assertEquals('iworx', $pwgrp['name']);
    }

    /**
     * Do somrthing to create the serialized language file.
     *
     * @return string
     */
    private function _createSerializedLanguageFile()
    {
        $lang = '/usr/local/interworx/lang/en-us';

        // if the file already exists then we want to remove it for testing
        if (IWorxFileSys::fileExists("$lang/global.lng.ser") === true) {
            IWorxFileSys::rm("$lang/global.lng.ser");
        }

        `~iworx/bin/php /home/interworx/TESTS/_resources/create-global-lng-ser.php`;

        // returns the location of the serialized language file
        return "$lang/global.lng.ser";
    }
}
