<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 02, 2020
 */

/**
 *
 */
class Upgrade_Prep_Version_6Test extends IWorx_Framework_TestCase
{
    /**
     * ensure getSteps only returns classes that extends Upgrade_Prep_Abstract
     */
    public function testGetSteps()
    {
        $SUT = new Upgrade_Prep_Version_6();
        $steps = $SUT->getSteps();

        $this->assertContainsOnlyInstancesOf('Upgrade_Prep_Abstract', $steps);
    }

    /**
     * test getUpgradePrerByKey - expects class IsOptedIn
     */
    public function testGetUpgradeStepByKeyShouldReturnClassIsOptedIn()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('is-opted-in');
        $this->assertInstanceOf(Upgrade_Prep_IsOptedIn::class, $Step);
    }

    /**
     * test getUpgradePrerByKey - expects class CustomThemes
     */
    public function testGetUpgradeStepByKeyShouldReturnClassCustomThemes()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('has-custom-themes');
        $this->assertInstanceOf(Upgrade_Prep_CustomThemes::class, $Step);
    }

    /**
     * test getUpgradePrerByKey - expects class IsCloudLinux
     */
    public function testGetUpgradeStepByKeyShouldReturnClassIsCloudLinux()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('is-cloud-linux');
        $this->assertInstanceOf(Upgrade_Prep_IsCloudLinux::class, $Step);
    }

    /**
     * test getUpgradePrerByKey - expects class IsCluster
     */
    public function testGetUpgradeStepByKeyShouldReturnClassIsCluster()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('is-cluster');
        $this->assertInstanceOf(Upgrade_Prep_IsCluster::class, $Step);
    }

    /**
     * test getUpgradePrerByKey - expects class IsHttpdSyntaxOk
     */
    public function testGetUpgradeStepByKeyShouldReturnClassIsHttpdSyntaxOk()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('httpd-syntax');
        $this->assertInstanceOf(Upgrade_Prep_IsHttpdSyntaxOk::class, $Step);
    }

    /**
     * test getUpgradePrerByKey - expects class IsLiteSpeed
     */
    public function testGetUpgradeStepByKeyShouldReturnClassIsLiteSpeed()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('is-litespeed');
        $this->assertInstanceOf(Upgrade_Prep_IsLiteSpeed::class, $Step);
    }

    /**
     * test getUpgradePrerByKey - expects class IsYumOk
     */
    public function testGetUpgradeStepByKeyShouldReturnClassIsYumOk()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('yum-upgrade');
        $this->assertInstanceOf(Upgrade_Prep_IsYumOk::class, $Step);
    }

    /**
     * test getUpgradePrerByKey - expects class IsUpToDate
     */
    public function testGetUpgradeStepByKeyShouldReturnClassIsUpToDate()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $Step = $SUT->getUpgradePrepByKey('is-up-to-date');
        $this->assertInstanceOf(Upgrade_Prep_IsUpToDate::class, $Step);
    }



    /**
     * should pass when upgrade step returns true.
     */
    public function testUpgradeStepsPassShouldReturnTrue()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $shouldPass = true;
        $steps = $this->_getMockSteps($shouldPass);

        $result = $SUT->upgradeStepsPass($steps);
        $this->assertTrue($result);
    }

    /**
     * should not pass when upgrade step returns false.
     */
    public function testUpgradeStepsPassShouldReturnFalse()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $shouldPass = false;
        $steps = $this->_getMockSteps($shouldPass);

        $result = $SUT->upgradeStepsPass($steps);
        $this->assertFalse($result);
    }

    /**
     * Get mock steps to test upgradeStepsPass
     *
     * @param boolean $shouldPass
     */
    private function _getMockSteps($shouldPass)
    {
        $Mock = $this->getMockBuilder(Upgrade_Prep_CustomThemes::class)
        ->setMethods(['_check'])
        ->getMock();

        $Mock->expects($this->any())
        ->method('_check')
        ->willReturn($shouldPass);

        return array($Mock);
    }

    /**
     * Expects exception when trying to set invalid value.
     */
    public function testSetSoftwareChannelShouldThrowExceptionIfInvalidValue()
    {
        $SUT = new Upgrade_Prep_Version_6();

        $this->expectException(Exception::class);
        $SUT->setSoftwareChannel('random string');

        $actual = $this->_getSoftwareChannelFlagValue();
        $this->assertNotEquals('random string', $actual);
    }

    /**
     * Expects env flag to be set for valid values.
     *
     * TODO: uncomment when all channels are supported.
     */
    // public function testSetSoftwareChannelShouldSetFlagIfValidValues()
    // {
    //     $validValues = array('stable', 'release', 'release-candidate', 'beta');
    //     $SUT = new Upgrade_Prep_Version_6();

    //     foreach ($validValues as $expected) {
    //         $SUT->setSoftwareChannel($expected);
    //         $actual = $this->_getSoftwareChannelFlagValue();
    //         $this->assertEquals($expected, $actual);
    //     }
    // }

    /**
     *
     */
    private function _getSoftwareChannelFlagValue()
    {
        return IW::Env()->getEnvFlag(Upgrade_Prep_Version_6::SOFTWARE_CHANNEL_FLAG);
    }
}
