<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 01, 2020
 */

/**
 * Test Suite for Upgrade_Prep_CustomThemes
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Upgrade_Prep_CustomThemesTest extends IWorx_Framework_TestCase
{
    const IGNORE_FLAG = 'upgrade-prep-7-has-custom-themes-ignore';
    /**
     * expects getName to return reference from lang file.
     */
    public function testGetNameShouldReturnLangReference()
    {
        $SUT = new Upgrade_Prep_CustomThemes();

        $expected = '##LG_NO_CUSTOM_THEMES_PRESENT##';
        $actual = $SUT->getName();

        $this->assertSame($expected, $actual);
    }

    /**
     * expects isIgnorable to return true.
     */
    public function testIsIgnorableShouldReturnTrue()
    {
        $SUT = new Upgrade_Prep_CustomThemes();
        $isIgnorable = $SUT->isIgnorable();

        $this->assertTrue($isIgnorable);
    }

    /**
     * expects ignore to add ignore flag & make isIgnored return true
     */
    public function testIgnore()
    {
        $SUT = new Upgrade_Prep_CustomThemes();

        // ensure not ignored before
        // testing ignore function
        IW::Env()->clearEnvFlag(self::IGNORE_FLAG);
        $this->assertFalse($SUT->isIgnored());

        $SUT->ignore();

        $this->assertTrue($SUT->isIgnored());
        $this->assertTrue(IW::Env()->getEnvFlag(self::IGNORE_FLAG));
    }

    /**
     * expects remove ignore to remove env flag & make isIgnored return false
     */
    public function testRemoveIgnore()
    {
        $SUT = new Upgrade_Prep_CustomThemes();

        // ensure that upgrade step is ignored
        // before test removeIgnore function
        IW::Env()->setEnvFlag(self::IGNORE_FLAG);
        $this->assertTrue($SUT->isIgnored());

        $SUT->removeIgnore();
        $this->assertFalse($SUT->isIgnored());
    }
}
