<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 01, 2020
 */

/**
 * Test Suite for Upgrade_Prep_IsHttpdSyntaxOk
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Upgrade_Prep_IsHttpdSyntaxOkTest extends IWorx_Framework_TestCase
{
    /**
     * expects getName to return reference from lang file.
     */
    public function testGetNameShouldReturnLangReference()
    {
        $SUT = new Upgrade_Prep_IsHttpdSyntaxOk();

        $expected = '##LG_HTTPD_SYNTAX_ERROR_FREE##';
        $actual = $SUT->getName();

        $this->assertSame($expected, $actual);
    }

    /**
     * Expects not ignorable.
     */
    public function testIsIgnorable()
    {
        $SUT = new Upgrade_Prep_IsHttpdSyntaxOk();
        $isIgnorable = $SUT->isIgnorable();
        $this->assertFalse($isIgnorable);
    }

    /**
     * Check should succeed when default conf/ no broken syntax.
     */
    public function testCheckShouldSucceed()
    {
        $SUT = new Upgrade_Prep_IsHttpdSyntaxOk();
        $this->assertTrue($SUT->check());
    }

    /**
     * should fail when httpd syntax is broken.
     */
    public function testCheckShouldFail()
    {
        $SUT = new Upgrade_Prep_IsHttpdSyntaxOk();

        $this->addSWAccountIfNecessary();
        $this->_addInvalidHttpdConfContent();
        $this->assertFalse($SUT->check());

        $this->deleteSWAccountIfNecessary();
    }

    /**
     *
     */
    private function _addInvalidHttpdConfContent()
    {
        $filePath = '/home/addedbyt/var/addedbytest.com/apache/broken.conf';
        $content = 'random string that should cause syntax error';
        file_put_contents($filePath, $content);
    }
}
