<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 01, 2020
 */

/**
 * Test Suite for Upgrade_Prep_IsOptedIn
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Upgrade_Prep_IsOptedInTest extends IWorx_Framework_TestCase
{
    private const OPT_OUT_FLAG = 'upgrade-prep-7-is-opted-in-opt-out';

    /**
     * expects isOptedIn to return fasle when env flag exists.
     */
    public function testIsOptedInShouldReturnFalse()
    {
        IW::Env()->setEnvFlag(self::OPT_OUT_FLAG);

        $SUT = new Upgrade_Prep_IsOptedIn();
        $this->assertFalse($SUT->isOptedIn());
    }

    /**
     * Expects isOptedIn to be true by default.
     */
    public function testIsOptedInShouldReturnTrue()
    {
        IW::Env()->clearEnvFlag(self::OPT_OUT_FLAG);

        $SUT = new Upgrade_Prep_IsOptedIn();
        $this->assertTrue($SUT->isOptedIn());
    }


    /**
     * Expect optIn to clear env flag.
     */
    public function testOptInShouldClearEnvFlag()
    {
        $this->_testOptOutFlagSet(false);

        $SUT = new Upgrade_Prep_IsOptedIn();
        $SUT->optOut();

        $this->_testOptOutFlagSet(true);
    }

    /**
     * Expect optOut to set env flag.
     */
    public function testOptOutShouldSetEnvFlag()
    {
        IW::Env()->setEnvFlag(self::OPT_OUT_FLAG);

        $this->_testOptOutFlagSet(true);

        $SUT = new Upgrade_Prep_IsOptedIn();
        $SUT->optIn();

        $this->_testOptOutFlagSet(false);
    }

    /**
     * @param boolean $expected
     */
    private function _testOptOutFlagSet($expected)
    {
        $actual = IW::Env()->getEnvFlag(self::OPT_OUT_FLAG);
        $msg = "expected opt out flag to " . ($expected ? "be set" : "NOT be set");
        $this->assertSame($expected, $actual, $msg);
    }

    /**
     * Expects not ignorable.
     */
    public function testIsIgnorable()
    {
        $SUT = new Upgrade_Prep_IsOptedIn();
        $isIgnorable = $SUT->isIgnorable();
        $this->assertFalse($isIgnorable);
    }
}
