<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    May 11, 2021
 */

/**
 * Test Suite for Upgrade_Prep_IsUpToDate
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Upgrade_Prep_IsUpToDateTest extends IWorx_Framework_TestCase
{
    /**
     * expects getName to return reference from lang file.
     */
    public function testGetNameShouldReturnLangReference()
    {
        $SUT = $this->_getSUT();

        $expected = '##LG_IS_UP_TO_DATE##';
        $actual = $SUT->getName();

        $this->assertSame($expected, $actual);
    }

    /**
     * expects isIgnorable to return false.
     */
    public function testIsIgnorableShouldReturnFalse()
    {
        $SUT = $this->_getSut();
        $isIgnorable = $SUT->isIgnorable();

        $this->assertFalse($isIgnorable);
    }

    /**
     * expects check() to return true
     */
    public function testCheckShouldReturnTrue()
    {
        //assuming default state is updated server - no missing steps
        $SUT  = $this->_getSUT();
        $actual = $SUT->check();
        $this->assertTrue($actual);
    }

    /**
     * expects check() to return false
     */
    public function testCheckShouldReturnFalse()
    {
        $step = '6653';
        $this->_simulateMissingStep($step);

        $SUT = $this->_getSUT();
        $actual = $SUT->check();
        $this->assertFalse($actual);

        $this->_addMissingStep($step);
    }

    /**
     * @param string $step
     */
    private function _simulateMissingStep($step)
    {
        $file = $this->_getVersionFilePath($step);
        IWorxFileSys::rm($file, false, true);
    }

    /**
     * @param string $step
     */
    private function _addMissingStep($step)
    {
        $file = $this->_getVersionFilePath($step);
        touch($file);
        IWorxFileSys::chown('iworx:iworx', $file);
        IWorxFileSys::chmod(0600, $file);
    }

    /**
     * @param string $step
     */
    private function _getVersionFilePath($step) : string
    {
        $base = Ini::get(Ini::IWORX_DIR, 'base');
        return "$base/etc/.upgrade/pre-installed/$step";
    }

    /**
     * Get class being tested
     */
    private function _getSUT() : Upgrade_Prep_IsUpToDate
    {
        $SUT = new Upgrade_Prep_IsUpToDate();
        return $SUT;
    }
}
