<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2013 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jun 24, 2013
 */

/**
 * Tests for the user classes.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class UserTest extends IWorx_Framework_TestCase
{
    /**
     * Deletes any leftover variables from previous tests.
     */
    public static function setUpBeforeClass()
    {
        $sql = 'DELETE FROM user_variables WHERE global_uid = "nw1"';

        IWorxDbUtil::db_query(IW::DB(), $sql);
    }

    /**
     * Test initial state.
     */
    public function testInitialState()
    {
        $User = $this->getMasterNodeWorx()->getMasterUser();

        $this->assertNull($User->getUserVariable('test'));
    }

    /**
     * Tests setting a user variable.
     */
    public function testSetUserVariable()
    {
        $User = $this->getMasterNodeWorx()->getMasterUser();

        $this->assertTrue($User->setUserVariable('test', 'test_value'));
    }

    /**
     * Tests that the set variable remains.
     *
     * @depends testSetUserVariable
     */
    public function testSetUserVariablePersists()
    {
        $User = $this->getMasterNodeWorx()->getMasterUser();

        $this->assertEquals('test_value', $User->getUserVariable('test'));
    }

    /**
     * Change a variable.
     *
     * @depends testSetUserVariablePersists
     */
    public function testChangingSetVariable()
    {
        $User = $this->getMasterNodeWorx()->getMasterUser();

        $User->setUserVariable('test', 'test_value_2');
        $this->assertEquals('test_value_2', $User->getUserVariable('test'));
    }

    /**
     * Tests clearing the user variable.
     *
     * @depends testSetUserVariable
     */
    public function testClearUserVariable()
    {
        $User = $this->getMasterNodeWorx()->getMasterUser();

        $User->clearUserVariable('test');

        $this->assertNull($User->getUserVariable('test'));
    }
}
