<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 14, 2011
 */

/**
 * Tests for HttpdConf class.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class HttpdConfTest extends IWorx_Framework_TestCase
{
    /**
     * Gets the conf file.
     *
     * @return HttpdConf
     */
    public function getConf()
    {
        $file = __DIR__ . '/vhost_test.conf';

        return new HttpdConf($file);
    }

    /**
     * Gets the first vhost from the conf.
     *
     * @return VirtualHost
     */
    public function getFirstVhost()
    {
        $Conf = $this->getConf();
        return array_shift($Conf->vhosts);
    }

    /**
     * Tests the number of vhosts parsed.
     */
    public function testParsingVhostCount()
    {
        $Conf = $this->getConf();

        $this->assertCount(2, $Conf->vhosts);
    }

    /**
     * Verifies that our getFirstVhost() function returns the expected type.
     */
    public function testParsingVhostType()
    {
        $this->assertInstanceOf('VirtualHost', $this->getFirstVhost());
    }

    /**
     * Verifies the primary IP.
     */
    public function testVhostIPParsing()
    {
        $Vhost = $this->getFirstVhost();

        $this->assertEquals($Vhost->ip, '208.69.127.151');
    }

    /**
     * Verifies the primary port.
     */
    public function testVhostPortParsing()
    {
        $Vhost = $this->getFirstVhost();

        $this->assertEquals($Vhost->port, '80');
    }

    /**
     * Verifies the count of IPs.
     */
    public function testVhostSecondIPParsing1()
    {
        $Vhost = $this->getFirstVhost();

        $this->assertCount(2, $Vhost->ips); // 2, not 3, because it's the same ip twice
    }

    /**
     * Verifies the secondary IP.
     */
    public function testVhostSecondIPParsing2()
    {
        $Vhost = $this->getFirstVhost();

        $ip = array_pop($Vhost->ips);

        $this->assertEquals('2001:db8:2040::1', $ip['ip']);
    }

    /**
     * Verifies the secondary port.
     */
    public function testVhostSecondIPParsing3()
    {
        $Vhost = $this->getFirstVhost();

        $ip = array_pop($Vhost->ips);

        $this->assertEquals(true, $ip['ports'][80]);
    }

    /**
     * Tests the output of IP string.
     */
    public function testIPString()
    {
        $Vhost = $this->getFirstVhost();

        $expected = '208.69.127.151:80 208.69.127.151:8000 [2001:db8:2040::1]:80';

        $this->assertEquals($expected, $Vhost->getIPString());
    }

    /**
     * Test changing the port.
     */
    public function testChangingPort()
    {
        $VHost = $this->getFirstVhost();

        $VHost->setPort(90);

        $this->assertEquals(true, $VHost->ips['208.69.127.151']['ports'][90]);
    }

    /**
     * Test changing the port.
     */
    public function testChangingPort2()
    {
        $VHost = $this->getFirstVhost();

        $VHost->setPort(90);

        $expected = '208.69.127.151:90 208.69.127.151:8000 [2001:db8:2040::1]:90';

        $this->assertEquals($expected, $VHost->getIPString());
    }

    /**
     * Test adding EXTERNAL_LB_IP to server env
     */
    public function testAddExternalLbIpConfig()
    {
        // At this point, there is really no way to test the node in a cluster.
        // We're just going to test if the IP was properly saved in the server config.
        $setIp = '10.11.12.13';
        $Server = new \Server();
        $Server->setConfig(\Cluster::CONFIG_EXTERNAL_LB_IP, $setIp);
        $getIp = $Server->getConfig(\Cluster::CONFIG_EXTERNAL_LB_IP);
        $this->assertEquals($setIp, $getIp);
    }
}
