<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Jan 22, 2010
 */

/**
 * Tests for the ShellAccount driver.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class ShellAccountTest extends IWorx_Framework_TestCase
{

    /**
     * Setup.
     */
    public function setUp()
    {
        // setup jail shelling if necessary
        if (JailShell::isInstalled() && !JailShell::isInitialized()) {
            JailShell::initJailEnv();
        }
        parent::setUp();
    }

    /**
     * Test.
     */
    public function testGetShells()
    {
        $shells = ShellAccount::getShells();

        $this->assertTrue(is_array($shells));
        $this->assertFalse(empty($shells));
    }

    /**
     * Test.
     */
    public function testUserExists()
    {
        $SW    = $this->getSWAccount();
        $uname = $SW->getUniqname();

        $this->assertTrue(ShellAccount::userExists($uname));
    }

    /**
     * Test.
     */
    public function testChangeShellWorks()
    {
        $SW    = $this->getSWAccount();
        $uname = $SW->getUniqname();

        $ok = ShellAccount::changeShell($uname, '/bin/bash');

        $this->assertTrue($ok);
        $this->assertEquals('/bin/bash', ShellAccount::getShell($uname));
    }

    /**
     * Test.
     */
    public function testChangePasswdWorks()
    {
        $SW        = $this->getSWAccount();
        $test_pass = 'abcdefg';

        $ok = ShellAccount::changePasswd($SW->getUniqname(), $test_pass);

        $this->assertTrue($ok);
    }

    /**
     * Test.
     */
    public function testChangePasswordCanLogin()
    {
        $SW        = $this->getSWAccount();
        $test_pass = 'abcdefgh';

        ShellAccount::changeShell($SW->getUniqname(), '/bin/bash');
        ShellAccount::changePasswd($SW->getUniqname(), $test_pass);

        sleep(1);

        $SSH = new IWorxSSH(
            '127.0.0.1',
            $SW->getUniqname(),
            $test_pass,
            SSHD::getPortNum()
        );

        $this->assertEquals(
            'success',
            $SSH->testConnection(),
            "SSH failed\n" .
                         "ip: 127.0.0.1\n" .
                         "user: {$SW->getUniqname()}\n" .
                         "pass: {$test_pass}\n" .
            'port: ' . SSHD::getPortNum()
        );
    }

    /**
     * Test.
     */
    public function testChangePasswdWorksWithFunnyChars()
    {
        $SW        = $this->getSWAccount();
        $test_pass = '~#!£1( abcd';

        ShellAccount::changeShell($SW->getUniqname(), '/bin/bash');
        ShellAccount::changePasswd($SW->getUniqname(), $test_pass);

        $SSH = new IWorxSSH(
            '127.0.0.1',
            $SW->getUniqname(),
            $test_pass,
            SSHD::getPortNum()
        );

        $this->assertEquals('success', $SSH->testConnection());
    }
}
