<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Apr 09, 2014
 */

/**
 * Test class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Bug3504CrossUserSecondaryDomainSymlinkTest extends IWorx_Framework_TestCase
{
    /**
     * Trac: #3504.
     */
    public function testCrossUserSecondaryDomainSymlinkAttack()
    {
        $target = '/etc/bug-3504-target';
        IWorxFileSys::rm($target, true, true);

        // same default perms/owner as /etc/shadow
        file_put_contents($target, 'top secret info here');
        chmod($target, 000);
        chown($target, 'root');
        chgrp($target, 'root');

        $this->deleteSWAccountIfNecessary();
        $this->addSWAccount();

        $evil_script = '/home/interworx/TESTS/_resources/' .
        'bug_3504_race_condition_exploiter.sh';

        IWorxFileSys::cp(
            $evil_script,
            $this->getSWAccount()->getPackageRoot() . '/EEEEVIL.sh'
        );

        $evil_script = $this->getSWAccount()->getPackageRoot() . '/EEEEVIL.sh';

        IWorxFileSys::chown(self::SW_UNIQNAME, $evil_script);
        IWorxFileSys::chmod(0700, $evil_script);

        $NaughtyCmd = new Command();
        $NaughtyCmd->stderrToDevNull()
        ->stdoutToDevNull()
        ->launchInBackground()
        ->setUser(self::SW_UNIQNAME)
        ->setCmd($evil_script)
        ->exec();

        $input = array( 'domain' => 'bug3504.com' );
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_DomainsSlave',
            'add',
            $input,
            $this->getSWAccount()
        );

        exec('killall -9 EEEEVIL.sh 2>/dev/null');

        $this->assertReplySuccessful($Reply);

        $perms = $this->getFilePermsAsString(fileperms($target));
        $owner = posix_getpwuid(fileowner($target));
        $owner = $owner['name'];
        $group = posix_getgrgid(filegroup($target));
        $group = $group['name'];

        $msg = 'Secondary domain addition chowned the hardlink in /etc';
        $this->assertEquals(
            'root:root|----------',
            "{$owner}:{$group}|{$perms}",
            $msg
        );
    }
}
